/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


/*

frmFileManager
Part of the Specrem-project.

File manager.
Copyright (C) 2002  Antti Kirjavainen

*/


//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FileManager.h"
#include "Public.cpp"
#include "..\Shared\strHandle.cpp"
#include "..\Shared\Commands.h"
#include "..\Shared\GetFileInfo.cpp"
#include "..\Shared\DriveHandle.cpp"
#include "Main.h"
#include "Execute.h"
#include "Properties.h"
#include "Work.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmFileManager *frmFileManager;
void UncheckMenu();

AnsiString DragSource;

double CountSize(TListView * lviObject);
int CountAttributes(const AnsiString sAttrib, TListView * lviObject);

//---------------------------------------------------------------------------
__fastcall TfrmFileManager::TfrmFileManager(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmFileManager::mnuCloseClick(TObject *Sender)
{
this->Close();        
}
//---------------------------------------------------------------------------
void __fastcall TfrmFileManager::FormResize(TObject *Sender)
{
grpRemote->Width = this->Width - 10;
grpLocal->Width = grpRemote->Width;
grpRemote->Height = (this->Height / 2) - 30;
grpLocal->Height = grpRemote->Height;
grpLocal->Top = grpRemote->Top + grpRemote->Height + 5;

lviRemoteDrive->Height = grpRemote->Height - 40;
lviLocalDrive->Height = lviRemoteDrive->Height;
cmdRemoteFileUp->Top = lviRemoteDrive->Top + lviRemoteDrive->Height + 2;
cmdLocalFileUp->Top = cmdRemoteFileUp->Top;

lviRemoteFiles->Height = (lviRemoteDrive->Height - txtRemotePath->Height) + 22;
lviRemoteFiles->Width = ((grpRemote->Width - lviRemoteDrive->Width)) - 12;
txtRemotePath->Width = lviRemoteFiles->Width;
txtRemotePath->Top = lviRemoteFiles->Top + lviRemoteFiles->Height;

lviLocalFiles->Height = lviRemoteFiles->Height;
lviLocalFiles->Width = lviRemoteFiles->Width;
txtLocalPath->Width = lviLocalFiles->Width;
txtLocalPath->Top = lviLocalFiles->Top + lviLocalFiles->Height;

}
//---------------------------------------------------------------------------
void __fastcall TfrmFileManager::UncheckMenu()
{
lviLocalFiles->ViewStyle = lviRemoteFiles->ViewStyle;
mnuLargeIcons->Checked = false;
mnuSmallIcons->Checked = false;
mnuList->Checked = false;
mnuReport->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmFileManager::mnuLargeIconsClick(TObject *Sender)
{
lviRemoteFiles->ViewStyle = vsIcon;
UncheckMenu();
mnuLargeIcons->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuSmallIconsClick(TObject *Sender)
{
lviRemoteFiles->ViewStyle = vsSmallIcon;
UncheckMenu();
mnuSmallIcons->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuListClick(TObject *Sender)
{
lviRemoteFiles->ViewStyle = vsList;
UncheckMenu();
mnuList->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuReportClick(TObject *Sender)
{
lviRemoteFiles->ViewStyle = vsReport;
UncheckMenu();
mnuReport->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuSelectAllClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) lviRemoteFiles->SelectAll();
if (lviLocalFiles->Focused()) lviLocalFiles->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::FormCreate(TObject *Sender)
{
lviLocalDrive->Clear();
frmMain->SplitToListView(lviLocalDrive, GetDrives());
frmMain->SwitchIcon(lviLocalDrive, 1, 0, "F");
frmMain->SwitchIcon(lviLocalDrive, 1, 1, "R");
frmMain->SendCommand(COMMAND_FILE_DRIVELIST, "", this->Tag);

txtRemotePath->Text = "C:\\";
txtLocalPath->Text = "C:\\";

mnuRefresh->Click();

this->Width = StrToIntDef(frmMain->ReadFromINI("Window", "FMWidth", this->Width), this->Width);
this->Height = StrToIntDef(frmMain->ReadFromINI("Window", "FMHeight", this->Height), this->Height);
this->WindowState = StrToIntDef(frmMain->ReadFromINI("Window", "FMState", this->WindowState), this->WindowState);
if (frmMain->ReadFromINIBool("Window", "FMViewLarge", false)) mnuLargeIcons->Click();
if (frmMain->ReadFromINIBool("Window", "FMViewSmall", false)) mnuSmallIcons->Click();
if (frmMain->ReadFromINIBool("Window", "FMViewDetails", false)) mnuReport->Click();
if (frmMain->ReadFromINIBool("Window", "FMViewList", false)) mnuList->Click();




if (!FileExists(ExtractFilePath(Application->ExeName) + "Language.INI")) return;

// Read from language file:
mnuFile->Caption = ReadFromLanguage("General", "mnuFile", mnuFile->Caption);
mnuClose->Caption = ReadFromLanguage("General", "mnuClose", mnuClose->Caption);
mnuSelectAll->Caption = ReadFromLanguage("General", "mnuSelectAll", mnuSelectAll->Caption);

mnuView->Caption = ReadFromLanguage("ServerMain", "mnuView", mnuView->Caption);
mnuReport->Caption = ReadFromLanguage("General", "mnuDetailIcons", mnuReport->Caption);
mnuLargeIcons->Caption = ReadFromLanguage("General", "mnuLargeIcons", mnuLargeIcons->Caption);
mnuSmallIcons->Caption = ReadFromLanguage("General", "mnuSmallIcons", mnuSmallIcons->Caption);
mnuList->Caption = ReadFromLanguage("General", "mnuListIcons", mnuList->Caption);

mnuExecute->Caption = ReadFromLanguage("ClientMain", "mnuExecute", mnuExecute->Caption);
mnuPrint->Caption = ReadFromLanguage("ClientMain", "mnuPrint", mnuPrint->Caption);
mnuProperties->Caption = ReadFromLanguage("General", "mnuProperties", mnuProperties->Caption);
mnuCreate->Caption = ReadFromLanguage("ClientMain", "mnuCreate", mnuCreate->Caption);
mnuCut->Caption = ReadFromLanguage("General", "mnuCut", mnuCut->Caption);
mnuCopy->Caption = ReadFromLanguage("ClientMain", "mnuCopyFile", mnuCopy->Caption);
mnuRename->Caption = ReadFromLanguage("ClientMain", "mnuRename", mnuRename->Caption);
mnuDelete->Caption = ReadFromLanguage("ClientMain", "mnuDeleteFile", mnuDelete->Caption);
mnuPaste->Caption = ReadFromLanguage("General", "mnuPaste", mnuPaste->Caption);
mnuDownload->Caption = ReadFromLanguage("ClientMain", "mnuDownload", mnuDownload->Caption);
mnuUpload->Caption = ReadFromLanguage("ClientMain", "mnuUpload", mnuUpload->Caption);
mnuShare->Caption = ReadFromLanguage("ClientMain", "mnuShare", mnuShare->Caption);
mnuRemoveShare->Caption = ReadFromLanguage("ClientMain", "mnuRemoveShare", mnuRemoveShare->Caption);

mnuLocalFileExecute->Caption = mnuExecute->Caption;
mnuLocalFilePrint->Caption = mnuPrint->Caption;
mnuLocalFolderCreate->Caption = mnuCreate->Caption;
mnuLocalFileCut->Caption = mnuCut->Caption;
mnuLocalFileCopy->Caption = mnuCopy->Caption;
mnuLocalFileRename->Caption = mnuRename->Caption;
mnuLocalFileDelete->Caption = mnuDelete->Caption;
mnuLocalFilePaste->Caption = mnuPaste->Caption;
mnuLocalFileUpload->Caption = mnuUpload->Caption;
mnuLocalFolderShare->Caption = mnuShare->Caption;
mnuLocalFolderRemoveShare->Caption = mnuRemoveShare->Caption;
mnuLocalFileProperties->Caption = mnuProperties->Caption;

mnuRemoteFileExecute->Caption = mnuExecute->Caption;
mnuRemoteFilePrint->Caption = mnuPrint->Caption;
mnuRemoteFolderCreate->Caption = mnuCreate->Caption;
mnuRemoteFileCut->Caption = mnuCut->Caption;
mnuRemoteFileCopy->Caption = mnuCopy->Caption;
mnuRemoteFileRename->Caption = mnuRename->Caption;
mnuRemoteFileDelete->Caption = mnuDelete->Caption;
mnuRemoteFilePaste->Caption = mnuPaste->Caption;
mnuRemoteFileDownload->Caption = mnuDownload->Caption;
mnuRemoteFolderShare->Caption = mnuShare->Caption;
mnuRemoteFolderRemoveShare->Caption = mnuRemoveShare->Caption;
mnuRemoteFileProperties->Caption = mnuProperties->Caption;

mnuLocalFileSingleCreate->Caption = mnuCreate->Caption;
mnuLocalFileSinglePaste->Caption = mnuPaste->Caption;

mnuRemoteFileSingleCreate->Caption = mnuCreate->Caption;
mnuRemoteFileSinglePaste->Caption = mnuPaste->Caption;

mnuRemoteDriveOpenCDROM->Caption = ReadFromLanguage("ClientMain", "mnuOpenCD", mnuRemoteDriveOpenCDROM->Caption);
mnuRemoteDriveCloseCDROM->Caption = ReadFromLanguage("ClientMain", "mnuCloseCD", mnuRemoteDriveCloseCDROM->Caption);
mnuRemoteDriveProperties->Caption = mnuProperties->Caption;
mnuLocalDriveOpenCDROM->Caption = mnuRemoteDriveOpenCDROM->Caption;
mnuLocalDriveCloseCDROM->Caption = mnuRemoteDriveCloseCDROM->Caption;
mnuLocalDriveProperties->Caption = mnuProperties->Caption;

grpRemote->Caption = ReadFromLanguage("General", "Remote", grpRemote->Caption);
grpLocal->Caption = ReadFromLanguage("General", "Local", grpLocal->Caption);

lviRemoteFiles->Column[0]->Caption = ReadFromLanguage("File", "Name", "Name");
lviRemoteFiles->Column[1]->Caption = ReadFromLanguage("File", "Size", "Size");
lviRemoteFiles->Column[2]->Caption = ReadFromLanguage("File", "Modified", "Modified");
lviRemoteFiles->Column[3]->Caption = ReadFromLanguage("Properties", "lblAttributes", "Attributes");
lviRemoteFiles->Column[4]->Caption = ReadFromLanguage("File", "Created", "Created");
lviRemoteFiles->Column[5]->Caption = ReadFromLanguage("File", "Accessed", "Accessed");

lviLocalFiles->Column[0]->Caption = lviRemoteFiles->Column[0]->Caption;
lviLocalFiles->Column[1]->Caption = lviRemoteFiles->Column[1]->Caption;
lviLocalFiles->Column[2]->Caption = lviRemoteFiles->Column[2]->Caption;
lviLocalFiles->Column[3]->Caption = lviRemoteFiles->Column[3]->Caption;
lviLocalFiles->Column[4]->Caption = lviRemoteFiles->Column[4]->Caption;
lviLocalFiles->Column[5]->Caption = lviRemoteFiles->Column[5]->Caption;
}
//---------------------------------------------------------------------------

double CountSize(TListView * lviObject)
{
unsigned short int i;
double SizeCount = 0;


for (i = 0; i < lviObject->Items->Count; ++i)
        {
        if (lviObject->Items->Item[i]->Selected) {
                                                 SizeCount += StrToFloatDef(lviObject->Items->Item[i]->SubItems->Strings[0], 0);
                                                 }
        }

return SizeCount;
}

int CountAttributes(const AnsiString sAttrib, TListView * lviObject)
{
unsigned short int i;
unsigned short int sCount = 0;


for (i = 0; i < lviObject->Items->Count; ++i)
        {
        if (lviObject->Items->Item[i]->Selected) {
                                                 if (lviObject->Items->Item[i]->SubItems->Strings[2].AnsiPos(sAttrib) > 0) sCount++;
                                                 }
        }

return sCount;
}

void __fastcall TfrmFileManager::mnuRemoteFilePropertiesClick(
      TObject *Sender)
{
//frmMain->SendCommand(COMMAND_FILE_PROPERTIES, AddSlash(txtRemotePath->Text) + lviRemoteFiles->Selected->Caption, this->Tag);

if (lviRemoteFiles->SelCount <= 0) return;

AnsiString sType;
unsigned short int FileCount;
unsigned short int FolderCount;
unsigned short int ArchiveCount;
unsigned short int ReadOnlyCount;
unsigned short int HiddenCount;
unsigned short int SystemCount;
TfrmProperties * frmPropertiesInst = NULL;


ArchiveCount = CountAttributes("A", lviRemoteFiles);
ReadOnlyCount = CountAttributes("R", lviRemoteFiles);
HiddenCount = CountAttributes("H", lviRemoteFiles);
SystemCount = CountAttributes("S", lviRemoteFiles);


FolderCount = CountAttributes("D", lviRemoteFiles);
FileCount = lviRemoteFiles->SelCount - FolderCount;

frmPropertiesInst = new TfrmProperties(Application);
if (frmPropertiesInst == NULL) return; // Memory allocation error


frmPropertiesInst->txtFilename->Clear();
frmPropertiesInst->Tag = lviRemoteFiles->SelCount;
frmPropertiesInst->txtLocation->Text = txtRemotePath->Text;
frmPropertiesInst->Tag = this->Tag;
frmPropertiesInst->txtFiles->Clear();

if (lviRemoteFiles->SelCount == 1) {
                                   imgLarge->GetBitmap(lviRemoteFiles->Selected->ImageIndex, frmPropertiesInst->imgIcon->Picture->Bitmap);
                                   frmPropertiesInst->txtFilename->Enabled = true;
                                   frmPropertiesInst->txtFilename->Text = lviRemoteFiles->Selected->Caption;
                                   sType = UpperCase(ExtractFileExt(lviRemoteFiles->Selected->Caption));
                                   if (FolderCount > 0) { frmPropertiesInst->txtType->Text = ReadFromLanguage("File", "Folder", "Folder"); } else { frmPropertiesInst->txtType->Text = sType.SubString(2, sType.Length() - 1); }
                                   frmPropertiesInst->txtDescription->Text = ReadFromLanguage("General", "Remote", "Remote");
                                   frmPropertiesInst->txtSize->Text = TotalSize(StrToFloatDef(lviRemoteFiles->Selected->SubItems->Strings[0], 0));
                                   frmPropertiesInst->txtCreated->Text = lviRemoteFiles->Selected->SubItems->Strings[3];
                                   frmPropertiesInst->txtModified->Text = lviRemoteFiles->Selected->SubItems->Strings[1];
                                   frmPropertiesInst->txtAccessed->Text = lviRemoteFiles->Selected->SubItems->Strings[4];
                                   }

if (lviRemoteFiles->SelCount > 1)  {
                                   if (FileCount > FolderCount) { imgLarge->GetBitmap(3, frmPropertiesInst->imgIcon->Picture->Bitmap); } else { imgLarge->GetBitmap(2, frmPropertiesInst->imgIcon->Picture->Bitmap); }
                                   frmPropertiesInst->txtFilename->Enabled = false;
                                   if (FileCount > 0) frmPropertiesInst->txtFilename->Text = IntToStr(FileCount) + " " + ReadFromLanguage("File", "files", "File(s)");
                                   if (FolderCount > 0) {
                                                        if (FileCount > 0) frmPropertiesInst->txtFilename->Text = frmPropertiesInst->txtFilename->Text + ", ";
                                                        frmPropertiesInst->txtFilename->Text = frmPropertiesInst->txtFilename->Text + IntToStr(FolderCount) + " " + ReadFromLanguage("File", "folders", "Folder(s)");
                                                        }
                                   frmPropertiesInst->txtType->Text = ReadFromLanguage("File", "Multiple", "Multiple files");
                                   frmPropertiesInst->txtDescription->Text = ReadFromLanguage("General", "Remote", "Remote");
                                   frmPropertiesInst->txtSize->Text = TotalSize(CountSize(lviRemoteFiles));
                                   frmPropertiesInst->txtFiles->Text = ListSelected(lviRemoteFiles, txtRemotePath->Text);
                                   frmPropertiesInst->txtCreated->Clear();
                                   frmPropertiesInst->txtModified->Clear();
                                   frmPropertiesInst->txtAccessed->Clear();
                                   frmPropertiesInst->lblCreated->Caption = "";
                                   frmPropertiesInst->lblModified->Caption = "";
                                   frmPropertiesInst->lblAccessed->Caption = "";
                                   }


if (ArchiveCount == lviRemoteFiles->SelCount) { frmPropertiesInst->chkArchive->State = cbChecked; } else { frmPropertiesInst->chkArchive->State = cbGrayed; }
if (ArchiveCount <= 0) frmPropertiesInst->chkArchive->State = cbUnchecked;
if (ReadOnlyCount == lviRemoteFiles->SelCount) { frmPropertiesInst->chkReadOnly->State = cbChecked; } else { frmPropertiesInst->chkReadOnly->State = cbGrayed; }
if (ReadOnlyCount <= 0) frmPropertiesInst->chkReadOnly->State = cbUnchecked;
if (HiddenCount == lviRemoteFiles->SelCount) { frmPropertiesInst->chkHidden->State = cbChecked; } else { frmPropertiesInst->chkHidden->State = cbGrayed; }
if (HiddenCount <= 0) frmPropertiesInst->chkHidden->State = cbUnchecked;
if (SystemCount == lviRemoteFiles->SelCount) { frmPropertiesInst->chkSystem->State = cbChecked; } else { frmPropertiesInst->chkSystem->State = cbGrayed; }
if (SystemCount <= 0) frmPropertiesInst->chkSystem->State = cbUnchecked;

frmPropertiesInst->chkOriginalArchive->State = frmPropertiesInst->chkArchive->State;
frmPropertiesInst->chkOriginalReadOnly->State = frmPropertiesInst->chkReadOnly->State;
frmPropertiesInst->chkOriginalHidden->State = frmPropertiesInst->chkHidden->State;
frmPropertiesInst->chkOriginalSystem->State = frmPropertiesInst->chkSystem->State;
frmPropertiesInst->txtOriginalFilename->Text = frmPropertiesInst->txtFilename->Text;

frmPropertiesInst->Caption = frmPropertiesInst->txtFilename->Text + " " + DelHK(ReadFromLanguage("General", "Properties", "Properties")); 

frmPropertiesInst->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteDrivePropertiesClick(
      TObject *Sender)
{
//frmMain->SendCommand(COMMAND_FILE_PROPERTIES, lviRemoteDrive->Selected->Caption, this->Tag);

}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteFilesEdited(TObject *Sender,
      TListItem *Item, AnsiString &S)
{
frmMain->SendCommand(COMMAND_FILE_RENAME, AddSlash(txtRemotePath->Text) + Item->Caption + SEPARATOR_SPLIT + AddSlash(txtRemotePath->Text) + S, this->Tag);
mnuRefresh->Click();
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::lviLocalFileEdited(TObject *Sender,
      TListItem *Item, AnsiString &S)
{
RenameFile(Item->Caption, S);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesEdited(TObject *Sender,
      TListItem *Item, AnsiString &S)
{
RenameFile(AddSlash(txtLocalPath->Text) + Item->Caption, AddSlash(txtLocalPath->Text) + S);
mnuRefresh->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRefreshClick(TObject *Sender)
{
mnuRefresh->Enabled = false;
mnuRefresh->Caption = ReadFromLanguage("General", "Refreshing", "Refreshing...");
this->Cursor = crAppStart;
lviRemoteFiles->Clear();
GetLocalList(txtLocalPath->Text);
frmMain->SendCommand(COMMAND_FOLDER_CHANGEDIR, txtRemotePath->Text, this->Tag);
frmMain->SendCommand(COMMAND_FILE_LIST_WIDE, "", this->Tag);
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::lviRemoteFilesDblClick(TObject *Sender)
{
AnsiString curItem = "";

if (!IsFolder(lviRemoteFiles, lviRemoteFiles->Selected->Index)) {
                                                                mnuRemoteFileExecute->Click();
                                                                return;
                                                                }

this->Cursor = crAppStart;

curItem = AddSlash(txtRemotePath->Text) + lviRemoteFiles->Selected->Caption;
lviRemoteFiles->Clear();
frmMain->SendCommand(COMMAND_FOLDER_CHANGEDIR, curItem, this->Tag);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteDriveDblClick(TObject *Sender)
{
lviRemoteFiles->Clear();
frmMain->SendCommand(COMMAND_FOLDER_CHANGEDIR, lviRemoteDrive->Selected->Caption + "\\", this->Tag);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFileRenameClick(TObject *Sender)
{
lviRemoteFiles->Selected->EditCaption();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileRenameClick(TObject *Sender)
{
lviLocalFiles->Selected->EditCaption();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFolderCreateClick(
      TObject *Sender)
{
frmMain->SendCommand(COMMAND_FOLDER_CREATE, AddSlash(txtRemotePath->Text) + "New Folder", this->Tag);
lviRemoteFiles->AddItem("New folder", lviRemoteFiles);
lviRemoteFiles->Items->Item[lviRemoteFiles->Items->Count - 1]->EditCaption();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFolderCreateClick(TObject *Sender)
{
CreateDir(AddSlash(txtLocalPath->Text) + "New Folder");
lviLocalFiles->AddItem("New folder", lviLocalFiles);
lviLocalFiles->Items->Item[lviLocalFiles->Items->Count - 1]->EditCaption();
}
//---------------------------------------------------------------------------

bool __fastcall TfrmFileManager::IsFolder(TListView * lviObject, const int lviIndex)
{
if (lviObject->Items->Item[lviIndex]->SubItems->Strings[2].AnsiPos("D") > 0) { return true; } else { return false; }
}

void __fastcall TfrmFileManager::SendFileCommand(const AnsiString wCommand, const AnsiString sPath, TListView * lviObject, const bool forDir)
{
unsigned short int w;

for (w = 0; w < lviObject->Items->Count; ++w)
        {
        if (lviObject->Items->Item[w]->Selected) {
                                                 if (forDir) {
                                                             if (IsFolder(lviObject, w)) {
                                                                                         //ShowMessage(AddSlash(sPath) + lviObject->Items->Item[w]->Caption);
                                                                                         frmMain->SendCommand(wCommand, AddSlash(sPath) + lviObject->Items->Item[w]->Caption, this->Tag);
                                                                                         }
                                                             } else {
                                                             if (!IsFolder(lviObject, w)) {
                                                                                          //ShowMessage(AddSlash(sPath) + lviObject->Items->Item[w]->Caption);
                                                                                          frmMain->SendCommand(wCommand, AddSlash(sPath) + lviObject->Items->Item[w]->Caption, this->Tag);
                                                                                          }
                                                             }
                                                 }
        }

}

void __fastcall TfrmFileManager::mnuPrintClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFilePrint->Click();
if (lviLocalFiles->Focused()) mnuLocalFilePrint->Click();
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::mnuExecuteClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileExecute->Click();
if (lviLocalFiles->Focused()) mnuLocalFileExecute->Click();
}
//---------------------------------------------------------------------------



void __fastcall TfrmFileManager::mnuFileClick(TObject *Sender)
{
CheckSelected();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteFilesMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
if (Button == mbRight) {
                       tagPOINT CursorPos;
                       GetCursorPos(&CursorPos);

                       CheckSelected();
                       if (lviRemoteFiles->SelCount > 0) { mnuRemoteFiles->Popup(CursorPos.x, CursorPos.y); } else { mnuRemoteFilesSingle->Popup(CursorPos.x, CursorPos.y); }
                       }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
if (Button == mbRight) {
                       tagPOINT CursorPos;
                       GetCursorPos(&CursorPos);

                       CheckSelected();
                       if (lviLocalFiles->SelCount > 0) { mnuLocalFiles->Popup(CursorPos.x, CursorPos.y); } else { mnuLocalFilesSingle->Popup(CursorPos.x, CursorPos.y); }
                       }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::txtRemotePathKeyPress(TObject *Sender,
      char &Key)
{
if (Key == 13)  {
                frmMain->SendCommand(COMMAND_FOLDER_CHANGEDIR, txtRemotePath->Text, this->Tag);
                lviRemoteFiles->Clear();
                }
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::CheckSelected()
{

if (txtClipboard->Text != "") {
                              mnuPaste->Enabled = true;
                              if (txtClipboard->Lines->Strings[0].SubString(1, 5) == "LOCAL") {
                                                                                                mnuLocalFilePaste->Enabled = true;
                                                                                                mnuRemoteFilePaste->Enabled = false;
                                                                                                mnuLocalFileSinglePaste->Enabled = true;
                                                                                                mnuRemoteFileSinglePaste->Enabled = false;
                                                                                              } else {
                                                                                                mnuLocalFilePaste->Enabled = false;
                                                                                                mnuRemoteFilePaste->Enabled = true;
                                                                                                mnuLocalFileSinglePaste->Enabled = false;
                                                                                                mnuRemoteFileSinglePaste->Enabled = true;
                                                                                              }
                              } else {
                              mnuPaste->Enabled = false;
                              mnuLocalFilePaste->Enabled = false;
                              mnuRemoteFilePaste->Enabled = false;
                              mnuLocalFileSinglePaste->Enabled = false;
                              mnuRemoteFileSinglePaste->Enabled = false;
                              }

if (lviRemoteFiles->Focused() && lviRemoteFiles->SelCount > 0) {
                                                                mnuUpload->Enabled = false;
                                                                mnuExecute->Enabled = true;
                                                                mnuPrint->Enabled = true;
                                                                mnuCreate->Enabled = true;
                                                                mnuCut->Enabled = true;
                                                                mnuCopy->Enabled = true;
                                                                mnuRename->Enabled = true;
                                                                mnuDelete->Enabled = true;
                                                                mnuDownload->Enabled = true;
                                                                mnuShare->Enabled = true;
                                                                mnuRemoveShare->Enabled = true;
                                                                mnuProperties->Enabled = true;
                                                                return;
                                                                } else {

                                                                mnuExecute->Enabled = false;
                                                                mnuPrint->Enabled = false;
                                                                mnuCreate->Enabled = false;
                                                                mnuCut->Enabled = false;
                                                                mnuCopy->Enabled = false;
                                                                mnuRename->Enabled = false;
                                                                mnuDelete->Enabled = false;
                                                                mnuDownload->Enabled = false;
                                                                mnuUpload->Enabled = false;
                                                                mnuShare->Enabled = false;
                                                                mnuRemoveShare->Enabled = false;
                                                                mnuProperties->Enabled = false;
                                                                if (lviRemoteFiles->Focused()) return;
                                                                }

if (lviLocalFiles->Focused() && lviLocalFiles->SelCount > 0) {
                                                                mnuDownload->Enabled = false;
                                                                mnuExecute->Enabled = true;
                                                                mnuPrint->Enabled = true;
                                                                mnuCreate->Enabled = true;
                                                                mnuCut->Enabled = true;
                                                                mnuCopy->Enabled = true;
                                                                mnuRename->Enabled = true;
                                                                mnuDelete->Enabled = true;
                                                                mnuUpload->Enabled = true;
                                                                mnuShare->Enabled = true;
                                                                mnuRemoveShare->Enabled = true;
                                                                mnuProperties->Enabled = true;
                                                                return;
                                                                } else {
                                                                mnuExecute->Enabled = false;
                                                                mnuPrint->Enabled = false;
                                                                mnuCreate->Enabled = false;
                                                                mnuCut->Enabled = false;
                                                                mnuCopy->Enabled = false;
                                                                mnuRename->Enabled = false;
                                                                mnuDelete->Enabled = false;
                                                                mnuDownload->Enabled = false;
                                                                mnuUpload->Enabled = false;
                                                                mnuShare->Enabled = false;
                                                                mnuRemoveShare->Enabled = false;
                                                                mnuProperties->Enabled = false;
                                                                if (lviLocalFiles->Focused()) return;
                                                                }
}

void __fastcall TfrmFileManager::FormClose(TObject *Sender,
      TCloseAction &Action)
{
frmMain->WriteToINI("Window", "FMWidth", this->Width);
frmMain->WriteToINI("Window", "FMHeight", this->Height);
frmMain->WriteToINI("Window", "FMState", this->WindowState);
frmMain->WriteToINIBool("Window", "FMViewLarge", mnuLargeIcons->Checked);
frmMain->WriteToINIBool("Window", "FMViewSmall", mnuSmallIcons->Checked);
frmMain->WriteToINIBool("Window", "FMViewDetails", mnuReport->Checked);
frmMain->WriteToINIBool("Window", "FMViewList", mnuList->Checked);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TfrmFileManager::ListSelected(TListView * lviObject, const AnsiString sPath)
{
unsigned short int w;
AnsiString retString = "";
AnsiString newPath;

newPath = sPath;
if (sPath != "") newPath = AddSlash(sPath);

for (w = 0; w < lviObject->Items->Count; ++w)
         {
         if (lviObject->Items->Item[w]->Selected) retString += newPath + lviObject->Items->Item[w]->Caption + "\r\n";
         }

return retString;
}


void __fastcall TfrmFileManager::mnuRemoteFileDeleteClick(TObject *Sender)
{
AnsiString langRead = ReadFromLanguage("InputBox", "msgDelete", ListSelected(lviRemoteFiles, "") + "\r\n" + "Are you sure you want to delete the selected files/folders?");
unsigned short int mAns;

mAns = MessageBox(Application->Handle, langRead.c_str(), DelHK(mnuDelete->Caption).c_str(), 4);
if (mAns != mrYes) return;

SendFileCommand(COMMAND_FILE_DELETE, txtRemotePath->Text, lviRemoteFiles, false);
SendFileCommand(COMMAND_FOLDER_DELETE, txtRemotePath->Text, lviRemoteFiles, true);


//mnuRefresh->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileDeleteClick(TObject *Sender)
{
//ReadFromLanguage("InputBox", "msgReplace", "Replace all files that have the same filename?")
AnsiString langRead = ReadFromLanguage("InputBox", "msgDelete", ListSelected(lviLocalFiles, "") + "\r\n" + "Are you sure you want to delete the selected files/folders?");
unsigned short int w;
unsigned short int mAns;
AnsiString newPath = AddSlash(txtLocalPath->Text);


mAns = MessageBox(Application->Handle, langRead.c_str(), DelHK(mnuDelete->Caption).c_str(), 4);
if (mAns != mrYes) return;


for (w = 0; w < lviLocalFiles->Items->Count; ++w)
        {
        try
        {
        if (lviLocalFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviLocalFiles, w)) {
                                                                                      } else {
                                                                                      DeleteFile(newPath + lviLocalFiles->Items->Item[w]->Caption);
                                                                                      }
                                                      }
        }
        catch (Exception &exception)
        {
        }
        }


mnuRefresh->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFolderShareClick(TObject *Sender)
{
unsigned short int w;
AnsiString FullCommand;

for (w = 0; w < lviRemoteFiles->Items->Count; ++w)
        {
        if (lviRemoteFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviRemoteFiles, w)) {
                                                                                       FullCommand = "NET.EXE" + SEPARATOR_SPLIT + PARAMETER_INVISIBLE  + SEPARATOR_SPLIT + "SHARE """ + lviRemoteFiles->Items->Item[w]->Caption + """=""" + AddSlash(txtRemotePath->Text) + lviRemoteFiles->Items->Item[w]->Caption + """";
                                                                                       frmMain->SendCommand(COMMAND_FILE_EXECUTE, FullCommand, this->Tag);
                                                                                       }
                                                      }
        }

}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFolderShareClick(TObject *Sender)
{
unsigned short int w;
AnsiString FullCommand;

for (w = 0; w < lviLocalFiles->Items->Count; ++w)
        {
        if (lviLocalFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviLocalFiles, w)) {
                                                                                      FullCommand = "SHARE """ + lviRemoteFiles->Items->Item[w]->Caption + """=""" + AddSlash(txtRemotePath->Text) + lviRemoteFiles->Items->Item[w]->Caption + """";
                                                                                      ShellExecute(Application->Handle, "open", "NET.EXE", FullCommand.c_str(), "", SW_HIDE);
                                                                                      }
                                                      }
        }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuShareClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFolderShare->Click();
if (lviLocalFiles->Focused()) mnuLocalFolderShare->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoveShareClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFolderRemoveShare->Click();
if (lviLocalFiles->Focused()) mnuLocalFolderRemoveShare->Click();

}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFileExecuteClick(TObject *Sender)
{
if (lviRemoteFiles->SelCount == 1) {
                                   TfrmExecute * frmExecuteInst = new TfrmExecute(Application);

                                   if (frmExecuteInst == NULL) return; // Memory allocation error

                                   frmExecuteInst->txtPath->Text = AddSlash(txtRemotePath->Text) + lviRemoteFiles->Selected->Caption;
                                   frmExecuteInst->Tag = this->Tag;
                                   frmExecuteInst->Caption = "[" + frmMain->HandleToHostname(this->Tag) + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuExecute", mnuExecute->Caption));
                                   frmExecuteInst->Show();
                                   return;
                                   }

if (lviRemoteFiles->SelCount > 1) {
                                  SendFileCommand(COMMAND_FILE_EXECUTE, txtRemotePath->Text, lviRemoteFiles, false);
                                  }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileExecuteClick(TObject *Sender)
{
unsigned short int w;
AnsiString FullPath = "";

for (w = 0; w < lviLocalFiles->Items->Count; ++w)
                                        {
        try
        {
                                        FullPath = AddSlash(txtLocalPath->Text) + lviLocalFiles->Items->Item[w]->Caption;
                                        if (lviLocalFiles->Items->Item[w]->Selected) ShellExecute(Application->Handle, "open", FullPath.c_str(), "", "", SW_SHOW);
        }
        catch (Exception &exception)
        {
        }
                                        }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFilePrintClick(TObject *Sender)
{
SendFileCommand(COMMAND_FILE_PRINT, txtRemotePath->Text, lviRemoteFiles, false);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFilePrintClick(TObject *Sender)
{
unsigned short int w;
AnsiString FullPath = "";

                              for (w = 0; w < lviLocalFiles->Items->Count; ++w)
                                        {
        try
        {
                                        FullPath = AddSlash(txtLocalPath->Text) + lviLocalFiles->Items->Item[w]->Caption;
                                        if (lviLocalFiles->Items->Item[w]->Selected) ShellExecute(Application->Handle, "print", FullPath.c_str(), "", "", SW_SHOW);
        }
        catch (Exception &exception)
        {
        }
                                        }
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::lviRemoteDriveMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
bool defBool;


if (Button == mbRight) {
                       tagPOINT CursorPos;
                       GetCursorPos(&CursorPos);

                       if (lviRemoteDrive->Selected->SubItems->Strings[0] == "Removable") {
                                                                                          defBool = true;
                                                                                          } else {
                                                                                          defBool = false;
                                                                                          }

                       N7->Visible = defBool;
                       mnuRemoteDriveOpenCDROM->Visible = defBool;
                       mnuRemoteDriveCloseCDROM->Visible = defBool;

                       if (lviRemoteDrive->SelCount > 0) mnuRemoteDrive->Popup(CursorPos.x, CursorPos.y);
                       }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalDriveMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
bool defBool;


if (Button == mbRight) {
                       tagPOINT CursorPos;
                       GetCursorPos(&CursorPos);

                       if (lviLocalDrive->Selected->SubItems->Strings[0] == "Removable") {
                                                                                          defBool = true;
                                                                                          } else {
                                                                                          defBool = false;
                                                                                          }

                       N15->Visible = defBool;
                       mnuLocalDriveOpenCDROM->Visible = defBool;
                       mnuLocalDriveCloseCDROM->Visible = defBool;

                       if (lviLocalDrive->SelCount > 0) mnuLocalDrive->Popup(CursorPos.x, CursorPos.y);
                       }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteDriveOpenCDROMClick(
      TObject *Sender)
{
frmMain->SendCommand(COMMAND_OPENCD, lviRemoteDrive->Selected->Caption, this->Tag);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteDriveCloseCDROMClick(
      TObject *Sender)
{
frmMain->SendCommand(COMMAND_CLOSECD, lviRemoteDrive->Selected->Caption, this->Tag);
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::cmdRemoteFileUpClick(TObject *Sender)
{
frmMain->SendCommand(COMMAND_FOLDER_CHANGEDIR, OneDirUp(txtRemotePath->Text), this->Tag);
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::mnuCreateClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFolderCreate->Click();
if (lviLocalFiles->Focused()) mnuLocalFolderCreate->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRenameClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileRename->Click();
if (lviLocalFiles->Focused()) mnuLocalFileRename->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuCutClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileCut->Click();
if (lviLocalFiles->Focused()) mnuLocalFileCut->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuCopyClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileCopy->Click();
if (lviLocalFiles->Focused()) mnuLocalFileCopy->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuDeleteClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileDelete->Click();
if (lviLocalFiles->Focused()) mnuLocalFileDelete->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuPasteClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFilePaste->Click();
if (lviLocalFiles->Focused()) mnuLocalFilePaste->Click();
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::mnuRemoteFolderRemoveShareClick(
      TObject *Sender)
{
unsigned short int w;
AnsiString FullCommand;

for (w = 0; w < lviRemoteFiles->Items->Count; ++w)
        {
        if (lviRemoteFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviRemoteFiles, w)) {
                                                                                      FullCommand = "NET.EXE"  + SEPARATOR_SPLIT + PARAMETER_INVISIBLE + SEPARATOR_SPLIT + "SHARE """ + AddSlash(txtRemotePath->Text) + lviRemoteFiles->Items->Item[w]->Caption + """ /DELETE";
                                                                                      frmMain->SendCommand(COMMAND_FILE_EXECUTE, FullCommand, this->Tag);
                                                                                      }
                                                      }
        }

}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFolderRemoveShareClick(
      TObject *Sender)
{
unsigned short int w;
AnsiString FullCommand;

for (w = 0; w < lviLocalFiles->Items->Count; ++w)
        {
        if (lviLocalFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviLocalFiles, w)) {
                                                                                      FullCommand = "SHARE """ + AddSlash(txtLocalPath->Text) + lviLocalFiles->Items->Item[w]->Caption + """ /DELETE";
                                                                                      ShellExecute(Application->Handle, "open", "NET.EXE", FullCommand.c_str(), "", SW_HIDE);
                                                                                      }
                                                      }
        }

}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::GetLocalList(const AnsiString sPath)
{
string msg;

lviLocalFiles->Clear();
msg = GetFolderList(sPath, true);

if (msg == "-1") {
                 return;
                 }

txtLocalPath->Text = sPath;
frmMain->SplitToListView(lviLocalFiles, AnsiString(msg.c_str()));
frmMain->SwitchIcon(lviLocalFiles, 3, 3, "-");
frmMain->SwitchIcon(lviLocalFiles, 3, 2, "D");
}


void __fastcall TfrmFileManager::lviLocalDriveDblClick(TObject *Sender)
{
GetLocalList(lviLocalDrive->Selected->Caption + "\\");
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesDblClick(TObject *Sender)
{
if (!IsFolder(lviLocalFiles, lviLocalFiles->Selected->Index)) {
                                                                mnuLocalFileExecute->Click();
                                                                return;
                                                                }

GetLocalList(AddSlash(txtLocalPath->Text) + lviLocalFiles->Selected->Caption);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::cmdLocalFileUpClick(TObject *Sender)
{
txtLocalPath->Text = OneDirUp(txtLocalPath->Text);
GetLocalList(txtLocalPath->Text);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::txtLocalPathKeyPress(TObject *Sender,
      char &Key)
{
if (Key == 13) {
                lviLocalFiles->Clear();
                GetLocalList(txtLocalPath->Text);
                }        
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::AddToClipboard(const AnsiString sType, const AnsiString sPath, TListView * sObject)
{
unsigned short int w;
AnsiString newPath = AddSlash(sPath);


txtClipboard->Clear();
txtClipboard->Lines->Add(sType);

for (w = 0; w < sObject->Items->Count; ++w)
        {
        if (sObject->Items->Item[w]->Selected) {
                                               //if (sType == COMMAND_FILE_MOVE || sType == "LOCALCUT") sObject->Items->Item[w]->StateIndex = 1;
                                               txtClipboard->Lines->Add(newPath + sObject->Items->Item[w]->Caption);
                                               }
        }

}

void __fastcall TfrmFileManager::PasteClipboard(const AnsiString sPath)
{
unsigned short int w;
AnsiString newPath = AddSlash(sPath);
AnsiString FullCommand;
AnsiString newFileName;


if (txtClipboard->Lines->Strings[0] == COMMAND_FILE_COPY || txtClipboard->Lines->Strings[0] == COMMAND_FILE_MOVE) {
         for (w = 1; w < txtClipboard->Lines->Count; ++w)
         {
         FullCommand = txtClipboard->Lines->Strings[w] + SEPARATOR_SPLIT + newPath + ExtractFileName(txtClipboard->Lines->Strings[w]);
         frmMain->SendCommand(txtClipboard->Lines->Strings[0], FullCommand, this->Tag);
         }
                                                                }

if (txtClipboard->Lines->Strings[0].SubString(1, 5) == "LOCAL") {
         for (w = 1; w < txtClipboard->Lines->Count; ++w)
                  {
         try
         {
                  newFileName = newPath + ExtractFileName(txtClipboard->Lines->Strings[w]);

                  if (txtClipboard->Lines->Strings[0] == "LOCALCUT") {
                                                                     MoveFile(txtClipboard->Lines->Strings[w].c_str(), newFileName.c_str());
                                                                     }
                  if (txtClipboard->Lines->Strings[0] == "LOCALCOPY") {
                                                                      CopyFile(txtClipboard->Lines->Strings[w].c_str(), newFileName.c_str(), false);
                                                                      }
         //FullCommand = txtClipboard->Lines->Strings[w] + SEPARATOR_SPLIT + ;
         //frmMain->SendCommand(sCommand, FullCommand, this->Tag);
        }
        catch (Exception &exception)
        {
        }
                  }
                                                                mnuRefresh->Click();
                                                                }


if (txtClipboard->Lines->Strings[0] == COMMAND_FILE_MOVE || txtClipboard->Lines->Strings[0] == "LOCALCUT") txtClipboard->Clear();

}

void __fastcall TfrmFileManager::mnuRemoteFileDownloadClick(
      TObject *Sender)
{
unsigned short int w;
unsigned int FileSize;
AnsiString newFileName;
bool aOverwrite;
bool aResume;
bool aCancel;

aOverwrite = frmMain->ReadFromINIBool("Options", "AlwaysOverwrite", false);
aResume = frmMain->ReadFromINIBool("Options", "AlwaysResume", true);
aCancel = frmMain->ReadFromINIBool("Options", "AlwaysCancel", false);


for (w = 0; w < lviRemoteFiles->Items->Count; ++w)
        {
        if (lviRemoteFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviRemoteFiles, w)) {
                                                                                       } else {
                                                                                       newFileName = AddSlash(txtLocalPath->Text) + lviRemoteFiles->Items->Item[w]->Caption;
                                                                                       FileSize = 0;
                                                                                       if (FileExists(newFileName)) {
                                                                                                                    if (aOverwrite) {
                                                                                                                                    try
                                                                                                                                    {
                                                                                                                                    FileSize = 0;
                                                                                                                                    DeleteFile(newFileName);

                                                                                                                                    }
                                                                                                                                    catch (Exception &exception)
                                                                                                                                    {
                                                                                                                                    frmMain->TypeCommandLine(this->Tag, exception.Message);
                                                                                                                                    continue;
                                                                                                                                    }

                                                                                                                                    }

                                                                                                                    if (aResume) {
                                                                                                                                    FileSize = ReadFileSize(newFileName.c_str());
                                                                                                                                    }

                                                                                                                    if (aCancel) {
                                                                                                                                    frmMain->TypeCommandLine(this->Tag, FullReplace(ReadFromLanguage("ClientMain", "msgSkipped", "%1 exists. Skipped."), "%1", newFileName));
                                                                                                                                    FileSize = 0;
                                                                                                                                    continue;
                                                                                                                                    }


                                                                                                                    }


                                                                                       frmWork->lviWork->AddItem(newFileName, frmWork->lviWork);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(AddSlash(txtRemotePath->Text) + lviRemoteFiles->Items->Item[w]->Caption);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(COMMAND_FILE_DOWNLOAD);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(frmMain->HandleToHostname(this->Tag));
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(FileSize);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(this->Tag);
                                                                                       }
                                                      }
        }




//frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Text = "d:\\joo.zip\r\nGET\r\njoo\r\n0\r\n" + IntToStr(tmpHandle);
frmWork->ProcessNextCommand(0);

}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::mnuRemoteFileCutClick(TObject *Sender)
{
AddToClipboard(COMMAND_FILE_MOVE, txtRemotePath->Text, lviRemoteFiles);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFileCopyClick(TObject *Sender)
{
AddToClipboard(COMMAND_FILE_COPY, txtRemotePath->Text, lviRemoteFiles);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuRemoteFilePasteClick(TObject *Sender)
{
PasteClipboard(txtRemotePath->Text);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFilePasteClick(TObject *Sender)
{
PasteClipboard(txtLocalPath->Text);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileCopyClick(TObject *Sender)
{
AddToClipboard("LOCALCOPY", txtLocalPath->Text, lviLocalFiles);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileCutClick(TObject *Sender)
{
AddToClipboard("LOCALCUT", txtLocalPath->Text, lviLocalFiles);
}
//---------------------------------------------------------------------------


void __fastcall TfrmFileManager::mnuPropertiesClick(TObject *Sender)
{
if (lviRemoteFiles->Focused()) mnuRemoteFileProperties->Click();
if (lviLocalFiles->Focused()) mnuLocalFileProperties->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalFileUploadClick(TObject *Sender)
{
unsigned short int w;
unsigned int FileSize;
AnsiString newFileName;
bool aOverwrite;
bool aResume;
bool aCancel;

aOverwrite = frmMain->ReadFromINIBool("Options", "AlwaysOverwrite", false);
aResume = frmMain->ReadFromINIBool("Options", "AlwaysResume", true);
aCancel = frmMain->ReadFromINIBool("Options", "AlwaysCancel", false);


for (w = 0; w < lviLocalFiles->Items->Count; ++w)
        {
        if (lviLocalFiles->Items->Item[w]->Selected) {
                                                      if (IsFolder(lviLocalFiles, w)) {
                                                                                       } else {
                                                                                       newFileName = AddSlash(txtRemotePath->Text) + lviLocalFiles->Items->Item[w]->Caption;
                                                                                       FileSize = 0;
                                                                                       /*
                                                                                       if (FileExists(newFileName)) {
                                                                                                                    if (aOverwrite) {
                                                                                                                                    try
                                                                                                                                    {
                                                                                                                                    FileSize = 0;
                                                                                                                                    DeleteFile(newFileName);

                                                                                                                                    }
                                                                                                                                    catch (Exception &exception)
                                                                                                                                    {
                                                                                                                                    frmMain->TypeCommandLine(this->Tag, exception.Message);
                                                                                                                                    continue;
                                                                                                                                    }

                                                                                                                                    }

                                                                                                                    if (aResume) {
                                                                                                                                    FileSize = ReadFileSize(newFileName.c_str());
                                                                                                                                    }

                                                                                                                    if (aCancel) {
                                                                                                                                    frmMain->TypeCommandLine(this->Tag, FullReplace(ReadFromLanguage("ClientMain", "msgSkipped", "%1 exists. Skipped."), "%1", newFileName));
                                                                                                                                    FileSize = 0;
                                                                                                                                    continue;
                                                                                                                                    }


                                                                                                                    }

                                                                                       */
                                                                                       frmWork->lviWork->AddItem(newFileName, frmWork->lviWork);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(AddSlash(txtLocalPath->Text) + lviLocalFiles->Items->Item[w]->Caption);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(COMMAND_FILE_UPLOAD);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(frmMain->HandleToHostname(this->Tag));
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(FileSize);
                                                                                       frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Add(this->Tag);
                                                                                       }
                                                      }
        }




//frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Text = "d:\\joo.zip\r\nGET\r\njoo\r\n0\r\n" + IntToStr(tmpHandle);
frmWork->ProcessNextCommand(0);


}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalDriveOpenCDROMClick(
      TObject *Sender)
{
OpenCloseCD(true, lviLocalDrive->Selected->Caption);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::mnuLocalDriveCloseCDROMClick(
      TObject *Sender)
{
OpenCloseCD(false, lviLocalDrive->Selected->Caption);
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteFilesStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
DragSource = "R";        
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
DragSource = "L";
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteFilesDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
if (DragSource == "L") { Accept = true; } else { Accept = false; }        
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
if (DragSource == "R") { Accept = true; } else { Accept = false; }
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviRemoteFilesDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
DragSource = "";
mnuUpload->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmFileManager::lviLocalFilesDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
DragSource = "";
mnuDownload->Click();
}
//---------------------------------------------------------------------------

