/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


/*

frmMain
Part of the Specrem-project.

Client main form.
Copyright (C) 2002  Antti Kirjavainen

*/


//---------------------------------------------------------------------------
#include <vcl.h>
#include <Jpeg.hpp>
#include "..\Shared\commands.h"
#include "..\Shared\ServerOptions.h"
#include "..\Shared\strHandle.cpp"
#include "..\Shared\hashmap.cpp"
#include "..\Shared\GenericList.hpp"
#include "Public.cpp"
//#include "Grouping.cpp"
//#include "TSocketInfo.h"
#pragma hdrstop

#include "Main.h"
#include "About.h"
#include "Chat.h"
#include "Users.h"
#include "Options.h"
#include "Find.h"
#include "Execute.h"
#include "Multi.h"
#include "Add.h"
#include "CommandLine.h"
#include "RemoteControl.h"
#include "Progress.h"
#include "TaskManager.h"
#include "FileManager.h"
#include "Password.h"
#include "Work.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "trayicon"
#pragma resource "*.dfm"

TfrmMain * frmMain;


unsigned short int State = 0;
unsigned short int ConnectionCount = 0;
unsigned int FilePort = 4339;
unsigned int CallbackPort = 2000;
bool AskForPassword = false;
enum TShutdownType { SHUTDOWN, REBOOT, LOGOFF, FORCE };
hashmap myDB;
GenericList lst;

unsigned short int SocketCount = 0;
bool Callback;
AnsiString AppPath = "";



// Temporarily variables for debugging:
unsigned int tmpHandle = 172;


// Language file defines:
AnsiString HourString = "Hours";
AnsiString MinuteString = "Minutes";
AnsiString SecondString = "Seconds";
AnsiString RemainingString = "Remaining";
AnsiString Password = "";



void RemoteShutdown(TShutdownType ShutdownParam);
void SaveAddresses();
void Disconnect();
void Connect();
void ProcessCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle);
void ExecuteLocalFile(AnsiString FileName, AnsiString Params);
void ReadScript();
void OpenFileSocket(const unsigned short int sPort, const unsigned int sHandle);
void LoadNewWindow(const unsigned short int WhatToLoad);
int ProcessBatchCommand(const AnsiString sCommand, const AnsiString sParams);


TfrmRemoteControl * frmSimpleMouseAndKey = NULL;


class TConnectedSocket
{
        public:
        unsigned int Port;
        AnsiString Address;
        AnsiString Router;
};




/*

Class TSocketInfo
Part of the Specrem-project.

*/



class TSocketInfo
{
        public:
        TSocketInfo() { tmrPing = NULL; tmrProgress = NULL; sckClientInst = NULL; sckFileInst = NULL; screenPart = NULL; frmUsersInst = NULL; frmChatInst = NULL; frmOptionsInst = NULL; frmRemoteControlInst = NULL; frmMouseMoveInst = NULL; frmFileManagerInst = NULL; frmProgressInst = NULL; frmTaskManagerInst = NULL; sPort = 1187; sHost = ""; sRouter = ""; usFilePos = 0; usFileSize = 0; usFileOp = 0; usEncryptionKey = NULL; usFileHandle = NULL; usFileName = ""; }
        ~TSocketInfo()  {
                        if (tmrPing != NULL) tmrPing->Free();
                        if (tmrProgress != NULL) tmrProgress->Free();
                        if (sckClientInst != NULL) sckClientInst->Free();
                        if (sckFileInst != NULL) sckFileInst->Free();
                        if (frmUsersInst != NULL) frmUsersInst->Free(); // Delete all the connection-specific objects
                        if (frmChatInst != NULL) frmChatInst->Free();
                        if (frmOptionsInst != NULL) frmOptionsInst->Free();
                        if (frmMouseMoveInst != NULL) frmMouseMoveInst->Free();
                        if (frmFileManagerInst != NULL) frmFileManagerInst->Free();
                        if (frmProgressInst != NULL) frmProgressInst->Free();
                        if (frmRemoteControlInst != NULL) frmRemoteControlInst->Free();
                        if (frmTaskManagerInst != NULL) frmTaskManagerInst->Free();
                        if (screenPart != NULL) screenPart->Free();
                        }

        unsigned int sPort;
        AnsiString sHost;
        AnsiString sRouter;

        TMemoryStream * screenPart; // For getting screen view (Full control, Screen spy).
        unsigned short int screenX1;
        unsigned short int screenX2; // Coordinates for the screen part,
        unsigned short int screenY1; // where to paint it to the remote control screen.
        unsigned short int screenY2;

        unsigned int usFileOp; // File operation, 0 = nothing, 1 = download, 2 = upload.
        __int64 usFilePos; // When sending file, save the file position here.
        __int64 usFileLastPos; // To calculate the transfer rate.
        unsigned int usFileSize; // When sending file, save the file size here.
        unsigned int usFileHandle; // When sending file, save the file handle here.
        AnsiString usFileName; // Filename saved here

        TTimer * tmrPing; // Ping the server and avoid ping timeout.
        TTimer * tmrProgress; // File transfer progress

        TClientSocket * sckClientInst; // Command socket
        TClientSocket * sckFileInst; // File transfer socket
        TfrmChat * frmChatInst; // Server chat
        TfrmUsers * frmUsersInst; // Server users
        TfrmServerOptions * frmOptionsInst; // Server settings 
        TfrmMulti * frmMouseMoveInst; // Mouse listen
        TfrmFileManager * frmFileManagerInst; // File manager
        TfrmProgress * frmProgressInst; // Window manager
        TfrmRemoteControl * frmRemoteControlInst; // Remote control (Simple mouse and key, Full control, Screen spy, Screenshot, Webcam shot).
        TfrmTaskManager * frmTaskManagerInst; // Window manager

        void SetEncryptionKey(AnsiString aKey) { usEncryptionKey = aKey; }
        AnsiString GetKey() const { return usEncryptionKey; }

        private:
        AnsiString usEncryptionKey;
};


int __fastcall TfrmMain::SendCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle)
{
AnsiString FullCommand;
unsigned short int s;
unsigned int nHandle = NULL;
TSocketInfo * infoKey = NULL;


if (ConnectionCount <= 0) return -1;

try
{

FullCommand = sCommand;
if (sParams != "") FullCommand += SEPARATOR_COMMAND + sParams;
FullCommand += SEPARATOR_END;

/*
// Encryption code here:
FullCommand = Encrypt(FullCommand);
*/

//txtDebug->Text = txtDebug->Text + "OUTGOING: " + FullCommand + "\r\n";

if (Callback)
            {
            for (s = 0; s < sckCallback->Socket->ActiveConnections; ++s)
                {
                sckCallback->Socket->Connections[s]->SendText(FullCommand);
                }
            return 0;
            }



if (sHandle != 0)
                {
                infoKey = (TSocketInfo *) myDB.getValue(sHandle);
                if (infoKey == NULL)
                                   {
                                   return -1;
                                   }
                if (infoKey->sckClientInst == NULL) return -1;
                //ShowMessage(IntToStr(
                infoKey->sckClientInst->Socket->SendText(FullCommand);
                //));
                }
                else
                {
                for (s = 0; s < lst.size(); ++s)
                         {
                         lst.get(s, (void *) &nHandle);
                         infoKey = (TSocketInfo *) myDB.getValue(nHandle);
                         if (infoKey != NULL) {
                                                if (infoKey->sckClientInst == NULL) continue;
                                                infoKey->sckClientInst->Socket->SendText(FullCommand);
                                                }
                         }
                }


return 0;

}
catch (Exception &exception)
{
        return -1;
}

}

void __fastcall TfrmMain::ShowScreenPart(const unsigned int sHandle, const bool Done)
{
TSocketInfo * infoKey = NULL;
TJPEGImage * jpgImage = new TJPEGImage;
unsigned int lastPos;


try
{
infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

//if infoKey->usFileOp > 0) SendCommand(COMMAND_TRANSFER_STOP, "", sHandle);

if (jpgImage == NULL) return;
if (infoKey->screenPart == NULL) return;

lastPos = infoKey->screenPart->Position;
infoKey->screenPart->Position = 0;
jpgImage->LoadFromStream(infoKey->screenPart);
if (Done) { infoKey->screenPart->Clear(); infoKey->usFileOp = 0; } else { infoKey->screenPart->Position = lastPos; }




if (infoKey->frmRemoteControlInst->tmrMouseMove->Tag != -1) {
                                                            if (infoKey->frmRemoteControlInst == NULL) {
                                                                                                        infoKey->frmRemoteControlInst = new TfrmRemoteControl(Application);
                                                                                                        if (infoKey->frmRemoteControlInst == NULL) return; // Memory allocation error
                                                                                                        }
                                                            infoKey->frmRemoteControlInst->Show();
                                                            infoKey->frmRemoteControlInst->Canvas->Draw(infoKey->screenX1, infoKey->screenY1, jpgImage);
                                                            } 

}
catch (Exception &exception)
{
}

jpgImage->Free();
if (!Done) return;


infoKey->usFileHandle = NULL;
if (infoKey->frmRemoteControlInst->tmrMouseMove->Tag != -1) SendCommand(COMMAND_SCREENSHOT, "", sHandle);
//if (infoKey->frmRemoteControlInst->tmrMouseMove->Tag == 2) SendCommand(COMMAND_WEBCAMSHOT, "", sHandle);
}

void __fastcall TfrmMain::StopTransfer(const unsigned int sHandle)
{
TSocketInfo * infoKey = NULL;

try
{
infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

infoKey->usFileOp = 0;
//if (infoKey->usFileOp > 0) SendCommand(COMMAND_TRANSFER_STOP, "", sHandle);
if (infoKey->usFileHandle != NULL) FileClose(infoKey->usFileHandle);

infoKey->usFileOp = 0;
infoKey->tmrProgress->Enabled = false;
if (infoKey->frmProgressInst != NULL) infoKey->frmProgressInst->Hide();
}
catch (Exception &exception)
{
}

infoKey->usFileHandle = NULL;
frmWork->Tag = 0;
frmWork->ProcessNextCommand(0);
return;

}

//---------------------------------------------------------------------------
int __fastcall TfrmMain::OpenFile(const AnsiString FileName, const unsigned int sHandle, const __int64 ResumeSize, const bool CreateIfNotFound)
{
TSocketInfo * infoKey = NULL;


try
{
infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return -1;
                   }

if (infoKey->usFileHandle != NULL) FileClose(infoKey->usFileHandle);


if (!FileExists(FileName))
                          {
                          if (CreateIfNotFound)
                                              {
                                              infoKey->usFileHandle = FileCreate(FileName);
                                              }
                                              else
                                              {
                                              return -1;
                                              }
                          }
                          else
                          {
                          infoKey->usFileHandle = FileOpen(FileName, 1);
                          }

//if (ResumeSize > 0)
//                  {
FileSeek(infoKey->usFileHandle, ResumeSize, 0);
infoKey->usFilePos = ResumeSize;
infoKey->usFileLastPos = ResumeSize;
//                  }
if (infoKey->usFileHandle == NULL) { return -1; } else { return 0; }
}
catch (Exception &exception)
{
        if (infoKey->usFileHandle != NULL) FileClose(infoKey->usFileHandle);
        infoKey->usFileHandle = NULL;
        return -1;
}

 
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::PasswordEntered(const AnsiString sPassword)
{
Password = sPassword;
mnuConnect->Click();
}

//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}


void __fastcall TfrmMain::WriteToINIBool(const AnsiString sSection, const AnsiString sKey, const bool sValue)
{
IniHandler INI;
char * sInt;


if (sValue) { sInt = "1"; } else { sInt = "0"; }


INI.readFromFile("SClient.Ini"); 
INI.add(sSection.c_str(), sKey.c_str(), sInt);
INI.save("SClient.Ini");
}


void __fastcall TfrmMain::WriteToINI(const AnsiString sSection, const AnsiString sKey, const AnsiString sValue)
{
IniHandler INI;

INI.readFromFile("SClient.Ini"); 
INI.add(sSection.c_str(), sKey.c_str(), sValue.c_str());
INI.save("SClient.Ini");
}


bool __fastcall TfrmMain::ReadFromINIBool(const AnsiString sSection, const AnsiString sKey, const bool sDefault)
{
IniHandler INI;
bool defReturn;
char Buffer[MAX_BUFFER_READ] = "";
AnsiString ansBuffer = "";

INI.readFromFile("SClient.Ini");
INI.getItemValue(sSection.c_str(), sKey.c_str(), Buffer);

defReturn = sDefault;
ansBuffer = AnsiString(Buffer);
if (ansBuffer == "1") defReturn = true;
if (ansBuffer == "0") defReturn = false;


return defReturn;
}


AnsiString __fastcall TfrmMain::ReadFromINI(const AnsiString sSection, const AnsiString sKey, const AnsiString sDefault)
{
IniHandler INI;
AnsiString defReturn = "";
char Buffer[MAX_BUFFER_READ] = "";


INI.readFromFile("SClient.Ini");
INI.getItemValue(sSection.c_str(), sKey.c_str(), Buffer);

defReturn = AnsiString(Buffer);
if (defReturn == "") defReturn = sDefault;


return defReturn;
}


//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormResize(TObject *Sender)
{
lviAddress->Width = this->Width - 10;
cboGroup->Width = lviAddress->Width;
lviAddress->Height = this->Height - 95;
}

//---------------------------------------------------------------------------
/*void __fastcall __closure AppException(TObject *Sender, Exception * E)
{
// Nothing...
}
*/
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuExitClick(TObject *Sender)
{
this->Close();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormClose(TObject *Sender, TCloseAction &Action)
{

Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::SaveAddresses()
{
// Save the address list:

unsigned short int l;
AnsiString curGroup;
AnsiString strl;


//lviAddress->SortType = ::stBoth;
//lviAddress->SortType = ::stNone;
lviAddress->SortType = 4;
lviAddress->SortType = 0;
curGroup = "Group_" + cboGroup->Text;

for (l = 0; l < 500; ++l)
        {
        strl = IntToStr(l);

        if (l >= lviAddress->Items->Count)
                                                  {
                                                  if (ReadFromINI(curGroup, "Address" + strl, "") == "")
                                                                                              {
                                                                                              // No old sections found.
                                                                                              return;
                                                                                              }
                                                                                              else
                                                                                              {
                                                                                              // Let's erase the old sections:
                                                                                              WriteToINI(curGroup, "Address" + strl, "");
                                                                                              WriteToINI(curGroup, "Port" + strl, "");
                                                                                              WriteToINI(curGroup, "Router" + strl, "");
                                                                                              WriteToINI(curGroup, "NIC" + strl, "");
                                                                                              }
                                                  continue;
                                                  }

        WriteToINI(curGroup, "Address" + strl, lviAddress->Items->Item[l]->Caption);
        if (lviAddress->Items->Item[l]->SubItems->Count >= 1) WriteToINI(curGroup, "Port" + strl, lviAddress->Items->Item[l]->SubItems->Strings[0]);
        if (lviAddress->Items->Item[l]->SubItems->Count >= 2) WriteToINI(curGroup, "Router" + strl, lviAddress->Items->Item[l]->SubItems->Strings[1]);
        if (lviAddress->Items->Item[l]->SubItems->Count >= 3) WriteToINI(curGroup, "NIC" + strl, lviAddress->Items->Item[l]->SubItems->Strings[2]);
        }



}


void __fastcall TfrmMain::ReadGroups()
{
// Read the group list:

unsigned short int g;
AnsiString GroupRead;

if (ReadFromINI("Groups", "Group0", "") == "") WriteToINI("Groups", "Group0", "Default");

for (g = 0; g < 500; ++g)
        {
        GroupRead = ReadFromINI("Groups", "Group" + IntToStr(g), "");
        if (GroupRead == "") {
                           break;
                           } else {
                           cboGroup->AddItem(GroupRead, cboGroup);
                           }
        }

if (cboGroup->Items->Count > 0) {
                                        cboGroup->Text = cboGroup->Items->Strings[0];
                                        OpenGroup(cboGroup->Text);
                                        }
}


void __fastcall TfrmMain::OpenGroup(const AnsiString GroupName)
{
// Read info from the selected group:
unsigned short int g;
AnsiString GroupRead;
AnsiString GroupRead2;
AnsiString curGroup;

lviAddress->Clear();


for (g = 0; g < 500; ++g)
        {
        curGroup = "Group_" + GroupName;
        GroupRead = ReadFromINI(curGroup, "Address" + IntToStr(g), "");
        GroupRead2 = ReadFromINI(curGroup, "Port" + IntToStr(g), "") + "\r" +
                                        ReadFromINI(curGroup, "Router" + IntToStr(g), "") + "\r" +
                                                        ReadFromINI(curGroup, "NIC" + IntToStr(g), "");
        if (GroupRead == "")
                          {
                          return;
                          } else {
                          lviAddress->AddItem(GroupRead, lviAddress);
                          lviAddress->Items->Item[lviAddress->Items->Count - 1]->SubItems->Text = GroupRead2;
                          }
        }
}

void __fastcall TfrmMain::Status(const AnsiString StatusText)
{
sttStatus->SimpleText = StatusText;
sttStatus->Refresh();
ttIcon->Hint = StatusText;
}

void RemoteShutdown(TShutdownType ShutdownParam)
{
AnsiString txtMsg;
AnsiString txtMsg2;
AnsiString strDelay;

InputQuery(DelHK(ReadFromLanguage("ClientMain", "mnuDelay", "Delay")), DelHK(ReadFromLanguage("ClientMain", "mnuDelay", "Delay")) + "?", strDelay);
if (strDelay.Length() == 0) strDelay = "0";

frmMain->SendCommand(COMMAND_SHUTDOWN_DELAY, strDelay, 0);

if (ShutdownParam == SHUTDOWN) txtMsg = DelHK(frmMain->mnuShutdown->Caption);
if (ShutdownParam == REBOOT) txtMsg = DelHK(frmMain->mnuReboot->Caption);
if (ShutdownParam == LOGOFF) txtMsg = DelHK(frmMain->mnuLogoff->Caption);
if (ShutdownParam == FORCE) txtMsg = DelHK(frmMain->mnuForceShutdown->Caption);
txtMsg2 = txtMsg + "?";

if (MessageBox(Application->Handle, txtMsg2.c_str(), txtMsg.c_str(), 4) == mrYes)
                {
                if (ShutdownParam == SHUTDOWN) frmMain->SendCommand(COMMAND_SHUTDOWN_SHUTDOWN, "", 0);
                if (ShutdownParam == REBOOT) frmMain->SendCommand(COMMAND_SHUTDOWN_REBOOT, "", 0);
                if (ShutdownParam == LOGOFF) frmMain->SendCommand(COMMAND_SHUTDOWN_LOGOFF, "", 0);
                if (ShutdownParam == FORCE) frmMain->SendCommand(COMMAND_SHUTDOWN_FORCE, "", 0);
                }

}

void __fastcall TfrmMain::mnuShutdownClick(TObject *Sender)
{
RemoteShutdown(SHUTDOWN);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuRebootClick(TObject *Sender)
{
RemoteShutdown(REBOOT);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuLogoffClick(TObject *Sender)
{
RemoteShutdown(LOGOFF);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuForceShutdownClick(TObject *Sender)
{
RemoteShutdown(FORCE);
}
//---------------------------------------------------------------------------
void ReadScript()
{
unsigned short int b;
AnsiString tmpStr;
AnsiString newCommand;
AnsiString newParams;
unsigned short int iPos;



for (b = 0; b < frmMain->txtTemp->Lines->Count; ++b)
         {
         tmpStr = frmMain->txtTemp->Lines->Strings[b];
         iPos = AnsiPos("//", tmpStr);
         if (iPos > 0) tmpStr = tmpStr.SubString(1, iPos - 1);

         if (tmpStr != "")
                         {
                         iPos = AnsiPos(SEPARATOR_COMMAND, tmpStr);
                         if (iPos > 0)
                                     {
                                     newCommand = tmpStr.SubString(1, iPos - 1);
                                     newParams = tmpStr.SubString(iPos + 1, tmpStr.Length() - iPos);
                                     }
                                     else
                                     {
                                     newCommand = tmpStr;
                                     newParams = "";
                                     }
                         ProcessBatchCommand(newCommand, newParams);
                         }
         }

}

int __fastcall TfrmMain::IsReserved(const unsigned int sHandle)
{
TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL) return true;

if (infoKey->usFileOp > 0) { return true; } else { return false; }
}

void __fastcall TfrmMain::SetFilename(const AnsiString newFileName, const unsigned int sHandle)
{
TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL) return;

infoKey->usFileName = newFileName;
}

void __fastcall TfrmMain::mnuConnectClick(TObject *Sender)
{
AnsiString msgTmp;
unsigned short int l;


if (State > 0) {
               Disconnect();
               return;
               }

if (lviAddress->SelCount == 0) return;




lviAddress->Enabled = false;
ConnectionCount = 0;

if (!AskForPassword) Password = frmMain->ReadFromINI("Options", "Password", "");

if (Password == "")
                  {
                  frmPassword->Show();
                  return;
                  }


// Connect to all selected addresses:
for (l = 0; l < lviAddress->Items->Count; ++l)
{
if (!lviAddress->Items->Item[l]->Selected) continue;

try
{

TSocketInfo * newSocket = new TSocketInfo();

if (newSocket == NULL)
                     {
                     // Failed to even create a new instance of the class:
                     continue;
                     }

State = 1;
Status(ReadFromLanguage("Status", "sttConnecting", "Connecting..."));
SocketCount = lviAddress->SelCount;
mnuConnect->Caption = ReadFromLanguage("ClientMain", "cmdStop", "&Stop");




newSocket->sckClientInst = new TClientSocket(frmMain);
if (newSocket->sckClientInst == NULL)
                                        {
                                        // Failed to create the new socket:
                                        delete newSocket;
                                        if (ConnectionCount <= 0) Disconnect();
                                        return;
                                        }

newSocket->sckClientInst->Close();
newSocket->sckClientInst->Host = lviAddress->Items->Item[l]->Caption;
newSocket->sckClientInst->Port = StrToIntDef(lviAddress->Items->Item[l]->SubItems->Strings[0], 1187);

newSocket->sRouter = lviAddress->Items->Item[l]->SubItems->Strings[1];
newSocket->sckClientInst->OnConnect = sckCommandConnect;
newSocket->sckClientInst->OnError = sckCommandError;
newSocket->sckClientInst->OnDisconnect = sckCommandDisconnect;
newSocket->sckClientInst->OnRead = sckCommandRead;

newSocket->sHost = newSocket->sckClientInst->Host;
newSocket->sPort = newSocket->sckClientInst->Port;
newSocket->tmrPing = new TTimer(frmMain);
newSocket->tmrProgress = new TTimer(frmMain);
newSocket->tmrProgress->Enabled = false;

// Ping the server every 2 minutes:
newSocket->tmrPing->Interval = 120000;

newSocket->tmrProgress->Interval = 1000;

newSocket->tmrPing->OnTimer = tmrPingTimer;
newSocket->tmrProgress->OnTimer = tmrProgressTimer;
if (newSocket->sRouter == "") newSocket->tmrPing->Enabled = true;

// Connect and add the class to the list:
newSocket->sckClientInst->Open();


// We cannot get a proper SocketHandle to use, until the socket is connected.
newSocket->tmrPing->Tag = newSocket->sckClientInst->Socket->SocketHandle;
newSocket->tmrProgress->Tag = newSocket->sckClientInst->Socket->SocketHandle;
newSocket->sckClientInst->Tag = newSocket->sckClientInst->Socket->SocketHandle;

tmpHandle = newSocket->sckClientInst->Socket->SocketHandle;

myDB.add(tmpHandle, newSocket);
lst.add(&tmpHandle, sizeof(unsigned int));

}
catch (Exception &exception)
{
if (ConnectionCount <= 0) Disconnect();
return;
}
}

}
void __fastcall TfrmMain::CheckMenus()
{
bool defBool;


if (lviAddress->SelCount == 0)  {
                                defBool = false;
                                } else {
                                defBool = true;
                                }

mnuConnect->Enabled = defBool;
mnuConnect2->Enabled = defBool;


if (State > 0) return;
mnuEdit->Enabled = defBool;
mnuEdit2->Enabled = defBool;
mnuRemove->Enabled = defBool;
mnuRemove2->Enabled = defBool;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::lviAddressMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
if (Button == mbRight)
                     {
                     tagPOINT CursorPos;
                     GetCursorPos(&CursorPos);

                     CheckMenus();
                     mnuPopUp->Popup(CursorPos.x, CursorPos.y);
                     }
}
//---------------------------------------------------------------------------

int ProcessBatchCommand(const AnsiString sCommand, const AnsiString sParams)
{
AnsiString tmpStr;
AnsiString tmpStr2;
unsigned short int iPos;

if (sCommand.AnsiCompareIC("CONNECT") == 0) {
                                            Disconnect();
                                            tmpStr = sParams + ":";
                                            iPos = tmpStr.AnsiPos(":");
                                            frmMain->lviAddress->AddItem(tmpStr.SubString(1, iPos - 1), frmMain->lviAddress);
                                            tmpStr = tmpStr.SubString(iPos + 1, tmpStr.Length() - iPos - 1);
                                            if (tmpStr == "") tmpStr = "1187";
                                            frmMain->lviAddress->Items->Item[frmMain->lviAddress->Items->Count - 1]->SubItems->Text = tmpStr; //+ "\r\n\r\n";
                                            frmMain->lviAddress->Items->Item[frmMain->lviAddress->Items->Count - 1]->Selected = true;
                                            frmMain->mnuConnect->Click();
                                            return 0;
                                            }
if (sCommand.AnsiCompareIC("PASSWORD") == 0) { Password = sParams; return 0; }
if (sCommand.AnsiCompareIC("DISCONNECT") == 0) { Disconnect; return 0; }
if (sCommand.AnsiCompareIC("FILEPORT") == 0) { FilePort = StrToInt(sParams); return 0; }
if (sCommand.AnsiCompareIC("QUIT") == 0) { frmMain->mnuExit->Click(); return 0; }


frmMain->SendCommand(sCommand, sParams, 0);

return 0;
}



void Connect()
{
ConnectionCount++;
frmMain->Status(IntToStr(ConnectionCount) + "/" + IntToStr(SocketCount) + " " + LowerCase(ReadFromLanguage("Status", "sttConnected", "connected")));

State = 2;
frmMain->lviAddress->Enabled = false;

frmMain->mnuEdit->Enabled = false;
frmMain->mnuEdit2->Enabled = false;
frmMain->mnuRemove->Enabled = false;
frmMain->mnuRemove2->Enabled = false;
frmMain->mnuSelectAll->Enabled = false;
frmMain->mnuSelectAll2->Enabled = false;

frmMain->mnuRemoteControl->Enabled = true;
frmMain->mnuCoolFunctions->Enabled = true;
frmMain->mnuSpyFunctions->Enabled = true;
frmMain->mnuShutdownMenu->Enabled = true;

frmMain->mnuConnect->Caption = ReadFromLanguage("ClientMain", "cmdDisconnect", "&Disconnect");
frmMain->mnuConnect2->Caption = frmMain->mnuConnect->Caption;
frmMain->mnuConnect3->Caption = frmMain->mnuConnect->Caption;
}

void Disconnect()
{
unsigned short int d;
TSocketInfo * infoKey = NULL;
unsigned int nHandle = NULL;


frmMain->lviAddress->Enabled = true;
State = 0;
ConnectionCount = 0;
Callback = false;
if (AskForPassword) Password = "";
//frmMain->sckCallback->Socket->

frmMain->mnuDisableControl->Checked = false;

frmMain->mnuEdit->Enabled = true;
frmMain->mnuEdit2->Enabled = true;
frmMain->mnuRemove->Enabled = true;
frmMain->mnuRemove2->Enabled = true;
frmMain->mnuSelectAll->Enabled = true;
frmMain->mnuSelectAll2->Enabled = true;

frmMain->mnuRemoteControl->Enabled = false;
frmMain->mnuCoolFunctions->Enabled = false;
frmMain->mnuSpyFunctions->Enabled = false;
frmMain->mnuShutdownMenu->Enabled = false;

frmMain->mnuConnect->Caption = ReadFromLanguage("ClientMain", "cmdConnect", "&Connect");
frmMain->mnuConnect2->Caption = frmMain->mnuConnect->Caption;
frmMain->mnuConnect3->Caption = frmMain->mnuConnect->Caption;

frmMain->lviAddress->Enabled = true;


                 for (d = 0; d < lst.size(); ++d)
                         {
                         try
                         {
                         lst.get(d, (void *) &nHandle);
                         infoKey = (TSocketInfo *) myDB.getValue(nHandle);
                         if (infoKey != NULL) {
                                                if (infoKey->sckClientInst == NULL) continue;
                                                infoKey->sckClientInst->Close();
                                                }
                         }
                         catch (Exception &exception)
                         {
                         }
                         }


lst.clear();

frmMain->Status(ReadFromLanguage("Status", "sttDisconnected", "Disconnected"));
}


void __fastcall TfrmMain::SwitchIcon(TListView * lviObject, const unsigned int newColumnIndex,
        const unsigned int newImageIndex, const AnsiString TextToFind)
{
unsigned short int i;


try
{

for (i = 0; i < lviObject->Items->Count; ++i)
         {
         if (lviObject->Items->Item[i]->SubItems->Strings[newColumnIndex - 1].AnsiPos(TextToFind) > 0) lviObject->Items->Item[i]->ImageIndex = newImageIndex;
         }

}
catch (Exception &exception)
{
}

}


void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
//Application->OnException = AppException;
AnsiString newFile;

AppPath = ExtractFilePath(Application->ExeName);

HourString = ReadFromLanguage("File", "Hour", HourString);
MinuteString = ReadFromLanguage("File", "Minute", MinuteString);
SecondString = ReadFromLanguage("File", "Seconds", SecondString);
RemainingString = ReadFromLanguage("File", "Remaining", RemainingString);

try
{
ReadGroups();
Disconnect();

AskForPassword = ReadFromINIBool("Options", "AskForPassword", true);
if (frmMain->ReadFromINIBool("Options", "MinimizeToTray", true)) ttIcon->Visible = true;
}
catch (Exception &exception)
{
}


if (FileExists(AppPath + "Language.INI"))
{
mnuFile->Caption = ReadFromLanguage("General", "mnuFile", mnuFile->Caption);
mnuExit->Caption = ReadFromLanguage("General", "mnuExit", mnuExit->Caption);
mnuExit2->Caption = mnuExit->Caption;
mnuHelp->Caption = ReadFromLanguage("General", "mnuHelp", mnuHelp->Caption);
mnuHelpFile->Caption = ReadFromLanguage("General", "mnuHelpFile", mnuHelpFile->Caption);
mnuHomepage->Caption = ReadFromLanguage("General", "mnuHomepage", mnuHomepage->Caption);
mnuSendABugReport->Caption = ReadFromLanguage("General", "mnuSendBugReport", mnuSendABugReport->Caption);
mnuAbout->Caption = ReadFromLanguage("General", "mnuAbout", mnuAbout->Caption);

mnuConnect->Caption = ReadFromLanguage("ClientMain", "cmdConnect", mnuConnect->Caption);
mnuConnect2->Caption = mnuConnect->Caption;
mnuConnect3->Caption = mnuConnect->Caption;
mnuCallback->Caption = ReadFromLanguage("ServerMain", "mnuCallback", mnuCallback->Caption);
mnuCallback2->Caption = mnuCallback->Caption;
mnuRemoteControl->Caption = ReadFromLanguage("ClientMain", "mnuRemoteControl", mnuRemoteControl->Caption);
mnuRemoteControl2->Caption = mnuRemoteControl->Caption;
mnuSimpleMouse->Caption = ReadFromLanguage("ClientMain", "mnuSimpleMouse", mnuSimpleMouse->Caption);
mnuFullControl->Caption = ReadFromLanguage("ClientMain", "mnuFullControl", mnuFullControl->Caption);
mnuExecute->Caption = ReadFromLanguage("ClientMain", "mnuExecute", mnuExecute->Caption);
mnuFindFiles->Caption = ReadFromLanguage("ClientMain", "mnuFind", mnuFindFiles->Caption);
mnuFileManager->Caption = ReadFromLanguage("ClientMain", "mnuFileManager", mnuFileManager->Caption);
mnuFileMngr->Caption = mnuFileManager->Caption;
mnuHostOptions->Caption = ReadFromLanguage("ClientMain", "mnuHostOptions", mnuHostOptions->Caption);
mnuHostInformation->Caption = ReadFromLanguage("ClientMain", "mnuHostInfo", mnuHostInformation->Caption);
mnuHostInfo->Caption = mnuHostInformation->Caption;
mnuHostUsers->Caption = ReadFromLanguage("ClientMain", "mnuHostUsers", mnuHostUsers->Caption);
mnuHostVersion->Caption = ReadFromLanguage("ClientMain", "mnuHostVersion", mnuHostVersion->Caption);
mnuTaskManager->Caption = ReadFromLanguage("ClientMain", "mnuTaskManager", mnuTaskManager->Caption);
mnuCoolFunctions->Caption = ReadFromLanguage("ClientMain", "mnuCoolFunctions", mnuCoolFunctions->Caption);
mnuVolumeControl->Caption = ReadFromLanguage("ClientMain", "mnuVolumeControl", mnuVolumeControl->Caption);
mnuDisableControl->Caption = ReadFromLanguage("ClientMain", "mnuDisableControl", mnuDisableControl->Caption);
mnuChat->Caption = ReadFromLanguage("Chat", "mnuChat", mnuChat->Caption);
mnuSendMessage->Caption = ReadFromLanguage("General", "mnuSendMessage", mnuSendMessage->Caption);
mnuSpyFunctions->Caption = ReadFromLanguage("ClientMain", "mnuSpyFunctions", mnuSpyFunctions->Caption);
mnuKeyboardListen->Caption = ReadFromLanguage("ClientMain", "mnuKeyboardListen", mnuKeyboardListen->Caption);
mnuMouseListen->Caption = ReadFromLanguage("ClientMain", "mnuMouseListen", mnuMouseListen->Caption);
mnuScreenSpy->Caption = ReadFromLanguage("ClientMain", "mnuScreenSpy", mnuScreenSpy->Caption);
mnuWebcam->Caption = ReadFromLanguage("ClientMain", "mnuWebcam", mnuWebcam->Caption);
mnuScreenshot->Caption = ReadFromLanguage("ClientMain", "mnuScreenShot", mnuScreenshot->Caption);
mnuShutdown->Caption = ReadFromLanguage("ClientMain", "mnuShutdown", mnuShutdown->Caption);
mnuCloseServer->Caption = ReadFromLanguage("ClientMain", "mnuCloseServer", mnuCloseServer->Caption);
mnuRestartServer->Caption = ReadFromLanguage("ClientMain", "mnuRestartServer", mnuRestartServer->Caption);
mnuShutdownMenu->Caption = ReadFromLanguage("ClientMain", "mnuShutdown", mnuShutdownMenu->Caption);
mnuShutdown->Caption = ReadFromLanguage("ClientMain", "mnuShutdown", mnuShutdown->Caption);
mnuLockComputer->Caption = ReadFromLanguage("ClientMain", "mnuLock", mnuLockComputer->Caption);
mnuReboot->Caption = ReadFromLanguage("ClientMain", "mnuReboot", mnuReboot->Caption);
mnuLogoff->Caption = ReadFromLanguage("ClientMain", "mnuLogoff", mnuLogoff->Caption);
mnuForceShutdown->Caption = ReadFromLanguage("ClientMain", "mnuForce", mnuForceShutdown->Caption);
mnuMonitorOff->Caption = ReadFromLanguage("ClientMain", "mnuMonitorOff", mnuMonitorOff->Caption);
mnuStartScreensaver->Caption = ReadFromLanguage("ClientMain", "mnuStartScreenSaver", mnuStartScreensaver->Caption);
mnuStopShutdown->Caption = ReadFromLanguage("ClientMain", "mnuStopShutdown", mnuStopShutdown->Caption);
mnuSelectAll->Caption = ReadFromLanguage("General", "mnuSelectAll", mnuSelectAll->Caption);
mnuSelectAll2->Caption = mnuSelectAll->Caption;
mnuOptions->Caption = ReadFromLanguage("ServerMain", "mnuOptions", mnuOptions->Caption);
mnuOptions2->Caption = mnuOptions->Caption;
mnuView->Caption = ReadFromLanguage("ServerMain", "mnuView", mnuView->Caption);
mnuReport->Caption = ReadFromLanguage("General", "mnuDetailIcons", mnuReport->Caption);
mnuLargeIcons->Caption = ReadFromLanguage("General", "mnuLargeIcons", mnuLargeIcons->Caption);
mnuSmallIcons->Caption = ReadFromLanguage("General", "mnuSmallIcons", mnuSmallIcons->Caption);
mnuList->Caption = ReadFromLanguage("General", "mnuListIcons", mnuList->Caption);

lviAddress->Column[0]->Caption = ReadFromLanguage("ClientMain", "lblAddress", "Address:");
lviAddress->Column[1]->Caption = ReadFromLanguage("General", "lblPort", "Port:");
lviAddress->Column[3]->Caption = ReadFromLanguage("STools", "lblHostNIC", "Adapter/NIC-address:");

mnuAboutTranslation->Caption = ReadFromLanguage("Translation", "mnuAboutTranslation", mnuAboutTranslation->Caption);
mnuAboutTranslation->Visible = True;




// Missing from the language file:
mnuCommandLine->Caption = ReadFromLanguage("ClientMain", "mnuCommandLine", mnuCommandLine->Caption);
lviAddress->Column[2]->Caption = ReadFromLanguage("ClientMain", "lblRouter", "Router:");
mnuAdd->Caption = ReadFromLanguage("ClientMain", "mnuAdd", mnuAdd->Caption);
mnuAdd2->Caption = mnuAdd->Caption;
mnuRemove->Caption = ReadFromLanguage("ClientMain", "mnuRemove", mnuRemove->Caption);
mnuRemove2->Caption = mnuRemove->Caption;
mnuEdit->Caption = ReadFromLanguage("ClientMain", "mnuEdit", mnuEdit->Caption);
mnuEdit2->Caption = mnuEdit->Caption;
mnuUpdate->Caption = ReadFromLanguage("ClientMain", "mnuUpdate", mnuUpdate->Caption);


}


if (ParamStr(1) != "")
                     {
                     newFile = ParamStr(1);
                     if (!FileExists(newFile))
                                                 {
                                                 newFile = AppPath + ParamStr(1);
                                                 }
                     if (!FileExists(newFile))
                                                 {
                                                 MessageBox(Application->Handle, FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found") & "!", "%", ParamStr(1)).c_str(), ReadFromLanguage("General", "Error", "Error").c_str(), 0);
                                                 }

                     txtTemp->Clear();
                     txtTemp->Lines->LoadFromFile(ParamStr(1));
                     ReadScript();
                     }
}

void ExecuteLocalFile(AnsiString FileName, AnsiString Params)
{
AnsiString txtMsg;
AnsiString txtMsg2;

txtMsg = FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found") + "!", "%", FileName);
txtMsg2 = ReadFromLanguage("General", "Error", "Error");

if (FileExists(FileName))
                        {
                        ShellExecute(Application->Handle, "open", FileName.c_str(), Params.c_str(), "", SW_SHOW);
                        }
                        else
                        {
                        MessageBox(Application->Handle, txtMsg.c_str(), txtMsg2.c_str(), 0);
                        }
                                
}

//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuUpdateClick(TObject *Sender)
{
ExecuteLocalFile(AppPath + "UPDATE.EXE", "");
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuHelpFileClick(TObject *Sender)
{
ExecuteLocalFile(AppPath + "HELP.HTM", "");
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuHomepageClick(TObject *Sender)
{
ShellExecute(Application->Handle, "open", "http://www.htk.fi/public/akirjavainen", "", "", SW_SHOWDEFAULT);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuSendABugReportClick(TObject *Sender)
{
ShellExecute(Application->Handle, "open", "mailto:akirjavainen@mail.htk.fi", "", "", SW_SHOWDEFAULT);
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuDisableControlClick(TObject *Sender)
{
if (mnuDisableControl->Checked)
                                {
                                mnuDisableControl->Checked = False;
                                SendCommand(COMMAND_ENABLE_CONTROL, "", 0);
                                }
                                else
                                {
                                mnuDisableControl->Checked = True;
                                SendCommand(COMMAND_DISABLE_CONTROL, "", 0);
                                }
}
//---------------------------------------------------------------------------

void LoadNewWindow(const unsigned short int WhatToLoad)
//void LoadNewWindow(TSocketInfo * WhatToLoad, TForm * WhatForm, const AnsiString cCaption)
{
// **********************************************************************
// This is BAD, BAD, BAD!
// Couldn't get the window loading working any other way.
// **********************************************************************
// At least this way the code is in one place, but a BAD method to do it.
// Did not bother to create a enumerated type for this, since this will
// need to be changed at some point. Too much code in this void.
// **********************************************************************
// 0 = Chat
// 1 = Users
// 2 = File manager
// 3 = Window manager
// 4 = Full control
// 5 = Screen spy
// **********************************************************************


unsigned short int s;
unsigned int nHandle = NULL;
TSocketInfo * infoKey = NULL;


for (s = 0; s < lst.size(); ++s)
        {
        lst.get(s, (void *) &nHandle);
        if (nHandle > 0)
                                            {
                                            infoKey = (TSocketInfo *) myDB.getValue(nHandle);
                                            if (infoKey != NULL)
                                                                {
                                                                /*
                                                                WhatToLoad = new WhatForm(Application);
                                                                WhatToLoad->Caption = "[" + infoKey->sHost + "] " + cCaption;
                                                                WhatToLoad->Tag = infoKey->sckClientInst->Tag;
                                                                */
                                                                if (WhatToLoad == 0)
                                                                                   {
                                                                                   if (infoKey->frmChatInst == NULL)
                                                                                                                   {
                                                                                                                   infoKey->frmChatInst = new TfrmChat(Application);
                                                                                                                   if (infoKey->frmChatInst == NULL) continue;
                                                                                                                   infoKey->frmChatInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuChat->Caption);
                                                                                                                   infoKey->frmChatInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                   }
                                                                                   infoKey->frmChatInst->Show();
                                                                                   }
                                                                if (WhatToLoad == 1)
                                                                                   {
                                                                                   if (infoKey->frmUsersInst == NULL)
                                                                                                                   {
                                                                                                                   infoKey->frmUsersInst = new TfrmUsers(Application);
                                                                                                                   if (infoKey->frmUsersInst == NULL) continue;
                                                                                                                   infoKey->frmUsersInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuHostUsers->Caption);
                                                                                                                   infoKey->frmUsersInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                   }
                                                                                   infoKey->frmUsersInst->Show();
                                                                                   infoKey->frmUsersInst->cmdRefresh->Click();
                                                                                   }
                                                                if (WhatToLoad == 2)
                                                                                   {
                                                                                   if (infoKey->frmFileManagerInst == NULL)
                                                                                                                          {
                                                                                                                          infoKey->frmFileManagerInst = new TfrmFileManager(Application);
                                                                                                                          if (infoKey->frmFileManagerInst == NULL) continue;
                                                                                                                          infoKey->frmFileManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuFileManager->Caption);
                                                                                                                          infoKey->frmFileManagerInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                          }
                                                                                   infoKey->frmFileManagerInst->Show();
                                                                                   }
                                                                if (WhatToLoad == 3)
                                                                                   {
                                                                                   if (infoKey->frmTaskManagerInst == NULL)
                                                                                                                          {
                                                                                                                          infoKey->frmTaskManagerInst = new TfrmTaskManager(Application);
                                                                                                                          if (infoKey->frmTaskManagerInst == NULL) continue;
                                                                                                                          infoKey->frmTaskManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuTaskManager->Caption);
                                                                                                                          infoKey->frmTaskManagerInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                          }
                                                                                   infoKey->frmTaskManagerInst->Show();
                                                                                   infoKey->frmTaskManagerInst->mnuRefresh->Click();
                                                                                   }
                                                                if (WhatToLoad == 4)
                                                                                   {
                                                                                   if (infoKey->frmRemoteControlInst == NULL)
                                                                                                                          {
                                                                                                                          infoKey->frmRemoteControlInst = new TfrmRemoteControl(Application);
                                                                                                                          if (infoKey->frmRemoteControlInst == NULL) continue;
                                                                                                                          infoKey->frmRemoteControlInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuFullControl->Caption);
                                                                                                                          infoKey->frmRemoteControlInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                          }
                                                                                   infoKey->frmRemoteControlInst->tmrMouseMove->Tag = 1;
                                                                                   infoKey->frmRemoteControlInst->Show();
                                                                                   }
                                                                if (WhatToLoad == 5)
                                                                                   {
                                                                                   if (infoKey->frmRemoteControlInst == NULL)
                                                                                                                          {
                                                                                                                          infoKey->frmRemoteControlInst = new TfrmRemoteControl(Application);
                                                                                                                          if (infoKey->frmRemoteControlInst == NULL) continue;
                                                                                                                          infoKey->frmRemoteControlInst->Caption = "[" + infoKey->sHost + "] " + DelHK(frmMain->mnuFullControl->Caption);
                                                                                                                          infoKey->frmRemoteControlInst->Tag = infoKey->sckClientInst->Tag;
                                                                                                                          }
                                                                                   infoKey->frmRemoteControlInst->tmrMouseMove->Tag = 2;
                                                                                   infoKey->frmRemoteControlInst->Show();
                                                                                   }
                                                                }
                                            }
        }

}

void __fastcall TfrmMain::mnuOptionsClick(TObject *Sender)
{
Application->CreateForm(__classid(TfrmOptions), &frmOptions);
frmOptions->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuSendMessageClick(TObject *Sender)
{
AnsiString newMsg;

InputQuery("[*] " + ReadFromLanguage("InputBox", "msgSendMessage", "Message:"), DelHK(mnuSendMessage->Caption), newMsg);
if (newMsg != "")
                  {
                  SendCommand(COMMAND_MESSAGE, newMsg, 0);
                  }

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuHostVersionClick(TObject *Sender)
{
SendCommand(COMMAND_HOST_VERSION, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuCloseServerClick(TObject *Sender)
{
if (MessageBox(Application->Handle, DelHK(mnuCloseServer->Caption + "?").c_str(), DelHK(mnuCloseServer->Caption).c_str(), 4) == mrYes)
                                                {
                                                SendCommand(COMMAND_SHUTDOWN_SERVER_CLOSE, "", 0);
                                                }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuRestartServerClick(TObject *Sender)
{
if (MessageBox(Application->Handle, DelHK(mnuRestartServer->Caption + "?").c_str(), DelHK(mnuRestartServer->Caption).c_str(), 4) == mrYes)
                                                {
                                                SendCommand(COMMAND_SHUTDOWN_SERVER_RESTART, "", 0);
                                                }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuStopShutdownClick(TObject *Sender)
{
SendCommand(COMMAND_SHUTDOWN_STOP, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuStartScreensaverClick(TObject *Sender)
{
SendCommand(COMMAND_SHUTDOWN_SCREENSAVER, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuLockComputerClick(TObject *Sender)
{
SendCommand(COMMAND_SHUTDOWN_LOCK, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuExecuteClick(TObject *Sender)
{
TfrmExecute * frmExecuteInst = new TfrmExecute(Application);

if (frmExecuteInst == NULL) return; // Memory allocation error

frmExecuteInst->Tag = 0;
frmExecuteInst->Caption = "[" + HandleToHostname(0) + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuExecute", mnuExecute->Caption));
frmExecuteInst->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuAboutClick(TObject *Sender)
{
TfrmAbout * frmAboutInst = new TfrmAbout(Application);
if (frmAboutInst == NULL) return; // Memory allocation error
frmAboutInst->Show();
this->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuCommandLineClick(TObject *Sender)
{
frmCommandLine->Show();
}
//---------------------------------------------------------------------------

void SetViewStyle(TViewStyle newStyle)
{
frmMain->mnuLargeIcons->Checked = False;
frmMain->mnuSmallIcons->Checked = False;
frmMain->mnuList->Checked = False;
frmMain->mnuReport->Checked = False;

if (newStyle == vsIcon) frmMain->mnuLargeIcons->Checked = True;
if (newStyle == vsSmallIcon) frmMain->mnuSmallIcons->Checked = True;
if (newStyle == vsList) frmMain->mnuList->Checked = True;
if (newStyle == vsReport) frmMain->mnuReport->Checked = True;

frmMain->lviAddress->ViewStyle = newStyle;
}

void __fastcall TfrmMain::cboGroupKeyPress(TObject *Sender, char &Key)
{
unsigned short int g;
AnsiString lstGroup;

if (Key == 13)
             {
             for (g = 0; g < cboGroup->Items->Count; ++g)
                        {
                        lstGroup = cboGroup->Items->Strings[g];

                        // If the group already exists...
                        if (lstGroup.AnsiCompareIC(cboGroup->Text) == 0)
                                                                       {
                                                                       OpenGroup(lstGroup);
                                                                       return;
                                                                       }
                        }

                        // 500 is the limit for groups and addresses in a group:
                        if (cboGroup->Items->Count >= 500) return;

                        // If not, create it and add it to the group list:
                        WriteToINI("Groups", "Group" + IntToStr(cboGroup->Items->Count), cboGroup->Text);
                        cboGroup->AddItem(cboGroup->Text, cboGroup);
                        OpenGroup(cboGroup->Text);
             }

}
//---------------------------------------------------------------------------






void __fastcall TfrmMain::mnuSelectAllClick(TObject *Sender)
{
lviAddress->SelectAll();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuEditClick(TObject *Sender)
{
this->Enabled = false;
Application->CreateForm(__classid(TfrmAdd), &frmAdd);

frmAdd->txtAddress->Text = lviAddress->Selected->Caption;
frmAdd->txtPort->Text = lviAddress->Selected->SubItems->Strings[0];
if (lviAddress->Selected->SubItems->Count > 1) frmAdd->txtRouter->Text = lviAddress->Selected->SubItems->Strings[1];
if (lviAddress->Selected->SubItems->Count > 2) frmAdd->txtAdapterNICAddress->Text = lviAddress->Selected->SubItems->Strings[2];

frmAdd->Tag = lviAddress->Selected->Index;

frmAdd->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuSmallIconsClick(TObject *Sender)
{
SetViewStyle(vsSmallIcon);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuListClick(TObject *Sender)
{
SetViewStyle(vsList);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuReportClick(TObject *Sender)
{
SetViewStyle(vsReport);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuLargeIconsClick(TObject *Sender)
{
SetViewStyle(vsIcon);
}

void __fastcall TfrmMain::lviAddressDblClick(TObject *Sender)
{
mnuConnect->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lviAddressKeyPress(TObject *Sender, char &Key)
{
if (Key == 13) mnuConnect->Click();        
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckCommandDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
unsigned int tmpHandle = Socket->SocketHandle;


try
{
TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(Socket->SocketHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

myDB.removeValue(Socket->SocketHandle);
lst.remove(&tmpHandle, sizeof(unsigned int));
delete infoKey;

}
catch (Exception &exception)
{
}


ConnectionCount--;
Status(IntToStr(ConnectionCount) + "/" + IntToStr(SocketCount) + " " + LowerCase(ReadFromLanguage("Status", "sttConnected", "connected")));
if (ConnectionCount <= 0) Disconnect();

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckCommandConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
Connect();        
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuCallbackClick(TObject *Sender)
{
Disconnect();

sckCallback->Close();
sckCallback->Port = CallbackPort;
sckCallback->Open();

Callback = true;
Status(FullReplace(ReadFromLanguage("Status", "sttNoConnection", "No connection, listening..."), "...", " (" + IntToStr(CallbackPort) + ")..."));
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuFindFilesClick(TObject *Sender)
{
Application->CreateForm(__classid(TfrmFind), &frmFind);
frmFind->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuTaskManagerClick(TObject *Sender)
{
LoadNewWindow(3);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuAdd2Click(TObject *Sender)
{
this->Enabled = false;
Application->CreateForm(__classid(TfrmAdd), &frmAdd);
frmAdd->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::cboGroupSelect(TObject *Sender)
{
OpenGroup(cboGroup->Text);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuRemove2Click(TObject *Sender)
{
unsigned short int l;


for (l = 0; l < lviAddress->Items->Count; ++l)
        {
        if (lviAddress->Items->Item[l]->Selected) lviAddress->Items->Delete(l);
        }

tmrSave->Enabled = True;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::tmrSaveTimer(TObject *Sender)
{
tmrSave->Enabled = False;
SaveAddresses();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuHostOptionsClick(TObject *Sender)
{
SendCommand(COMMAND_SERVER_OPTIONS, "", 0);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TfrmMain::HandleToHostname(const unsigned int sHandle)
{
if (sHandle == 0) return "*";

TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return NULL;
                   }

return infoKey->sHost;
}

void __fastcall TfrmMain::TypeCommandLine(unsigned int sHandle, AnsiString sText)
{
AnsiString tmpStr;

tmpStr = HandleToHostname(sHandle);

// To get the "\n"-marks to ENTER-letters...
sText = SwitchToListFormat(sText);

frmCommandLine->Show();


if (tmpStr == "")
                {
                frmCommandLine->txtCommandLine->Text = frmCommandLine->txtCommandLine->Text + sText + "\r\n";
                }
                else
                {
                frmCommandLine->txtCommandLine->Text = frmCommandLine->txtCommandLine->Text + "[" + tmpStr + "] " + sText + "\r\n";
                }
}

void OpenFileSocket(const unsigned short int sPort, const unsigned int sHandle)
{
TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

if (infoKey->sckFileInst == NULL)
                                {
                                infoKey->sckFileInst = new TClientSocket(Application);
                                if (infoKey->sckFileInst == NULL) return;
                                }
                                else
                                {
                                if (infoKey->sckFileInst->Socket->Connected)
                                                                           {
                                                                           infoKey->sckFileInst->Socket->SendText("R");
                                                                           return;
                                                                           }
                                }

infoKey->sckFileInst->Close();
infoKey->sckFileInst->Host = infoKey->sHost;
infoKey->sckFileInst->Port = sPort;
infoKey->sckFileInst->OnConnect = frmMain->sckFileInstConnect;
infoKey->sckFileInst->OnDisconnect = frmMain->sckFileInstDisconnect;
infoKey->sckFileInst->OnError = frmMain->sckFileInstError;
infoKey->sckFileInst->OnRead = frmMain->sckFileInstRead;
infoKey->sckFileInst->Tag = sHandle;
infoKey->sckFileInst->Open();
infoKey->tmrProgress->Enabled = true;
}

void ProcessCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle)
{
AnsiString tmpStr;
AnsiString tmpStr2;
AnsiString tmpStr3;
TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }


if (sCommand.AnsiCompareIC(COMMAND_REMOTE_MOUSE_MOVE) == 0) {
                                                if (infoKey->frmMouseMoveInst == NULL)
                                                                                 {
                                                                                 infoKey->frmMouseMoveInst = new TfrmMulti(Application);
                                                                                 if (infoKey->frmMouseMoveInst == NULL) return;
                                                                                 infoKey->frmMouseMoveInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuMouseListen", "Mouse listen"));
                                                                                 infoKey->frmMouseMoveInst->Tag = sHandle;
                                                                                 infoKey->frmMouseMoveInst->mnuFile->Visible = false;
                                                                                 infoKey->frmMouseMoveInst->imgMouseCursor->Visible = true;
                                                                                 infoKey->frmMouseMoveInst->Show();
                                                                                 }
                                               infoKey->frmMouseMoveInst->imgMouseCursor->Left = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 1), 0);
                                               infoKey->frmMouseMoveInst->imgMouseCursor->Top = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 2), 0);
                                               //getCursorPos(MouseCoord);
                                               //DrawIconEx(infoKey->frmMouseMoveInst.canvas.handle, MouseCoord.x - (infoKey->frmMouseMoveInst.Left + 2),MouseCoord.y - (infoKey->frmMouseMoveInst.Top + 2), GetCursor, 0, 0, 0, 0, DI_NORMAL);
                                               //infoKey->frmMouseMoveInst.Canvas.Draw(MouseCoord.x - (Self.Left + 2) - 1, MouseCoord.y - (infoKey->frmMouseMoveInst.Top + 2), Image1.Picture.Graphic);
                                               return; }

if (sCommand.AnsiCompareIC(COMMAND_PASSWORD) == 0) {
                                                   frmMain->SendCommand(COMMAND_ENCRYPTION_KEY, "", sHandle);
                                                   return; }

if (sCommand.AnsiCompareIC(COMMAND_ENCRYPTION_KEY) == 0) {
                                                         infoKey->SetEncryptionKey(sParams); 
                                                         frmMain->SendCommand(Password, "", sHandle);
                                                         return; }
if (sCommand.AnsiCompareIC(COMMAND_SCREEN_PART) == 0) {
                                                      if (infoKey->usFileHandle != NULL) return;

                                                      if (infoKey->screenPart == NULL) {
                                                                                       infoKey->screenPart = new TMemoryStream();
                                                                                       if (infoKey->screenPart == NULL) return; // Memory allocation error...
                                                                                       }
                                                      if (infoKey->frmRemoteControlInst == NULL)
                                                                                                {
                                                                                                infoKey->frmRemoteControlInst = new TfrmRemoteControl(Application);
                                                                                                if (infoKey->frmRemoteControlInst == NULL) return; // Memory allocation error...
                                                                                                }
                                                      //infoKey->frmRemoteControlInst->tmrMouseMove->Tag = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 7), 0);

                                                      infoKey->screenPart->Clear();
                                                      infoKey->screenPart->Position = 0;
                                                      infoKey->screenX1 = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 1), 0);
                                                      infoKey->screenY1 = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 2), 0);
                                                      infoKey->screenX2 = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 3), 0);
                                                      infoKey->screenY2 = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 4), 0);
                                                      infoKey->usFileOp = 3;
                                                      infoKey->usFilePos = 0;
                                                      infoKey->usFileSize = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 5), 0);
                                                      infoKey->usFileHandle = 3;

                                                      OpenFileSocket(StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 6), 4339), sHandle);
                                                      return; }
if (sCommand.AnsiCompareIC(COMMAND_CHAT) == 0) {
                                                if (infoKey->frmChatInst == NULL)
                                                                                 {
                                                                                 infoKey->frmChatInst = new TfrmChat(Application);
                                                                                 if (infoKey->frmChatInst == NULL) return;
                                                                                 infoKey->frmChatInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("Chat", "mnuChat", "Chat"));
                                                                                 infoKey->frmChatInst->Tag = sHandle;
                                                                                 }
                                               infoKey->frmChatInst->txtChat->Text = infoKey->frmChatInst->txtChat->Text + sParams + "\r\n";
                                               if (infoKey->frmChatInst->mnuPopUpChat->Checked) infoKey->frmChatInst->Show();
                                               return; }
if (sCommand.AnsiCompareIC(COMMAND_MESSAGE) == 0) {
                                                  tmpStr = "[" + infoKey->sckClientInst->Host + "] " + ReadFromLanguage("InputBox", "msgSendMessage", "Message:");
                                                  tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 1) + "\r\n\r\n" + Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                  MessageBox(NULL, tmpStr2.c_str(), tmpStr.c_str(), 0);
                                                  return; }

if (sCommand.AnsiCompareIC(COMMAND_FOLDER_OK) == 0) {
                                                     if (infoKey->frmFileManagerInst == NULL)
                                                                                            {
                                                                                            infoKey->frmFileManagerInst = new TfrmFileManager(Application);
                                                                                            if (infoKey->frmFileManagerInst == NULL) return; // Memory allocation error
                                                                                            infoKey->frmFileManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuFileManager", "File manager"));
                                                                                            }
                                                    infoKey->frmFileManagerInst->txtRemotePath->Text = sParams;
                                                    infoKey->frmFileManagerInst->lviRemoteFiles->Clear();
                                                    frmMain->SendCommand(COMMAND_FILE_LIST_WIDE, "", sHandle);
                                                    return; }

if (sCommand.AnsiCompareIC(COMMAND_FILE_LIST_WIDE) == 0) {
                                                     if (infoKey->frmFileManagerInst == NULL)
                                                                                            {
                                                                                            infoKey->frmFileManagerInst = new TfrmFileManager(Application);
                                                                                            if (infoKey->frmFileManagerInst == NULL) return; // Memory allocation error
                                                                                            infoKey->frmFileManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuFileManager", "File manager"));
                                                                                            }
                                                     frmMain->SplitToListView(infoKey->frmFileManagerInst->lviRemoteFiles, sParams);
                                                     frmMain->SwitchIcon(infoKey->frmFileManagerInst->lviRemoteFiles, 3, 3, "-");
                                                     frmMain->SwitchIcon(infoKey->frmFileManagerInst->lviRemoteFiles, 3, 2, "D");
                                                     infoKey->frmFileManagerInst->mnuRefresh->Caption = ReadFromLanguage("General", "Refresh", "&Refresh");
                                                     infoKey->frmFileManagerInst->mnuRefresh->Enabled = true;
                                                     infoKey->frmFileManagerInst->Cursor = crDefault;
                                                     infoKey->frmFileManagerInst->Tag = sHandle;
                                                     infoKey->frmFileManagerInst->Show();
                                                     frmMain->SendCommand(COMMAND_OK, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_DRIVELIST) == 0) {
                                                     if (infoKey->frmFileManagerInst == NULL)
                                                                                            {
                                                                                            infoKey->frmFileManagerInst = new TfrmFileManager(Application);
                                                                                            if (infoKey->frmFileManagerInst == NULL) return; // Memory allocation error
                                                                                            infoKey->frmFileManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuFileManager", "File manager"));
                                                                                            }
                                                     infoKey->frmFileManagerInst->lviRemoteDrive->Clear();
                                                     frmMain->SplitToListView(infoKey->frmFileManagerInst->lviRemoteDrive, sParams);
                                                     frmMain->SwitchIcon(infoKey->frmFileManagerInst->lviRemoteDrive, 1, 0, "F");
                                                     frmMain->SwitchIcon(infoKey->frmFileManagerInst->lviRemoteDrive, 1, 1, "R");
                                                     infoKey->frmFileManagerInst->mnuRefresh->Caption = ReadFromLanguage("General", "Refresh", "&Refresh");
                                                     infoKey->frmFileManagerInst->mnuRefresh->Enabled = true;
                                                     infoKey->frmFileManagerInst->Cursor = crDefault;
                                                     infoKey->frmFileManagerInst->Tag = sHandle;
                                                     infoKey->frmFileManagerInst->Show();
                                                     //frmMain->SendCommand(COMMAND_OK, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_LIST) == 0) {
                                                     if (infoKey->frmUsersInst == NULL)
                                                                                      {
                                                                                      infoKey->frmUsersInst = new TfrmUsers(Application);
                                                                                      if (infoKey->frmUsersInst == NULL) return;
                                                                                      infoKey->frmUsersInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuHostUsers", "Host users"));
                                                                                      }
                                                     infoKey->frmUsersInst->lviUsers->Clear();
                                                     infoKey->frmUsersInst->lviUsers->AddItem(ReadFromLanguage("General", "Server", "Server"), infoKey->frmUsersInst->lviUsers);
                                                     infoKey->frmUsersInst->lviUsers->Items->Item[0]->SubItems->Add("Server");
                                                     infoKey->frmUsersInst->lviUsers->Items->Item[0]->SubItems->Add("Server");
                                                     frmMain->SplitToListView(infoKey->frmUsersInst->lviUsers, sParams);
                                                     frmMain->SwitchIcon(infoKey->frmUsersInst->lviUsers, 2, 0, "U");
                                                     frmMain->SwitchIcon(infoKey->frmUsersInst->lviUsers, 2, 1, "A");
                                                     frmMain->SwitchIcon(infoKey->frmUsersInst->lviUsers, 2, 2, "S");
                                                     infoKey->frmUsersInst->cmdRefresh->Caption = ReadFromLanguage("General", "Refresh", "&Refresh");
                                                     infoKey->frmUsersInst->cmdRefresh->Enabled = true;
                                                     infoKey->frmUsersInst->Cursor = crDefault;
                                                     infoKey->frmUsersInst->Tag = sHandle;
                                                     infoKey->frmUsersInst->lblConnectedCount->Caption = FullReplace(ReadFromLanguage("General", "lblConnectedAmount", "% client(s) connected to the server."), "%", IntToStr(infoKey->frmUsersInst->lviUsers->Items->Count - 1)); 
                                                     infoKey->frmUsersInst->Show();
                                                     //frmMain->SendCommand(COMMAND_OK, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_LIST) == 0) {
                                                     if (infoKey->frmTaskManagerInst == NULL)
                                                                                      {
                                                                                      infoKey->frmTaskManagerInst = new TfrmTaskManager(Application);
                                                                                      if (infoKey->frmTaskManagerInst == NULL) return;
                                                                                      infoKey->frmTaskManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuTaskManager", "Task manager"));
                                                                                      }
                                                     infoKey->frmTaskManagerInst->lviWindows->Clear();
                                                     frmMain->SplitToListView(infoKey->frmTaskManagerInst->lviWindows, sParams);
                                                     infoKey->frmTaskManagerInst->mnuRefresh->Caption = ReadFromLanguage("General", "Refresh", "&Refresh");
                                                     infoKey->frmTaskManagerInst->mnuRefresh->Enabled = true;
                                                     infoKey->frmTaskManagerInst->Cursor = crDefault;
                                                     infoKey->frmTaskManagerInst->Tag = sHandle;
                                                     infoKey->frmTaskManagerInst->Show();
                                                     //frmMain->SendCommand(COMMAND_OK, "", sHandle);
                                                     frmMain->SendCommand(COMMAND_PROCESS_LIST, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_PROCESS_LIST) == 0) {
                                                     if (infoKey->frmTaskManagerInst == NULL)
                                                                                      {
                                                                                      infoKey->frmTaskManagerInst = new TfrmTaskManager(Application);
                                                                                      if (infoKey->frmTaskManagerInst == NULL) return;
                                                                                      infoKey->frmTaskManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuTaskManager", "Task manager"));
                                                                                      }
                                                     infoKey->frmTaskManagerInst->lviProcess->Clear();
                                                     frmMain->SplitToListView(infoKey->frmTaskManagerInst->lviProcess, sParams);
                                                     infoKey->frmTaskManagerInst->mnuRefresh->Caption = ReadFromLanguage("General", "Refresh", "&Refresh");
                                                     infoKey->frmTaskManagerInst->mnuRefresh->Enabled = true;
                                                     infoKey->frmTaskManagerInst->Cursor = crDefault;
                                                     infoKey->frmTaskManagerInst->Tag = sHandle;
                                                     infoKey->frmTaskManagerInst->stbStatus->Panels->Items[0]->Text = ReadFromLanguage("SystemInfo", "msgProcesses", "Processes: ") + IntToStr(infoKey->frmTaskManagerInst->lviProcess->Items->Count);
                                                     infoKey->frmTaskManagerInst->Show();
                                                     //frmMain->SendCommand(COMMAND_OK, "", sHandle);
                                                     frmMain->SendCommand(COMMAND_CPU_USAGE, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_SERVER_OPTIONS) == 0) {
                                                     if (infoKey->frmOptionsInst == NULL)
                                                                                        {
                                                                                        infoKey->frmOptionsInst = new TfrmServerOptions(Application);
                                                                                        if (infoKey->frmOptionsInst == NULL) return;
                                                                                        infoKey->frmOptionsInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ServerMain", "mnuOptions", "Host options"));
                                                                                        infoKey->frmOptionsInst->Tag = sHandle;
                                                                                        }
                                                     infoKey->frmOptionsInst->txtTemp->Clear();
                                                     tmpStr = SwitchToListFormat(sParams);
                                                     infoKey->frmOptionsInst->txtTemp->Text = tmpStr;
                                                     infoKey->frmOptionsInst->ReadSettings(infoKey->frmOptionsInst->txtTemp);
                                                     infoKey->frmOptionsInst->Show();
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_ERROR) == 0) {
                                                frmMain->TypeCommandLine(sHandle, UpperCase(ReadFromLanguage("General", "Error", "Error")) + ": " + sParams);
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_HELP) == 0) {
                                                frmMain->TypeCommandLine(sHandle, sParams);
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_HOST_INFORMATION) == 0) {
                                                       frmMain->TypeCommandLine(sHandle, sParams);
                                                       return; }
if (sCommand.AnsiCompareIC(COMMAND_HOST_VERSION) == 0) {
                                                       frmMain->TypeCommandLine(sHandle, sParams);
                                                       return; }
if (sCommand.AnsiCompareIC(COMMAND_CPU_USAGE) == 0) {
                                                     if (infoKey->frmTaskManagerInst == NULL)
                                                                                      {
                                                                                      infoKey->frmTaskManagerInst = new TfrmTaskManager(Application);
                                                                                      if (infoKey->frmTaskManagerInst == NULL) return; // Memory allocation error
                                                                                      infoKey->frmTaskManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuTaskManager", "Task manager"));
                                                                                      infoKey->frmTaskManagerInst->Tag = sHandle;
                                                                                      }
                                                     infoKey->frmTaskManagerInst->stbStatus->Panels->Items[1]->Text = ReadFromLanguage("SystemInfo", "msgProcesses", "CPU Usage: ") + Split2Params(sParams, SEPARATOR_SINGLE, 1);
                                                     infoKey->frmTaskManagerInst->stbStatus->Panels->Items[2]->Text = ReadFromLanguage("SystemInfo", "msgProcesses", "Mem Usage: ") + Split2Params(sParams, SEPARATOR_SINGLE, 2);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_CMD) == 0) {
                                                       frmMain->TypeCommandLine(sHandle, sParams);
                                                       return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_DRIVELIST) == 0) {
                                                         if (infoKey->frmFileManagerInst == NULL)
                                                                                                {
                                                                                                infoKey->frmFileManagerInst = new TfrmFileManager(Application);
                                                                                                if (infoKey->frmFileManagerInst == NULL) return;
                                                                                                infoKey->frmFileManagerInst->Caption = "[" + infoKey->sHost + "] " + DelHK(ReadFromLanguage("ClientMain", "mnuFileManager", "File manager"));
                                                                                                infoKey->frmFileManagerInst->Tag = sHandle;
                                                                                                }
                                                         frmMain->SplitToListView(infoKey->frmFileManagerInst->lviRemoteDrive, tmpStr);
                                                         return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_OK) == 0) {
                                                  if (infoKey->usFileName == "") return;
                                                  infoKey->usFileOp = 1;
                                                  infoKey->usFileSize = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 1), 0);
                                                  OpenFileSocket(StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 2), 4339), sHandle);
                                                  return; }
if (sCommand.AnsiCompareIC(COMMAND_OK) == 0)
                                           {
                                           if (infoKey->sRouter != "") {
                                                                     frmMain->SendCommand(COMMAND_ROUTER, infoKey->sRouter, sHandle);
                                                                     infoKey->sRouter = "";
                                                                     }

                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_PING) == 0) return;


}

void __fastcall TfrmMain::sckCommandRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
AnsiString newData;
AnsiString newCommand;
AnsiString newParams;
AnsiString tmpStr;
unsigned int iPos;
unsigned int iPos2;
unsigned int i;

try
{
newData = Socket->ReceiveText();
if (newData == "") return;
txtLog->Text = txtLog->Text + newData;
// Command parser:
do
    {
    iPos = AnsiPos(SEPARATOR_END, newData);
    tmpStr = newData.SubString(1, iPos - 1);

    if (AnsiPos(SEPARATOR_COMMAND, tmpStr) == 0)
                                     {
                                     newData = newData.SubString(iPos + 2, newData.Length() - iPos - 2);
                                     ProcessCommand(tmpStr, "", Socket->SocketHandle);
                                     continue;
                                     }

    for (i = 1; i < tmpStr.Length(); ++i)
                {
                if (tmpStr.SubString(i, 1) == SEPARATOR_COMMAND || tmpStr.SubString(i, 1) == SEPARATOR_SINGLE)
                                                                {
                                                                iPos2 = i;
                                                                newCommand = tmpStr.SubString(1, i - 1);
                                                                newParams = tmpStr.SubString(iPos2 + 1, tmpStr.Length() - iPos2);
                                                                newData = newData.SubString(iPos + 2, newData.Length() - iPos - 2);
                                                                ProcessCommand(newCommand, newParams, Socket->SocketHandle);
                                                                break;
                                                                }
                }
    }

while (newData.Length() > 2);
}
catch (Exception &exception)
{
        Socket->Close();
        return;
}




}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckCallbackClientConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
if (State >= 2)
              {
              Socket->Close();
              return;
              }

Connect();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::sckCallbackClientDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
Disconnect();        
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckCommandError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{

try
{
if (Socket->Connected) Socket->Close();

if (ErrorCode != 10061)
                      {
                      ConnectionCount--;
                      if (ConnectionCount <= 0) Disconnect();
                      }
}
catch (Exception &exception)
{
        return;
}

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuSimpleMouseClick(TObject *Sender)
{
if (frmSimpleMouseAndKey == NULL) {
                                  frmSimpleMouseAndKey = new TfrmRemoteControl(Application);
                                  if (frmSimpleMouseAndKey == NULL) return; // Memory allocation error...
                                  frmSimpleMouseAndKey->Tag = 0;
                                  frmSimpleMouseAndKey->tmrMouseMove->Tag = 1;
                                  }
frmSimpleMouseAndKey->Show();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuFullControlClick(TObject *Sender)
{
LoadNewWindow(4);
//SendCommand(COMMAND_SCREEN_VIEW, PARAMETER_START, 0);
SendCommand(COMMAND_SCREENSHOT, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuScreenSpyClick(TObject *Sender)
{
LoadNewWindow(5);
SendCommand(COMMAND_SCREEN_VIEW, PARAMETER_START, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuHostUsersClick(TObject *Sender)
{
LoadNewWindow(1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuChatClick(TObject *Sender)
{
LoadNewWindow(0);
}
//---------------------------------------------------------------------------

int __fastcall TfrmMain::SplitToListView(TListView * lviObject, const AnsiString pData)
{
unsigned short int i;
unsigned short int iPos;
unsigned short int wLimit;
AnsiString firstCaption;
AnsiString subCaption;


// TStringList created, use it to split the items:
TStringList * strList = new TStringList;
strList->Text = pData;


for (i = 0; i < strList->Count; ++i)
        {
        // Are there SubItems?
        iPos = AnsiPos(SEPARATOR_SPLIT, strList->Strings[i]);
        if (iPos > 0)
                    // Aha! SubItems found, because the single separator exists!
                    {
                    // Get the item...
                    firstCaption = strList->Strings[i].SubString(1, iPos - 1);
                    // ...and the SubItem:
                    subCaption = strList->Strings[i].SubString(iPos + 1, strList->Strings[i].Length() - iPos);

                    wLimit = 0;

                    // Replace the single separators with RETURN, so that the ListView will split them
                    // to the SubItems->Text:
                    do
                    {
                    // If the replace doesn't work, make sure we don't get stuck:
                    wLimit++;
                    subCaption = FullReplace(subCaption, SEPARATOR_SPLIT, "\r\n");
                    if (wLimit >= 100) break;
                    }
                    while (subCaption.AnsiPos(SEPARATOR_SPLIT) > 0);

                    // And then just add to list:
                    lviObject->AddItem(firstCaption, lviObject);
                    lviObject->Items->Item[lviObject->Items->Count - 1]->SubItems->Text = subCaption;
                    }
                    else
                    {
                    // No subitems, add the item:
                    firstCaption = strList->Strings[i];
                    subCaption = "";
                    if (firstCaption != "") lviObject->AddItem(firstCaption, lviObject);
                    }
        }


// Delete the TStringList and exit function:
strList->Free();
return 0;
}




void __fastcall TfrmMain::sckFileInstConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
Socket->SendText("R");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckFileInstDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
unsigned int sHandle;

if (Socket->Connected) Socket->Close();


TSocketInfo * infoKey = NULL;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

StopTransfer(sHandle);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckFileInstError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{

try
{
if (Socket->Connected) Socket->Close();

if (ErrorCode == 10061)
                      {
                      // File port not open at the server side...
                      }
}
catch (Exception &exception)
{
        return;
}

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::sckFileInstRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
unsigned char Buffer[4096];
unsigned int lenBuf = 4096;
unsigned int sHandle;
TSocketInfo * infoKey = NULL;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

if (infoKey->usFileHandle == NULL) return; // No upload/download in progress


try
{
// User is using screen view (Full control, Screen spy):
if (infoKey->usFileOp == 3)
                          {
                          lenBuf = Socket->ReceiveLength();
                          Socket->ReceiveBuf(Buffer, lenBuf);
                          if (infoKey->screenPart == NULL) {
                                                           //ShowScreenPart(sHandle, true);
                                                           return;
                                                           }

                          infoKey->screenPart->Write(Buffer, lenBuf);
                          infoKey->usFilePos += lenBuf;

                          if (infoKey->usFilePos >= infoKey->usFileSize - 1) {
                                                                             ShowScreenPart(sHandle, true);
                                                                             return;
                                                                             } else {
                                                                             ShowScreenPart(sHandle, false);
                                                                             }
                          Socket->SendText("R");
                          return;
                          }



// User is downloading a file:
if (infoKey->usFileOp == 1)
                          {
                          lenBuf = Socket->ReceiveLength();
                          Socket->ReceiveBuf(Buffer, lenBuf);
                          FileWrite(infoKey->usFileHandle, Buffer, lenBuf);
                          infoKey->usFilePos += lenBuf;
                          if (infoKey->usFilePos >= infoKey->usFileSize - 1)
                                                                           {
                                                                           StopTransfer(sHandle);
                                                                           return;
                                                                           }
                          Socket->SendText("R");
                          //Socket->ReceiveText(); 
                          return;
                          }


Socket->ReceiveText();

// User is uploading a file:
if (infoKey->usFilePos >= infoKey->usFileSize - 1)
                 {
                 StopTransfer(sHandle);
                 return;
                 }

//FileSeek(infoKey->usFileHandle, infoKey->usFilePos, 0);
if ((infoKey->usFilePos + lenBuf) > infoKey->usFileSize) lenBuf = infoKey->usFileSize - infoKey->usFilePos;
infoKey->usFilePos += lenBuf;
FileRead(infoKey->usFileHandle, Buffer, lenBuf);

if (!Socket->Connected)
                        {
                        StopTransfer(sHandle);
                        Socket->Close();
                        return;
                        }

Socket->SendBuf(Buffer, lenBuf);
}
catch (Exception &exception)
{
Socket->Close();
}

}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuScreenshotClick(TObject *Sender)
{
/*
frmWork->lviWork->AddItem("d:\\jee.zip", frmWork->lviWork);
frmWork->lviWork->Items->Item[frmWork->lviWork->Items->Count - 1]->SubItems->Text = "d:\\joo.zip\r\nGET\r\njoo\r\n0\r\n" + IntToStr(tmpHandle);
frmWork->ProcessNextCommand();
*/
SendCommand(COMMAND_SCREENSHOT, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuWebcamClick(TObject *Sender)
{
SendCommand(COMMAND_WEBCAMSHOT, "", 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuWorkClick(TObject *Sender)
{
frmWork->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuMouseListenClick(TObject *Sender)
{
// Start mouse listen on all connected computers.
// Send the timer interval as parameter:
SendCommand(COMMAND_MOUSELISTEN_START, IntToStr(50), 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::tmrPingTimer(TObject *Sender)
{
unsigned int sHandle;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
SendCommand(COMMAND_PING, "", sHandle);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuFileManagerClick(TObject *Sender)
{
LoadNewWindow(2);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuFileMngrClick(TObject *Sender)
{
//mnuFileManager->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuHostInfoClick(TObject *Sender)
{
//mnuHostInformation->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuRemoteControl2Click(TObject *Sender)
{
//mnuFullControl->Click();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::tmrProgressTimer(TObject *Sender)
{
unsigned int sHandle;
unsigned int KBs = 0;
unsigned int Remaining = 0;
unsigned short int h, m, s;
AnsiString hs, ms, ss;
AnsiString strRemaining;


sHandle = dynamic_cast<TComponent *>(Sender)->Tag;

TSocketInfo * infoKey = NULL;

infoKey = (TSocketInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   return;
                   }

if (infoKey->frmProgressInst == NULL)
                                     {
                                     infoKey->frmProgressInst = new TfrmProgress(Application);
                                     if (infoKey->frmProgressInst == NULL) return;
                                     infoKey->frmProgressInst->Tag = sHandle;
                                     infoKey->frmProgressInst->Show();
                                     }

if (infoKey->usFileOp <= 0 || infoKey->usFileOp > 2)
                          {
                          infoKey->tmrProgress->Enabled = false;
                          infoKey->frmProgressInst->Hide();
                          return;
                          }

if (infoKey->usFileOp == 1)
                          {
                          infoKey->frmProgressInst->Caption = ReadFromLanguage("Status", "sttReceiving", "Receiving...");
                          }
if (infoKey->usFileOp == 2)
                          {
                          infoKey->frmProgressInst->Caption = ReadFromLanguage("Status", "sttTransfering", "Transfering...");
                          }
KBs = infoKey->usFilePos - infoKey->usFileLastPos;
infoKey->usFileLastPos = infoKey->usFilePos;
if (KBs > 0) Remaining = (infoKey->usFileSize - infoKey->usFilePos) / KBs;


h = Remaining / 3600;
m = Remaining / 60%60;
s = Remaining % 60;

hs = IntToStr(h);
if (hs.Length() < 2) hs = "0" + hs;
ms = IntToStr(m);
if (ms.Length() < 2) ms = "0" + ms;
ss = IntToStr(s);
if (ss.Length() < 2) ss = "0" + ss;


if (h > 0) strRemaining += hs + " " + HourString + " ";
if (m > 0) strRemaining += ms + " " + MinuteString + " ";
strRemaining += ss + " " + SecondString + " ";
strRemaining += RemainingString;


infoKey->frmProgressInst->lblRemainingTime->Caption = strRemaining;

//if (KBs > 0) KBs = KBs / 1024;

infoKey->frmProgressInst->prgProgress->Position = (infoKey->usFilePos * 100) / infoKey->usFileSize;
//infoKey->frmProgressInst->lblSpeed->Caption = IntToStr(KBs) + " KB/Sec.";
infoKey->frmProgressInst->lblSpeed->Caption = TotalSize(KBs) + "/Sec.";
infoKey->frmProgressInst->lblFileName->Caption = infoKey->usFileName;

infoKey->frmProgressInst->Show();

}
//---------------------------------------------------------------------------




void __fastcall TfrmMain::mnuHostInformationClick(TObject *Sender)
{
SendCommand(COMMAND_HOST_INFORMATION, "", 0);
}
//---------------------------------------------------------------------------




void __fastcall TfrmMain::mnuFileClick(TObject *Sender)
{
CheckMenus();        
}
//---------------------------------------------------------------------------

