/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


/*

SServer
Part of the Specrem-project.

Server service.
Copyright (C) 2002  Antti Kirjavainen

*/


//---------------------------------------------------------------------------
#include <vcl.h>
#include "Main.h"
#include "Chat.h"
#include "AutoIt.h"
#include "..\Shared\Settings.cpp"
#include "..\Shared\Commands.h"
#include "..\Shared\ServerOptions.h"
#include "..\Shared\strHandle.cpp"
#include "..\Shared\hashmap.cpp"
#include "..\Shared\DriveHandle.cpp"
#include "..\Shared\GetFileInfo.cpp"
#include "Public.cpp"
#include "TUserInfo.h"
#include "SystemShutdown.cpp"
#include "CPU_Usage.cpp"
#include "CMDHook.cpp"
#include "StreamCompress.cpp"
#include "Systray.cpp"
#include "Shutdown.h"
#include "Uptime.cpp"
#include "WebCam.cpp"
#include "wild.cpp"
#include "Processes.cpp"
#include "ServerStr.cpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "trayicon"
#pragma link "AutoItDLL.lib"
#pragma resource "*.dfm"

TSServer7 *SServer7;
TfrmChat * frmChatInst = NULL;
TfrmShutdown * frmShutdownInst = NULL;

AnsiString AppPath;
AnsiString AdministratorPassword;
AnsiString GuestPassword;
AnsiString strAccept = "";
AnsiString strReject = "";

bool Callback;
bool GuestAccess;
bool TelnetAccess;
bool PingClients;
bool DoAccept;
bool DoReject;
bool Invisible;

unsigned short int MaxConnections = 20;
unsigned short int MaxConnectionsPerAddress = 3;
unsigned short int FilePort = 4339;
hashmap myDB;
enum TListType { FILELIST, WIDELIST, DRIVELIST, USERLIST, HOSTINFO };
enum TRoute { ROUTECONNECT, ROUTESEND };
TShutdownType SD;

TScrollBox * VideoBox = NULL;


unsigned short int CountAddress(const AnsiString cAddress);
void CheckScreen();
void KickClient(const unsigned int sHandle, const AnsiString Reason);
void KickAll();
void ExitServer();
int FindHandle(const int sHandle);
int FindAddress(const AnsiString sAddress);
void SendList(const TListType lstSend, const unsigned int sHandle, const AnsiString sPath);
void ShowHelp(unsigned short int sAccess, unsigned int sHandle);
void Route(const AnsiString rCommand, unsigned int rHandle, TRoute rWork);
void ProcessCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle);
void MonitorOff();
void StartScreenSaver();
string getFileInfoStr(FileInfo * ptrFileInfo);
int LoadSettings();
int ReadSettings();
int SaveSettings();
void CheckConnectedAddressesForBans();
void BanAddress(AnsiString bAddress);
void UnBanAddress(AnsiString bAddress);
bool AcceptAddress(const AnsiString sAddress);
void SendBuffer(const unsigned int sHandle, const unsigned int sLines);
int CaptureScreen(TMemoryStream * streamCapture);

//---------------------------------------------------------------------------
__fastcall TSServer7::TSServer7(TComponent* Owner)
	: TService(Owner)
{
}

TServiceController __fastcall TSServer7::GetServiceController(void)
{
	return (TServiceController) ServiceController;
}

void __stdcall ServiceController(unsigned CtrlCode)
{
	SServer7->Controller(CtrlCode);
}
//---------------------------------------------------------------------------
void CheckConnectedAddressesForBans()
{
unsigned short int s;

if (Callback)
            {
            return;
            }

if (SServer7->sckCommand->Socket->ActiveConnections == 0) return;

for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
        {
        if (!AcceptAddress(SServer7->sckCommand->Socket->Connections[s]->RemoteAddress))
                                                                                        {
                                                                                        KickClient(SServer7->sckCommand->Socket->Connections[s]->SocketHandle, ReadFromLanguage("Logging", "logBanKick", "% kicked (banned)") + ".");
                                                                                        }
        }

}

void BanAddress(AnsiString bAddress)
{
// I disabled the configuration file saving from here.
// Bans are made only for one session and kept in memory.

DoReject = true;
strReject += bAddress + "\r\n";

CheckConnectedAddressesForBans();

}

void UnBanAddress(AnsiString bAddress)
{
short int b;
TStringList * banList = new TStringList();

if (banList == NULL) return;
banList->Text = strReject;

b = banList->IndexOf(bAddress);
if (b < 0) return;

banList->Delete(b); 
strReject = banList->Text;
banList->Free();
}

int SaveSettings()
{
// Save and encrypt settings from the memo:
try
{
        //frmShutdown->txtTemp->Text = Encrypt(frmShutdown->txtTemp->Text);
        frmShutdown->txtTemp->Lines->SaveToFile(AppPath + "SServer.cfg");
}
catch (Exception &exception)
{
        return -1;
}

return 0;
}

int CaptureScreen(TMemoryStream * streamCapture)
{
  HDC dc = NULL;
  TRect rect;
  Graphics::TCanvas *ScreenCanvas = new Graphics::TCanvas;
  TImage * imgCapture = new TImage(SServer7);


  if (ScreenCanvas == NULL) return -1;
  if (streamCapture == NULL) return -1;
  if (imgCapture == NULL) return -1;

try
{

  dc = GetDC(0);
  ScreenCanvas->Handle = dc;

  imgCapture->Picture->Bitmap->Width = Screen->Width;
  imgCapture->Picture->Bitmap->Height= Screen->Height;
  rect = Rect(0,0,Screen->Width, Screen->Height);
  imgCapture->Picture->Bitmap->Canvas->CopyRect(rect, ScreenCanvas, rect);

  streamCapture->Clear();
  streamCapture->Position = 0;

  imgCapture->Picture->Bitmap->SaveToStream(streamCapture);

}
catch (Exception &exception)
{
        return -1;
}

  imgCapture->Free();
  delete ScreenCanvas;
  ReleaseDC(NULL, dc);

return 0;
}

void SendBuffer(const unsigned int sHandle, const unsigned int sLines)
{
TStringList * TList = NULL;
unsigned short int i;
AnsiString strList;
AnsiString sBuffer = "";
TUserInfo * infoKey = NULL;

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL) return;

if (infoKey->usBuffer == "") return;

TList = new TStringList();
if (TList == NULL) return; //Memory allocation error

TList->Text = infoKey->usBuffer;
infoKey->usBuffer = "";


// Add items to the list to be sent. The extra items will be added back
// to the buffer and sent later (when client sends OK-command):
for (i = 0; i < TList->Count; ++i)
        {
        if (i >= sLines)  {
                          sBuffer += "\r" + TList->Strings[i];
                          } else {
                          strList += "\r" + TList->Strings[i];
                          }
        }

TList->Free();

if (sBuffer != "") infoKey->usBuffer = sBuffer;

//AnsiString sA;
//sA = infoKey->usBufferCommand + strList;
//MessageBox(NULL, sA.c_str(), "", 0);
if (infoKey->usBufferCommand != "" && strList != "") SServer7->SendCommand(infoKey->usBufferCommand, strList, sHandle);
}

int LoadSettings()
{
// Load and unencrypt settings file to the memo:
try
{
        //Forms::Application->CreateForm(__classid(TfrmShutdown), &frmShutdown);
        frmShutdown->txtTemp->Clear();
        frmShutdown->txtTemp->Lines->LoadFromFile(AppPath + "SServer.cfg");
        //frmShutdown->txtTemp->Text = Decrypt(frmShutdown->txtTemp->Text);
        return 0;
}
catch (Exception &exception)
{
        return -1;
}
}

int ReadSettings()
{
// Read settings from the memo:

try
{
if (LoadSettings() == -1) return -1;

if (ConvertWordToBoolean(ReadSetting("Options", "TrayIcon", "Yes", frmShutdown->txtTemp))) {
                                                                                           Invisible = false;
                                                                                           TrayIcon(Forms::Application->Handle, "SServer7", 0);
                                                                                           SServer7->tmrRefreshTrayIcon->Enabled = true;
                                                                                           } else {
                                                                                           Invisible = true;
                                                                                           }

GuestAccess = ConvertWordToBoolean(ReadSetting("Options", "GuestAccess", "No", frmShutdown->txtTemp));
TelnetAccess = ConvertWordToBoolean(ReadSetting("Options", "Telnet", "Yes", frmShutdown->txtTemp));
PingClients = ConvertWordToBoolean(ReadSetting("Options", "Ping", "No", frmShutdown->txtTemp));
Callback = ConvertWordToBoolean(ReadSetting("Options", "Callback", "No", frmShutdown->txtTemp));
//Binding = ReadSetting("Options", "Bind", "", frmShutdown->txtTemp);
SServer7->sckCallback->Host = ReadSetting("Options", "CallbackAddress", "", frmShutdown->txtTemp);
SServer7->sckCallback->Port = StrToIntDef(ReadSetting("Options", "CallbackPort", "1187", frmShutdown->txtTemp), 1187);
SServer7->sckCommand->Port = StrToIntDef(ReadSetting("Options", "Port", "1187", frmShutdown->txtTemp), 1187);
MaxConnections = StrToIntDef(ReadSetting("Options", "MaxTotal", "20", frmShutdown->txtTemp), 20);
MaxConnectionsPerAddress = StrToIntDef(ReadSetting("Options", "MaxAddr", "3", frmShutdown->txtTemp), 3);
FilePort = StrToIntDef(ReadSetting("Options", "FilePort", "4339", frmShutdown->txtTemp), 4339);

AdministratorPassword = ReadSetting("Passwords", "Administrator", "", frmShutdown->txtTemp);
GuestPassword = ReadSetting("Passwords", "Guest", "", frmShutdown->txtTemp);

DoAccept = ConvertWordToBoolean(ReadSetting("Options", "Accept", "No", frmShutdown->txtTemp));
DoReject = ConvertWordToBoolean(ReadSetting("Options", "Reject", "No", frmShutdown->txtTemp));
if (DoAccept) strAccept = ReadSetting("Accept", "", "", frmShutdown->txtTemp);
if (DoReject) strReject = ReadSetting("Reject", "", "", frmShutdown->txtTemp);

frmShutdown->txtTemp->Clear();

return 0;
}
catch (Exception &exception)
{
        return -1;
}
}

void __fastcall TSServer7::ServiceStart(TService *Sender, bool &Started)
{
// *****************************************************************************
// *****************************************************************************
// *****************************************************************************
// ServiceStart
// *****************************************************************************
// *****************************************************************************
// Load settings, and in case of a failure, cancel start.
// Start listening with sckCommand (or connecting with Callback).
// *****************************************************************************
// *****************************************************************************
// *****************************************************************************

AnsiString tmpStr;


AppPath = ExtractFilePath(Forms::Application->ExeName);


try
{


if (ParamStr(1).AnsiCompareIC("/SETTINGS") == 0 || !FileExists(AppPath + "SServer.cfg"))
                                               {
                                               tmpStr = AppPath + "SConfig.exe";
                                               if (FileExists(tmpStr))
                                                                     {
                                                                     this->ErrCode = 4;
                                                                     ShellExecute(NULL, "open", tmpStr.c_str(), "", "", SW_SHOW);
                                                                     }
                                                                     else
                                                                     {
                                                                     this->ErrCode = 2;
                                                                     }
                                               Started = false;
                                               return;
                                               }



        if (ReadSettings() == -1)
                                {
                                this->ErrCode = 4;
                                Started = false;
                                return;
                                }
        if (Callback)
                    {
                    tmrCallback->Enabled = true;
                    }
                    else
                    {
                    sckCommand->Open();
                    if (FilePort > 0)
                                    {
                                    SServer7->sckFile->Port = FilePort;
                                    SServer7->sckFile->Open();
                                    }
                    }

}
catch (Exception &exception)
{
        this->ErrCode = 10048;
        Started = false;
        return;
}



Started = true;
}
//---------------------------------------------------------------------------

void ExitServer()
{
if (!Invisible) TrayIcon(Forms::Application->Handle, "SServer7", 2);
KickAll();
if (frmChatInst != NULL)
                        {
                        frmChatInst->Free();
                        frmChatInst = NULL;
                        }
frmShutdown->Free();
if (frmShutdownInst != NULL)
                           {
                           frmShutdownInst->Free();
                           frmShutdownInst = NULL;
                           }

SServer7->sckFile->Close();
SServer7->sckCommand->Close();
//SServer7->sckFile->Free();
//SServer7->sckCommand->Free();
}

void __fastcall TSServer7::ServiceStop(TService *Sender, bool &Stopped)
{
Stopped = true;
SServer7->Free();
ExitServer();
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::SendCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle)
{
// *****************************************************************************
// *****************************************************************************
// SendCommand
// *****************************************************************************
// Encrypt the command and send it.
// *****************************************************************************
// *****************************************************************************

unsigned int h;
AnsiString FullCommand;
TUserInfo * infoKey = NULL;
AnsiString tmpStr;


if (SServer7->sckCommand->Socket->ActiveConnections == 0 && !Callback) return;

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   KickClient(sHandle, "");
                   return;
                   }

tmpStr = infoKey->GetKey();
FullCommand = sCommand;
if (sParams != "") FullCommand += SEPARATOR_COMMAND + sParams;
FullCommand += SEPARATOR_END;


if (tmpStr != NULL)
                  {
                  // FullCommand = Encrypt(FullCommand, tmpStr);
                  // Encryption code here:
                  }


h = FindHandle(sHandle);

if (h != -1)
           {
           if (Callback)
                        {
                        SServer7->sckCallback->Socket->SendText(FullCommand);
                        }
                        else
                        {
                        SServer7->sckCommand->Socket->Connections[h]->SendText(FullCommand);
                        }
           }

}

int FindHandle(const int sHandle)
{
// Find index by handle:

unsigned short int s;
short int r;

r = -1;
if (Callback)
            {
            return 1;
            }

if (SServer7->sckCommand->Socket->ActiveConnections == 0) return -1;

for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
        {
        if (SServer7->sckCommand->Socket->Connections[s]->SocketHandle == sHandle)
                {
                r = s;
                }
        }


return r;
}

int FindAddress(const AnsiString sAddress)
{
// Find handle by address:

unsigned short int s;

if (Callback) return 1;
if (SServer7->sckCommand->Socket->ActiveConnections == 0) return -1;

for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
        {
        if (SServer7->sckCommand->Socket->Connections[s]->RemoteAddress == sAddress)
                {
                return SServer7->sckCommand->Socket->Connections[s]->SocketHandle;
                }
        }


return -1;
}

void StartScreenSaver()
{
void * lDesktopWindow;

// Get the desktop window handle:
lDesktopWindow = GetDesktopWindow();
if (lDesktopWindow == NULL) return;

// ...and send a message to start the screensaver:
SendMessage(lDesktopWindow, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
}

void MonitorOff()
{
void * lDesktopWindow;

// Get the desktop window handle:
lDesktopWindow = GetDesktopWindow();
if (lDesktopWindow == NULL) return;

// ...and send a message to turn of the monitor:
SendMessage(lDesktopWindow, WM_SYSCOMMAND, SC_MONITORPOWER, 1);
}

void CheckScreen()
{
// *****************************************************************************
// *****************************************************************************
// CheckScreen
// *****************************************************************************
// Has something in the screen changed?
// If so, send the new part of screen to clients.
// *****************************************************************************
// *****************************************************************************

unsigned short int s;
unsigned int sHandle;
bool ViewersFound;
TMemoryStream * scrNew = new TMemoryStream();
TMemoryStream * scrOld = new TMemoryStream();
TUserInfo * infoKey = NULL;

unsigned short int X1 = 1;
unsigned short int X2 = 1;
unsigned short int Y1 = 0;
unsigned short int Y2 = 0;

if (Callback)
            {
            return;
            }

if (SServer7->sckCommand->Socket->ActiveConnections == 0) return;

return;
do
{

scrNew->Clear();


//scrNew->Write()


if (StreamChanged(scrNew, scrOld))
                                 {
                                  CompressStream(scrNew, 10, true);
                                  scrOld->Clear();
                                  scrOld->Position = 0;
                                  scrNew->Position = 0;
                                  scrOld->CopyFrom(scrNew, scrNew->Size);
                                  for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
                                           {
                                                sHandle = SServer7->sckCommand->Socket->Connections[s]->SocketHandle;
                                                infoKey = (TUserInfo *) myDB.getValue(sHandle);
                                                if (infoKey == NULL) continue;
                                                if (infoKey->usScreenView && infoKey->usFileHandle == NULL)
                                                                         {
                                                                         ViewersFound = true;
                                                                         if (infoKey->screenPart == NULL) infoKey->screenPart = new TMemoryStream();
                                                                         if (infoKey->screenPart == NULL) continue; // Could not create new stream...
                                                                         infoKey->screenPart->Clear();
                                                                         infoKey->screenPart->Position = 0;
                                                                         scrNew->Position = 0;
                                                                         infoKey->screenPart->CopyFrom(scrNew, scrNew->Size);
                                                                         infoKey->usFileOp = 3;
                                                                         infoKey->usFilePos = 0;
                                                                         infoKey->usFileSize = scrNew->Size;
                                                                         infoKey->usFileHandle = 3;

                                                                         // X1, Y1, X2, Y2, File size, Data port, Type:
                                                                         SServer7->SendCommand(COMMAND_SCREEN_PART, IntToStr(X1) + SEPARATOR_SPLIT + IntToStr(Y1) + SEPARATOR_SPLIT +
                                                                                                IntToStr(X2) + SEPARATOR_SPLIT + IntToStr(Y2) + SEPARATOR_SPLIT + IntToStr(infoKey->usFileSize) +
                                                                                                SEPARATOR_SPLIT + IntToStr(FilePort) + SEPARATOR_SPLIT + IntToStr(1), sHandle);
                                                                         }
                                           }
                                 }

if (!ViewersFound) return;

}
while (SServer7->sckCommand->Socket->ActiveConnections > 0);

}

void __fastcall TSServer7::SendToAll(const AnsiString sCommand, const AnsiString sParams, const unsigned int sNotToHandle)
{
// *****************************************************************************
// *****************************************************************************
// SendToAll
// *****************************************************************************
// Send commands to all clients, except for sNotToHandle.
// Usually this is the client that sent the command in the first place, so do
// not loopback it.
// *****************************************************************************
// *****************************************************************************

unsigned short int s;
unsigned int sHandle;
TUserInfo * infoKey = NULL;

if (Callback)
            {
            SendCommand(sCommand, sParams, 1);
            return;
            }

if (sckCommand->Socket->ActiveConnections == 0) return;

for (s = 0; s < sckCommand->Socket->ActiveConnections; ++s)
        {
        sHandle = sckCommand->Socket->Connections[s]->SocketHandle;
        if (sHandle != sNotToHandle)
                                   {
                                   infoKey = (TUserInfo *) myDB.getValue(sHandle);
                                   if (infoKey == NULL) continue;
                                   if (infoKey->GetAccess() >= 1) SendCommand(sCommand, sParams, sHandle);
                                   }
        }
}

void KickClient(const unsigned int sHandle, const AnsiString Reason)
{
// *****************************************************************************
// *****************************************************************************
// KickClient
// *****************************************************************************
// Kick a client with or without a reason.
// *****************************************************************************
// *****************************************************************************

int tmpInt;
AnsiString tmpStr;
TUserInfo * infoKey = NULL;

infoKey = (TUserInfo *) myDB.getValue(sHandle);
tmpInt = FindHandle(sHandle);

if (infoKey == NULL)
                    {
                    if (Reason != "")
                                    {
                                    tmpStr = FullReplace(Reason, "%", SServer7->sckCommand->Socket->Connections[tmpInt]->RemoteAddress);
                                    SServer7->SendToAll(COMMAND_CHAT, tmpStr, sHandle);
                                    if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                    if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";
                                    }
                    }
                    else
                    {
                    if (Reason != "")
                                    {
                                    tmpStr = FullReplace(Reason, "%", infoKey->GetNick());
                                    SServer7->SendToAll(COMMAND_CHAT, tmpStr, sHandle);
                                    if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                    if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";
                                    }

                    if (infoKey->UsingWebcam) CloseCamera();
                    delete infoKey;
                    myDB.removeValue(sHandle);
                    }




if (Callback)
            {
            SServer7->sckCallback->Close();
            return;
            }

if (tmpInt > -1)
                {
                if (SServer7->sckCommand->Socket->Connections[tmpInt]->Connected) SServer7->sckCommand->Socket->Connections[tmpInt]->Close();
                }


}

void SendList(const TListType lstSend, const unsigned int sHandle, const AnsiString sPath)
{
// *****************************************************************************
// *****************************************************************************
// To keep the code clean, the lists are created here.
// *****************************************************************************
// *****************************************************************************
AnsiString strList;
unsigned short int i = 0;
unsigned short int c = 0;
TUserInfo * infoKey = NULL;
String tmpStr;
int tmpInt;
string msg;


if (lstSend == FILELIST) {
                         msg = GetFolderList(sPath, false);
                         if (msg == "-1")  {
                                           SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "DriveNotReady", "Drive not ready") + "!", sHandle);
                                           return;
                                           }

                         strList = SEPARATOR_SINGLE + AnsiString(msg.c_str());
                         SServer7->SendCommand(COMMAND_FILE_LIST, strList, sHandle);
                         return;
                       	 }
if (lstSend == WIDELIST) {
                         msg = GetFolderList(sPath, true);
                         if (msg == "-1")  {
                                                                  SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "DriveNotReady", "Drive not ready") + "!", sHandle);
                                                                  return;
                                                                  }
                         infoKey = (TUserInfo *) myDB.getValue(sHandle);
                         if (infoKey == NULL) return;

                         strList = SEPARATOR_SINGLE + AnsiString(msg.c_str());
                         infoKey->usBuffer = strList;
                         infoKey->usBufferCommand = COMMAND_FILE_LIST_WIDE;

                         SendBuffer(sHandle, 50);
                         return;
                       	 }
if (lstSend == DRIVELIST) {
                          tmpStr = SEPARATOR_SINGLE + GetDrives();
                          if (tmpStr == NULL)  {
                                                SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "DriveNotReady", "Drive not ready") + "!", sHandle);
                                                return;
                                                }
                          SServer7->SendCommand(COMMAND_FILE_DRIVELIST, tmpStr, sHandle);
                          return;
                          }


if (lstSend == HOSTINFO) {
                         strList = SEPARATOR_SINGLE;
                         strList += "Specrem Server " + VERSION + SEPARATOR_SINGLE;
                         strList += ReadFromLanguage("SystemInfo", "msgUptime", "Uptime") + " " + Uptime() + SEPARATOR_SINGLE;
                         strList += ReadFromLanguage("SystemInfo", "msgCPU", "CPU") + " " + IntToStr(GetCPUUsage()) + " %" + SEPARATOR_SINGLE;
                         SServer7->SendCommand(COMMAND_HOST_INFORMATION, strList, sHandle);
                         return;
                         }



if (lstSend == USERLIST) {
                        for (i = 0; i < SServer7->sckCommand->Socket->ActiveConnections; ++i)
                                        {
                                        infoKey = (TUserInfo *) myDB.getValue(SServer7->sckCommand->Socket->Connections[i]->SocketHandle);
                                        if (infoKey == NULL)
                                                           {
                                                           KickClient(sHandle, "");
                                                           return;
                                                           }
                                        if (infoKey->GetAccess() == 0) tmpStr = "Connecting...";
                                        if (infoKey->GetAccess() == 1) tmpStr = "Guest";
                                        if (infoKey->GetAccess() == 2) tmpStr = "Administrator";

                                        strList = strList +
                                                  SServer7->sckCommand->Socket->Connections[i]->RemoteAddress +
                                                  SEPARATOR_SPLIT +
                                                  infoKey->GetNick() +
                                                  SEPARATOR_SPLIT +
                                                  tmpStr +
                                                  SEPARATOR_SINGLE;
                                        }
                        strList = SEPARATOR_SINGLE + strList;
                        SServer7->SendCommand(COMMAND_USERS_LIST, strList, sHandle);
                        return;
                        }




}


//---------------------------------------------------------------------------

void __fastcall TSServer7::sckCommandClientConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
// *****************************************************************************
// *****************************************************************************
//  Client connect & accept
// *****************************************************************************
//  Check if connection limits are full, create the class instance to the list
//  and load the needed objects for the client. 
// *****************************************************************************
// *****************************************************************************


if (sckCommand->Socket->ActiveConnections > MaxConnections)
                                              {
                                              Socket->Close();
                                              return;
                                              }

if (CountAddress(Socket->RemoteAddress) > MaxConnectionsPerAddress)
                                                                   {
                                                                   Socket->Close();
                                                                   return;
                                                                   }


// Check, if the address is in the reject list, OR if ONLY certain addresses
// are in the accept-list, does this address exist there?
if (!AcceptAddress(Socket->RemoteAddress))
                                         {
                                         Socket->Close();
                                         return;
                                         }

/*
// Join flood is disabled and would not be very useful...

strJoinFlood->Add(Socket->RemoteAddress);
if (CountStringListItems(strJoinFlood, Socket->RemoteAddress) > 8)
                                                                 {
                                                                 BanAddress(Socket->RemoteAddress);
                                                                 Socket->Close();
                                                                 return;
                                                                 }
*/
TUserInfo * lstUser = new TUserInfo();
if (lstUser == NULL)
                   {
                   KickClient(Socket->SocketHandle, "");
                   return;
                   }


                   lstUser->SetAddress(Socket->RemoteAddress);
lstUser->SetHandle(Socket->SocketHandle);
lstUser->SetNick(Socket->RemoteAddress);
lstUser->ClearCommand();

lstUser->SetEncryptionKey(NULL); 

lstUser->usLogin = new TTimer(SServer7);
lstUser->usFlood = new TTimer(SServer7);
lstUser->usPing = new TTimer(SServer7);
if (lstUser->usLogin == NULL || lstUser->usLogin == NULL || lstUser->usPing == NULL )
                   {
                   KickClient(Socket->SocketHandle, "");
                   return;
                   }

// Login timeoout 1 minute, flood buffer clear 30 seconds, ping timeout 10 minutes:
lstUser->usLogin->OnTimer = tmrLoginTimer;
lstUser->usFlood->OnTimer = tmrFloodTimer;
lstUser->usPing->OnTimer = tmrPingTimer;
lstUser->usLogin->Interval = 60000;
lstUser->usFlood->Interval = 30000;
lstUser->usPing->Interval = 600000;
lstUser->usLogin->Tag = Socket->SocketHandle;
lstUser->usFlood->Tag = Socket->SocketHandle;
lstUser->usPing->Tag = Socket->SocketHandle;
lstUser->usLogin->Enabled = true;
lstUser->usFlood->Enabled = false;
lstUser->usPing->Enabled = true;

// Save the class to the list:
myDB.add(Socket->SocketHandle, lstUser);


SendCommand(COMMAND_PASSWORD, "", Socket->SocketHandle);
}
//---------------------------------------------------------------------------


unsigned short int CountAddress(const AnsiString cAddress)
{
// How many connections from an address?

unsigned short int s;
unsigned short int r;

r = 0;

for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
        {
        if (SServer7->sckCommand->Socket->Connections[s]->RemoteAddress == cAddress) { r++; }
        }


return r;
}

//---------------------------------------------------------------------------

void ShowHelp(unsigned short int sAccess, unsigned int sHandle)
{
// *****************************************************************************
// *****************************************************************************
// Send command list to the client
// *****************************************************************************
// *****************************************************************************


AnsiString strHelp;
AnsiString strSeparator;

strSeparator = SEPARATOR_SINGLE;


strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "Basic commands:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_HELP + strSeparator;
strHelp += COMMAND_CHAT + " chat_text" + strSeparator;
strHelp += COMMAND_EXIT + strSeparator;
strHelp += COMMAND_MESSAGE + " message" + strSeparator;
strHelp += COMMAND_MESSAGE_USER + " user" + SEPARATOR_SPLIT + "message" + strSeparator;
strHelp += COMMAND_HOST_VERSION + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

if (sAccess < 2) return;


strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "Remote control:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_CMD + " command" + strSeparator;
strHelp += COMMAND_DISABLE_CONTROL + strSeparator;
strHelp += COMMAND_ENABLE_CONTROL + strSeparator;
strHelp += COMMAND_TEXT + " text" + strSeparator;
strHelp += COMMAND_SERVER_OPTIONS + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "User control:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_USERS_BAN + " address" + strSeparator;
strHelp += COMMAND_USERS_BANS + strSeparator;
strHelp += COMMAND_USERS_CLEARBANS + strSeparator;
strHelp += COMMAND_USERS_KICK + " address" + strSeparator;
strHelp += COMMAND_USERS_KICKALL + strSeparator;
strHelp += COMMAND_USERS_LIST + strSeparator;
strHelp += COMMAND_USERS_UNBAN + " address" + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "Window control:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_WINDOW_ALWAYSONTOP + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_FLASH + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_FOCUS + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_FREEZE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_HIDE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_LIST + " " + PARAMETER_VISIBLE + "/" + PARAMETER_INVISIBLE + strSeparator;
strHelp += COMMAND_WINDOW_MAXIMIZE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_MINIMIZE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_MOVE + " window_caption" + SEPARATOR_SPLIT + "new_x" + SEPARATOR_SPLIT + "new_y" + strSeparator;
strHelp += COMMAND_WINDOW_NOTALWAYSONTOP + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_RENAME + " window_caption" + SEPARATOR_SPLIT + "new_caption" + strSeparator;
strHelp += COMMAND_WINDOW_SHOW + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_TERMINATE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_UNFREEZE + " window_caption" + strSeparator;
strHelp += COMMAND_PROCESS_KILL + " process_PID" + strSeparator;
strHelp += COMMAND_PROCESS_LIST + strSeparator;
strHelp += COMMAND_PROCESS_PRIORITY + " process_PID" + SEPARATOR_SPLIT + PARAMETER_REALTIME + "/" + PARAMETER_HIGH + "/" + PARAMETER_NORMAL + "/" + PARAMETER_IDLE + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "File control:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_FOLDER_CHANGEDIR + " new_folder" + strSeparator;
strHelp += COMMAND_FOLDER_CREATE + " new_folder" + strSeparator;
strHelp += COMMAND_FILE_COPY + "file" + SEPARATOR_SPLIT + "file" + strSeparator;
strHelp += COMMAND_FILE_DELETE + " name" + SEPARATOR_SPLIT + "new_name" + strSeparator;
strHelp += COMMAND_FOLDER_DELETE + " folder" + strSeparator;
strHelp += COMMAND_FILE_DRIVELIST + strSeparator;
strHelp += COMMAND_FILE_EXECUTE + " file" + SEPARATOR_SPLIT + PARAMETER_INVISIBLE + "/" + PARAMETER_VISIBLE + "/" + PARAMETER_MINIMIZED + "/" + PARAMETER_MAXIMIZED + SEPARATOR_SPLIT + "parameters" + strSeparator;
strHelp += COMMAND_FILE_FIND + " file_prefix" + SEPARATOR_SPLIT + "folder" + strSeparator;
strHelp += COMMAND_FILE_LIST + strSeparator;
strHelp += COMMAND_FILE_MOVE + " name" + SEPARATOR_SPLIT + "new_name" + strSeparator;
strHelp += COMMAND_FILE_RENAME + " name" + SEPARATOR_SPLIT + "new_name" + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "Spy:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_HOST_INFORMATION + strSeparator;
strHelp += COMMAND_KEYBOARDLISTEN_START + strSeparator;
strHelp += COMMAND_KEYBOARDLISTEN_STOP + strSeparator;
strHelp += COMMAND_MOUSELISTEN_START + strSeparator;
strHelp += COMMAND_MOUSELISTEN_STOP + strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

strHelp = "";
strHelp += "-----------------------" + strSeparator;
strHelp += "Shutdown:" + strSeparator;
strHelp += "-------------------------" + strSeparator;
strHelp += COMMAND_SHUTDOWN_FORCE + strSeparator;
strHelp += COMMAND_SHUTDOWN_LOCK + strSeparator;
strHelp += COMMAND_SHUTDOWN_LOGOFF + strSeparator;
strHelp += COMMAND_SHUTDOWN_REBOOT + strSeparator;
strHelp += COMMAND_SHUTDOWN_SHUTDOWN + strSeparator;
strHelp += COMMAND_SHUTDOWN_MONITOR + strSeparator;
strHelp += COMMAND_SHUTDOWN_SCREENSAVER + strSeparator;
strHelp += COMMAND_SHUTDOWN_SERVER_CLOSE + strSeparator;
strHelp += COMMAND_SHUTDOWN_SERVER_RESTART + strSeparator;
strHelp += COMMAND_SHUTDOWN_STOP + strSeparator;
strHelp += strSeparator;
SServer7->SendCommand(COMMAND_HELP, strHelp, sHandle);

return;
}

void Route(const AnsiString rCommand, unsigned int rHandle, TRoute rWork)
{
// *****************************************************************************
// *****************************************************************************
// Route
// *****************************************************************************
// Work as a router. Send commands and make new connections.
// *****************************************************************************
// *****************************************************************************


TUserInfo * infoKey = NULL;
infoKey = (TUserInfo *) myDB.getValue(rHandle);
AnsiString tmpStr;

if (rWork == ROUTESEND) {
                        if (infoKey->usRouter->Socket->Connected) infoKey->usRouter->Socket->SendText(rCommand);
                        return; }

if (rWork == ROUTECONNECT) {
                         // Avoid loading another router socket for the same client:
                         if (infoKey->usRouter != NULL) return;

                         tmpStr = Split2Params(rCommand, ":", 1);

                         // No connection to localhost allowed:
                         if (tmpStr.SubString(1, 3) == "127") return;
                         if (tmpStr.AnsiCompareIC("localhost") == 0) return;
                         if (tmpStr.AnsiCompareIC(SServer7->sckCommand->Socket->LocalAddress) == 0) return;
                         if (tmpStr.AnsiCompareIC(SServer7->sckCommand->Socket->LocalHost) == 0) return;

                         // Create the router socket:
                         infoKey->usRouter = new TClientSocket(SServer7);

                         // If failed, kick the client off:
                         if (infoKey->usRouter == NULL)
                                                      {
                                                      KickClient(rHandle, "");
                                                      return;
                                                      }
                         // Connect to the new host:
                         infoKey->usRouter->Host = tmpStr;
                         infoKey->usRouter->Port = StrToIntDef(Split2Params(rCommand, ":", 2), 1187);
                         infoKey->usRouter->OnConnect = SServer7->sckRouteConnect;
                         infoKey->usRouter->OnDisconnect = SServer7->sckRouteDisconnect;
                         infoKey->usRouter->OnError = SServer7->sckRouteError;
                         infoKey->usRouter->OnRead = SServer7->sckRouteRead;
                         infoKey->usRouter->Tag = rHandle;
                         infoKey->usRouter->Open();
                         return; }

}

AnsiString GetFullFoldername(AnsiString dPath, AnsiString dFolderName)
{
if (DirectoryExists(dPath + "\\" + dFolderName)) return dPath + dFolderName;
if (DirectoryExists(dFolderName)) return dFolderName;
return NULL;
}

AnsiString GetFullFilename(AnsiString dPath, AnsiString dFileName)
{
if (FileExists(dPath + dFileName)) return dPath + dFileName;
if (FileExists(dFileName)) return dFileName;
return NULL;
}

void KickAll()
{
// Kick all connections...

unsigned short int s;

for (s = 0; s < SServer7->sckCommand->Socket->ActiveConnections; ++s)
        {
        KickClient(SServer7->sckCommand->Socket->Connections[s]->SocketHandle, "");
        }
}

AnsiString SplitLine(const AnsiString sText)
{
return FullReplace(sText, SEPARATOR_SPLIT, "\r");
}

void ProcessCommand(const AnsiString sCommand, const AnsiString sParams, const unsigned int sHandle)
{
// *****************************************************************************
// *****************************************************************************
// ProcessCommand
// *****************************************************************************
// Here the commands are handled. Damn, neither C++ nor Delphi support CASE
// with String-type variables, like VB did... well, IF does the job:
// *****************************************************************************
// *****************************************************************************


AnsiString newCommand;
TUserInfo * infoKey = NULL;
AnsiString tmpStr;
AnsiString tmpStr2;
AnsiString tmpStr3;
AnsiString tmpStr4;
TDateTime tmpDateTime;
int tmpInt;
int tmpInt2;
TStringList * strLines = NULL;

try
{

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   KickClient(sHandle, "");
                   return;
                   }



// If not logged in, check for password and kick if invalid:
if (infoKey->GetAccess() == 0)
                             {
                             if (sCommand.AnsiCompareIC(COMMAND_ENCRYPTION_KEY) == 0) { if (infoKey->GetKey() != NULL) { KickClient(sHandle, ReadFromLanguage("Logging", "logKickLoginError", "% kicked (login error, hacking?)") + "."); return; } else { tmpStr = CreateEncryptionKey(); infoKey->SetEncryptionKey(tmpStr); SServer7->SendCommand(COMMAND_ENCRYPTION_KEY, tmpStr, sHandle); return; } }
                             if (!TelnetAccess) {
                                              if (infoKey->GetKey() == NULL) {
                                                                             KickClient(sHandle, "");
                                                                             return; }
                                                }
                             if (sCommand.AnsiCompare(AdministratorPassword) == 0) {
                                                                           infoKey->SetAccess(2);
                                                                           infoKey->usLogin->Enabled = false;
                                                                           // Disable flood protection for administrators:
                                                                           infoKey->usFlood->Enabled = false;
                                                                           infoKey->usFloodAmount = 0;
                                                                           SServer7->SendCommand(COMMAND_OK, PARAMETER_PASSWORD_ACCEPTED, sHandle);
                                                                           tmpStr = FullReplace(ReadFromLanguage("Logging", "logJoined", "% has joined as"), "%", infoKey->GetAddress()) + " " + infoKey->GetNick() + ".";
                                                                           SServer7->SendToAll(COMMAND_CHAT, tmpStr, sHandle);
                                                                           if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                                                           if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";
                                                                           return; }
                             if (!GuestAccess) { KickClient(sHandle, ""); return; }
                             if (sCommand.AnsiCompare(GuestPassword) == 0) {
                                                                           infoKey->SetAccess(1);
                                                                           infoKey->usLogin->Enabled = false;
                                                                           // Enable flood protection:
                                                                           infoKey->usFlood->Enabled = true;
                                                                           SServer7->SendCommand(COMMAND_OK, PARAMETER_PASSWORD_ACCEPTED, sHandle);
                                                                           tmpStr = FullReplace(ReadFromLanguage("Logging", "logJoined", "% has joined as"), "%", infoKey->GetAddress()) + " " + infoKey->GetNick() + ".";
                                                                           SServer7->SendToAll(COMMAND_CHAT, tmpStr, sHandle);
                                                                           if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                                                           if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";
                                                                           return; }
                             KickClient(sHandle, "");
                             return;
                             }


if (sCommand == "") return;


// Restricted user (guest) can perform the following:
if (sCommand.AnsiCompareIC(COMMAND_CHAT) == 0) {
                                                tmpStr = infoKey->GetNick() + ": " + sParams;
                                                SServer7->SendToAll(COMMAND_CHAT, tmpStr, 0);
                                                if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                                if (frmChatInst != NULL) {
                                                                         frmChatInst->Caption = "[" + ReadFromLanguage("General", "Server", "Server") + "] " + DelHK(ReadFromLanguage("Chat", "mnuChat", "Chat"));
                                                                         frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";
                                                                         frmChatInst->Show();
                                                                         }
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_EXIT) == 0) {
                                                KickClient(sHandle, ReadFromLanguage("Logging", "logLeft", "% has left") + ".");
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_HELP) == 0) {
                                                ShowHelp(infoKey->GetAccess(), sHandle);
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_HOST_VERSION) == 0) {
                                                        SServer7->SendCommand(COMMAND_HOST_VERSION, "Specrem Server " + VERSION, sHandle);
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_MESSAGE) == 0) {
                                                  tmpStr = FullReplace(ReadFromLanguage("General", "msgMessageFrom", "Message from %"), "%", infoKey->GetNick()) + ", " + DateToStr(tmpDateTime.CurrentDate()) + " " + TimeToStr(tmpDateTime.CurrentTime());
                                                  SServer7->SendToAll(COMMAND_MESSAGE, tmpStr + SEPARATOR_SPLIT + sParams, sHandle);
                                                  tmpStr2 = ReadFromLanguage("InputBox", "msgSendMessage", "Message:");
                                                  tmpStr3 = tmpStr + "\r\n\r\n" + sParams.c_str();
                                                  SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                  MessageBox(NULL, tmpStr3.c_str(), tmpStr2.c_str(), 0);
                                                  return; }
if (sCommand.AnsiCompareIC(COMMAND_MESSAGE_USER) == 0) {
                                                        tmpStr = FullReplace(ReadFromLanguage("General", "msgPrivateMessageFrom", "Private message from %"), "%", infoKey->GetNick()) + ", " + DateToStr(tmpDateTime.CurrentDate()) + " " + TimeToStr(tmpDateTime.CurrentTime());
                                                        tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                        tmpStr3 = tmpStr + "\r\n\r\n" + Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                        tmpStr4 = ReadFromLanguage("InputBox", "msgSendMessage", "Message:");
                                                        if (tmpStr2 == "Server")
                                                                                           {
                                                                                           MessageBox(NULL, tmpStr3.c_str(), tmpStr4.c_str(), 0);
                                                                                           return;
                                                                                           }
                                                        tmpInt = FindAddress(Split2Params(sParams, SEPARATOR_SPLIT, 1));
                                                        if (tmpInt > -1) {
                                                                         SServer7->SendCommand(COMMAND_MESSAGE, tmpStr + SEPARATOR_SPLIT + Split2Params(sParams, SEPARATOR_SPLIT, 2), tmpInt);
                                                                         SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                         } else {
                                                                         SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "SendMessage", "Message send failed") + ".", sHandle);
                                                                         };
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_NICK) == 0) {
                                                tmpStr = CheckNick(sParams);
                                                if (tmpStr == NULL) {
                                                                    SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Chat", "msgInvalidNick", "Invalid nick") + ".", sHandle);
                                                                    return; }
                                                tmpStr2 = FullReplace(ReadFromLanguage("Logging", "logNewNick", "% is now known as"), "%", infoKey->GetNick()) + " " + sParams + ".";
                                                SServer7->SendToAll(COMMAND_CHAT, tmpStr2, 0);
                                                if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
                                                if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr2 + "\r\n";
                                                infoKey->SetNick(sParams);
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_LIST) == 0) {
                                                     SendList(USERLIST, sHandle, "");
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_OK) == 0) { SendBuffer(sHandle, 50); return; }
if (sCommand.AnsiCompareIC(COMMAND_PING) == 0) { return; }


if (infoKey->GetAccess() < 2)
                             {
                             SServer7->SendCommand(COMMAND_ERROR, PARAMETER_INVALID_COMMAND, sHandle);
                             return;
                             }




// *****************************************************************************
// *****************************************************************************
// And now, the admin priviledges:
// *****************************************************************************
// *****************************************************************************
if (sCommand.AnsiCompareIC(COMMAND_REMOTE_MOUSE_MOVE) == 0) {
                                                            tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                            tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                            SetCursorPos(StrToIntDef(tmpStr, 0), StrToIntDef(tmpStr2, 0));
                                                            SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                            return; }
if (sCommand.AnsiCompareIC(COMMAND_REMOTE_MOUSE_DOWN) == 0) {
                                                            if (sParams.AnsiCompareIC(PARAMETER_LEFT) == 0) mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0); 
                                                            if (sParams.AnsiCompareIC(PARAMETER_MIDDLE) == 0) mouse_event(MOUSEEVENTF_MIDDLEDOWN, 0, 0, 0, 0);
                                                            if (sParams.AnsiCompareIC(PARAMETER_RIGHT) == 0) mouse_event(MOUSEEVENTF_RIGHTDOWN, 0, 0, 0, 0);
                                                            SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                            return; }
if (sCommand.AnsiCompareIC(COMMAND_REMOTE_MOUSE_UP) == 0) {
                                                            if (sParams.AnsiCompareIC(PARAMETER_LEFT) == 0) mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
                                                            if (sParams.AnsiCompareIC(PARAMETER_MIDDLE) == 0) mouse_event(MOUSEEVENTF_MIDDLEUP, 0, 0, 0, 0);
                                                            if (sParams.AnsiCompareIC(PARAMETER_RIGHT) == 0) mouse_event(MOUSEEVENTF_RIGHTUP, 0, 0, 0, 0);
                                                            SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                            return; }
if (sCommand.AnsiCompareIC(COMMAND_REMOTE_MOUSE_DBLCLICK) == 0) {
                                                            // Only do one LEFTUP-event, because I noticed that this way
                                                            // it works. It sends the previous LEFTUP- & LEFTDOWN-events
                                                            // before this, so more events would make it to double-double-click :)
                                                            mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
                                                            SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                            return; }

if (sCommand.AnsiCompareIC(COMMAND_FILE_LIST) == 0) {
                                                    SendList(FILELIST, sHandle, infoKey->GetPath());
                                                    return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_LIST_WIDE) == 0) {
                                                    SendList(WIDELIST, sHandle, infoKey->GetPath());
                                                    return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_DRIVELIST) == 0) {
                                                         SendList(DRIVELIST, sHandle, "");
                                                         return; }

if (sCommand.AnsiCompareIC(COMMAND_FOLDER_CHANGEDIR) == 0) {
                                                           if (sParams == "..") {
                                                                                tmpStr = OneDirUp(infoKey->GetPath());
                                                                                infoKey->SetPath(tmpStr);
                                                                                SServer7->SendCommand(COMMAND_FOLDER_OK, infoKey->GetPath(), sHandle);
                                                                                return;
                                                                                }
                                                           if (sParams == "\\") {
                                                                                tmpStr = infoKey->GetPath();
                                                                                tmpStr = tmpStr.SubString(1, 1) + ":\\";
                                                                                infoKey->SetPath(tmpStr);
                                                                                SServer7->SendCommand(COMMAND_FOLDER_OK, infoKey->GetPath(), sHandle);
                                                                                return;
                                                                                }
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           tmpStr = GetFullFoldername(infoKey->GetPath(), sParams);
                                                           if (tmpStr != NULL) {
                                                                                infoKey->SetPath(tmpStr);
                                                                                SServer7->SendCommand(COMMAND_FOLDER_OK, infoKey->GetPath(), sHandle);
                                                                                } else {
                                                                                SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "DriveNotReady", "Drive not ready") + "!", sHandle);
                                                                                }
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_FOLDER_CREATE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }

                                                           try
                                                           {
                                                           CreateDir(sParams);
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           }
                                                           catch (Exception &exception)
                                                           {
                                                           SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FolderCreate", "Unable to create the folder %"), "%", sParams) + "!", sHandle);
                                                           }
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_FOLDER_DELETE) == 0) {
                                                        tmpStr = GetFullFoldername(infoKey->GetPath(), sParams);
                                                        /*if (tmpStr != NULL) FolderDelete(tmpStr);*/
                                                        SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                        //SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FolderDelete", "Errors while deleting the folder %"), "%", sParams) + "!", sHandle);
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_COPY) == 0) {
                                                      //if (!
                                                      tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                      tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                      CopyFile(tmpStr.c_str(), tmpStr2.c_str(), false);
                                                      //) SendCommand(COMMAND_ERROR, , sHandle);
                                                      SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                      return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_MOVE) == 0) {
                                                      //if (!
                                                      tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                      tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                      MoveFile(tmpStr.c_str(), tmpStr2.c_str());
                                                      //) SendCommand(COMMAND_ERROR, , sHandle);
                                                      SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                      return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_DELETE) == 0) {
                                                      tmpStr = GetFullFilename(infoKey->GetPath(), sParams);
                                                      if (tmpStr != NULL) {
                                                                        DeleteFile(tmpStr);
                                                                        if (FileExists(tmpStr)) {
                                                                                                SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FileDelete", "Unable to delete the file %"), "%", sParams) + "!", sHandle);
                                                                                                return;
                                                                                                }
                                                                        SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                        } else {
                                                                        SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found"), "%", sParams) + "!", sHandle);
                                                                        }
                                                      return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_DOWNLOAD) == 0) {
                                                        tmpStr = GetFullFilename(infoKey->GetPath(), Split2Params(sParams, SEPARATOR_SPLIT, 1));
                                                        if (tmpStr != NULL)
                                                                          {
                                                                          tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                                          infoKey->usFileOp = 1;
                                                                          infoKey->usFilePos = StrToIntDef(tmpStr2, 0);
                                                                          infoKey->usFileSize = ReadFileSize(tmpStr.c_str());

                                                                          if (infoKey->usFileSize < infoKey->usFilePos) {
                                                                                                                        //SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found"), "%", tmpStr) + "!", sHandle);
                                                                                                                        return; }

                                                                          infoKey->usFileHandle = FileOpen(tmpStr, 0);
                                                                          SServer7->SendCommand(COMMAND_FILE_OK, IntToStr(infoKey->usFileSize) + SEPARATOR_SPLIT + IntToStr(FilePort), sHandle);
                                                                          }
                                                                          else
                                                                          {
                                                                          SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found"), "%", Split2Params(sParams, SEPARATOR_SPLIT, 1)) + "!", sHandle);
                                                                          }
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_TRANSFER_STOP) == 0) {
                                                        infoKey->usFileOp = 0;
                                                        if (infoKey->screenPart != NULL) infoKey->screenPart->Clear();
                                                        tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                        infoKey->usFilePos = 0;
                                                        infoKey->usFileSize = 0;
                                                        if (infoKey->usFileHandle != NULL) FileClose(infoKey->usFileHandle);
                                                        infoKey->usFileHandle = NULL;
                                                        SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_RENAME) == 0) {
                                                        tmpStr = GetFullFilename(infoKey->GetPath(), Split2Params(sParams, SEPARATOR_SPLIT, 1));
                                                        tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                        if (tmpStr == NULL) {
                                                                            tmpStr = GetFullFoldername(infoKey->GetPath(), Split2Params(sParams, SEPARATOR_SPLIT, 1));
                                                                            }
                                                        if (tmpStr == NULL) {
                                                                          SServer7->SendCommand(COMMAND_ERROR, FullReplace(ReadFromLanguage("Errors", "FileNotFound", "File % not found"), "%", Split2Params(sParams, SEPARATOR_SPLIT, 1)) + "!", sHandle);
                                                                          return; }

                                                        if (tmpStr2 != NULL) {
                                                                             RenameFile(tmpStr, tmpStr2);
                                                                             SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                             } else {
                                                                             SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle);
                                                                             }
                                                        return; }
//if (sCommand.AnsiCompareIC(COMMAND_FILE_DOWNLOAD) == 0) { tmpStr = GetFullFilename(infoKey->GetPath(), sParams); if (tmpStr != NULL) SendFile(tmpStr); return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_EXECUTE) == 0) {
                                                        if (sParams == "") {
                                                                           SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle);
                                                                           return; }

                                                        tmpStr = GetFullFilename(Split2Params(sParams, SEPARATOR_SPLIT, 1), sParams);
                                                        if (tmpStr == NULL) {
                                                                            tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                                            }
                                                        tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                        tmpStr3 = Split2Params(sParams, SEPARATOR_SPLIT, 3);
                                                        if (tmpStr == NULL) tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                        if (tmpStr2 == "") tmpStr2 = PARAMETER_VISIBLE;
                                                        if (tmpStr2.AnsiCompareIC(PARAMETER_VISIBLE) == 0) ShellExecute(NULL, "open", tmpStr.c_str(), tmpStr3.c_str(), "", SW_SHOW);
                                                        if (tmpStr2.AnsiCompareIC(PARAMETER_INVISIBLE) == 0) ShellExecute(NULL, "open", tmpStr.c_str(), tmpStr3.c_str(), "", SW_HIDE);
                                                        if (tmpStr2.AnsiCompareIC(PARAMETER_MAXIMIZED) == 0) ShellExecute(NULL, "open", tmpStr.c_str(), tmpStr3.c_str(), "", SW_MAXIMIZE);
                                                        if (tmpStr2.AnsiCompareIC(PARAMETER_MINIMIZED) == 0) ShellExecute(NULL, "open", tmpStr.c_str(), tmpStr3.c_str(), "", SW_MINIMIZE);
                                                        SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_PRINT) == 0) {
                                                        tmpStr = GetFullFilename(Split2Params(sParams, SEPARATOR_SPLIT, 1), sParams);
                                                        if (tmpStr == NULL) {
                                                                            tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                                            }
                                                        ShellExecute(NULL, "print", tmpStr.c_str(), "", "", SW_HIDE);
                                                        SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                        return; }
if (sCommand.AnsiCompareIC(COMMAND_FILE_FIND) == 0) { return; }

if (sCommand.AnsiCompareIC(COMMAND_ROUTER) == 0) {
                                                 Route(sParams, sHandle, ROUTECONNECT);
                                                 return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_KICK) == 0) {
                                                     if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                     tmpInt = FindAddress(sParams);
                                                     if (tmpInt > -1)
                                                                    { KickClient(tmpInt, ReadFromLanguage("Logging", "logHasBeenKicked", "% has been kicked") + " (" + ReadFromLanguage("Logging", "logChatKickKill", "killed") + ").");
                                                                    SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                    }
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_CLEARBANS) == 0) {
                                                          DoReject = false;
                                                          strReject = "";
                                                          SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                          return;
                                                          }
if (sCommand.AnsiCompareIC(COMMAND_USERS_BAN) == 0) {
                                                     if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                     tmpInt = FindAddress(sParams);
                                                     if (tmpInt > -1) KickClient(tmpInt, ReadFromLanguage("Logging", "logHasBeenKicked", "% has been kicked") + " (" + ReadFromLanguage("Logging", "logChatKickKill", "killed") + ").");
                                                     BanAddress(sParams);
                                                     SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_UNBAN) == 0) {
                                                     if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                     UnBanAddress(sParams);
                                                     SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_BANS) == 0) {
                                                     tmpStr = ConvertBooleanToWord(DoReject) + SEPARATOR_SINGLE;
                                                     tmpStr += FullReplace(strReject, "\r\n", SEPARATOR_SINGLE);
                                                     SServer7->SendCommand(COMMAND_USERS_BANS, tmpStr, sHandle);
                                                     return; }
if (sCommand.AnsiCompareIC(COMMAND_USERS_KICKALL) == 0) { KickAll(); return; }

if (sCommand.AnsiCompareIC(COMMAND_OPENCD) == 0) { OpenCloseCD(true, sParams); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_CLOSECD) == 0) { OpenCloseCD(false, sParams); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_DELAY) == 0) { SServer7->tmrShutdown->Tag = StrToIntDef(sParams, 0) * 60; SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_SERVER_CLOSE) == 0) { ExitServer(); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_LOCK) == 0) { ShellExecute(NULL, "open", "RunDll32.exe", "user32.dll,LockWorkStation", "", SW_SHOW); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_LOGOFF) == 0) { SD = LOGOFF; SServer7->tmrShutdown->Enabled = true; SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_REBOOT) == 0) { SD = REBOOT; SServer7->tmrShutdown->Enabled = true; SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_SHUTDOWN) == 0) { SD = SHUTDOWN; SServer7->tmrShutdown->Enabled = true; SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_FORCE) == 0) { SD = FORCE; SServer7->tmrShutdown->Enabled = true; SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_MONITOR) == 0) { MonitorOff(); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_SCREENSAVER) == 0) { StartScreenSaver(); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_SERVER_RESTART) == 0) { SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_SHUTDOWN_STOP) == 0) { SServer7->tmrShutdown->Enabled = false; if (frmShutdownInst == NULL) frmShutdownInst = new TfrmShutdown(SServer7); if (frmShutdownInst != NULL) frmShutdownInst->Hide(); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }

if (sCommand.AnsiCompareIC(COMMAND_DISABLE_CONTROL) == 0) { AUTOIT_BlockInput(1); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }
if (sCommand.AnsiCompareIC(COMMAND_ENABLE_CONTROL) == 0) { AUTOIT_BlockInput(0); SServer7->SendCommand(COMMAND_OK, "", sHandle); return; }

if (sCommand.AnsiCompareIC(COMMAND_SERVER_OPTIONS) == 0) {
                                                         // If parameters specified, these are the options.
                                                         // Otherwise send the options.
                                                         if (sParams.Length() < 50)
                                                                          {
                                                                          if (LoadSettings() != -1)
                                                                                                {
                                                                                                tmpStr = frmShutdown->txtTemp->Text;
                                                                                                tmpStr = SwitchToSendFormat(tmpStr);
                                                                                                SServer7->SendCommand(COMMAND_SERVER_OPTIONS, tmpStr, sHandle);
                                                                                                }
                                                                          }
                                                                          else
                                                                          {
                                                                          frmShutdown->txtTemp->Clear();
                                                                          tmpStr = SwitchToListFormat(sParams);
                                                                          frmShutdown->txtTemp->Text = tmpStr;
                                                                          SaveSettings();
                                                                          SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                          }
                                                         frmShutdown->txtTemp->Clear();
                                                         return; }

if (sCommand.AnsiCompareIC(COMMAND_CMD) == 0) {
                                                // Run command line command with CMD.EXE:
                                                if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                tmpStr = sParams + "\r\n";
                                                tmpStr2 = AnsiString(CMDExec(tmpStr.c_str()));
                                                // Delete 2 lines from start and end:
                                                strLines = new TStringList();
                                                strLines->Text = tmpStr2;
                                                if (strLines->Count < 4) {
                                                                          return;
                                                                          } else {
                                                                          strLines->Delete(0);
                                                                          strLines->Delete(0);
                                                                          strLines->Delete(strLines->Count - 1);
                                                                          }
                                                tmpStr2 = strLines->Text;
                                                strLines->Free();
                                                tmpStr2 = SwitchToSendFormat(tmpStr2);
                                                SServer7->SendCommand(COMMAND_CMD, tmpStr2, sHandle);
                                                return; }
if (sCommand.AnsiCompareIC(COMMAND_MOUSELISTEN_START) == 0) {
                                                            if (infoKey->usMouseListen == NULL)
                                                                                              {
                                                                                              infoKey->usMouseListen = new TTimer(SServer7);
                                                                                              if (infoKey->usMouseListen == NULL) return;
                                                                                              infoKey->usMouseListen->OnTimer = SServer7->tmrMouseListenTimer;

                                                                                              // Default interval is half seconds (500), if using Telnet.
                                                                                              // Client will usually use 50 as the default.
                                                                                              infoKey->usMouseListen->Interval = StrToIntDef(sParams, 500);
                                                                                              infoKey->usMouseListen->Tag = sHandle;
                                                                                              }

                                                            tagPOINT CursorPos;
                                                            GetCursorPos(&CursorPos);
                                                            infoKey->SetLastMousePos(CursorPos.x, CursorPos.y);
                                                            SServer7->SendCommand(COMMAND_REMOTE_MOUSE_MOVE, IntToStr(CursorPos.x) + SEPARATOR_SPLIT + IntToStr(CursorPos.y), sHandle);
                                                            infoKey->usMouseListen->Enabled = true;
                                                            return;
                                                            }
if (sCommand.AnsiCompareIC(COMMAND_MOUSELISTEN_STOP) == 0) {
                                                            if (infoKey->usMouseListen == NULL) return;
                                                            infoKey->usMouseListen->Enabled = false;
                                                            SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                            return;
                                                            }
if (sCommand.AnsiCompareIC(COMMAND_SCREEN_VIEW) == 0) {
                                                      if (sParams.AnsiCompareIC(PARAMETER_START) == 0) {
                                                                                                       infoKey->usScreenView = true;
                                                                                                       CheckScreen();
                                                                                                       SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                                                       }
                                                      if (sParams.AnsiCompareIC(PARAMETER_STOP) == 0) {
                                                                                                       infoKey->usScreenView = false;
                                                                                                       SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                                                                       }
                                                      return;
                                                      }
if (sCommand.AnsiCompareIC(COMMAND_SCREENSHOT) == 0) {
                                                     if (infoKey->screenPart == NULL) infoKey->screenPart = new TMemoryStream();
                                                     if (infoKey->screenPart == NULL) return; // Could not create new stream...

                                                     infoKey->screenPart->Clear();
                                                     infoKey->screenPart->Position = 0;
                                                     //TakeScreenShot(infoKey->screenPart);
                                                     if (CaptureScreen(infoKey->screenPart) < 0) {
                                                                                                          SServer7->SendCommand(COMMAND_ERROR, "", sHandle);
                                                                                                          return; }
                                                     if (infoKey->screenPart->Size <= 0) return;

                                                     if (CompressStream(infoKey->screenPart, 20, true) < 0) {
                                                                                                            SServer7->SendCommand(COMMAND_ERROR, "", sHandle);
                                                                                                            return; }

                                                     infoKey->usFileOp = 3;
                                                     infoKey->usFilePos = 0;
                                                     infoKey->usFileSize = infoKey->screenPart->Size;
                                                     infoKey->usFileHandle = 3;

                                                     // X1, Y1, X2, Y2, File size, Data port, Type:
                                                     SServer7->SendCommand(COMMAND_SCREEN_PART, IntToStr(0) + SEPARATOR_SPLIT + IntToStr(0) + SEPARATOR_SPLIT +
                                                                                                IntToStr(Screen->Width - 1) + SEPARATOR_SPLIT + IntToStr(Screen->Height - 1) + SEPARATOR_SPLIT + IntToStr(infoKey->usFileSize) +
                                                                                                SEPARATOR_SPLIT + IntToStr(FilePort) + SEPARATOR_SPLIT + IntToStr(0), sHandle);

                                                     return;
                                                     }
if (sCommand.AnsiCompareIC(COMMAND_WEBCAMSHOT) == 0) {
                                                     if (infoKey->usFileHandle != NULL) return;

                                                     if (infoKey->screenPart == NULL) infoKey->screenPart = new TMemoryStream();
                                                     if (infoKey->screenPart == NULL) return; // Could not create a new stream...

                                                     if (VideoBox == NULL) VideoBox = new TScrollBox(frmShutdown);
                                                     if (VideoBox == NULL) return;
                                                     VideoBox->Parent = frmShutdown;
                                                     if (!infoKey->UsingWebcam) OpenCamera(VideoBox);
                                                     infoKey->UsingWebcam = true;

                                                     if (CapturePicture(AppPath + "LSCRAPT.BMP") == -1) {
                                                                                                        CloseCamera();
                                                                                                        infoKey->UsingWebcam = false;
                                                                                                        SServer7->SendCommand(COMMAND_ERROR, "", sHandle);
                                                                                                        return;
                                                                                                        }
                                                     //CloseCamera();
                                                     //infoKey->UsingWebcam = false;
                                                     infoKey->screenPart->Clear();
                                                     infoKey->screenPart->Position = 0;
                                                     infoKey->screenPart->LoadFromFile(AppPath + "LSCRAPT.BMP");
                                                     if (CompressStream(infoKey->screenPart, 100, false) == -1) return;
                                                     if (infoKey->screenPart->Size <= 0) return;
                                                     infoKey->usFileOp = 3;
                                                     infoKey->usFilePos = 0;
                                                     infoKey->usFileSize = infoKey->screenPart->Size;
                                                     infoKey->usFileHandle = 3;
                                                     infoKey->usScreenView = false;

                                                     // X1, Y1, X2, Y2, File size, Data port, Type:
                                                     SServer7->SendCommand(COMMAND_SCREEN_PART, IntToStr(0) + SEPARATOR_SPLIT + IntToStr(0) + SEPARATOR_SPLIT +
                                                                                                IntToStr(Screen->Width - 1) + SEPARATOR_SPLIT + IntToStr(Screen->Height - 1) + SEPARATOR_SPLIT + IntToStr(infoKey->usFileSize) +
                                                                                                SEPARATOR_SPLIT + IntToStr(FilePort) + SEPARATOR_SPLIT + IntToStr(2), sHandle);

                                                     return; }

if (sCommand.AnsiCompareIC(COMMAND_WINDOW_TERMINATE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           AUTOIT_WinClose(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_SHOW) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           AUTOIT_WinShow(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_HIDE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           AUTOIT_WinHide(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_MAXIMIZE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           AUTOIT_WinMaximize(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_MINIMIZE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           AUTOIT_WinMinimize(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_FOCUS) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           //AUTOIT_WinRestore(sParams.c_str(), "");
                                                           AUTOIT_WinActivate(sParams.c_str(), "");
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_RENAME) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                           tmpStr2 = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                           AUTOIT_WinSetTitle(tmpStr.c_str(), "", tmpStr2.c_str());
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_MOVE) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 1);
                                                           tmpInt = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 2), 0);
                                                           tmpInt2 = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 3), 0);
                                                           AUTOIT_WinMove(tmpStr.c_str(), "", tmpInt, tmpInt2, Screen->Width, Screen->Height);
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_HOST_INFORMATION) == 0) {
                                                           SendList(HOSTINFO, sHandle, "");
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_CPU_USAGE) == 0) {
                                                           SServer7->SendCommand(COMMAND_CPU_USAGE, IntToStr(GetCPUUsage()) + " %", sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_PROCESS_LIST) == 0) {
                                                           tmpStr = ListProcesses();
                                                           if (tmpStr == "-1") {
                                                                               SServer7->SendCommand(COMMAND_ERROR, ReadFromLanguage("Errors", "ProcessList", "Unable to get the process list!"), sHandle);
                                                                               return; }

                                                           SServer7->SendCommand(COMMAND_PROCESS_LIST, tmpStr, sHandle);
                                                           return; }
if (sCommand.AnsiCompareIC(COMMAND_WINDOW_LIST) == 0) {
                                                           if (sParams.AnsiCompareIC(PARAMETER_INVISIBLE) == 0) {
                                                                                                                SServer7->SendCommand(COMMAND_WINDOW_LIST, ListWindows(false), sHandle);
                                                                                                                } else {
                                                                                                                SServer7->SendCommand(COMMAND_WINDOW_LIST, ListWindows(true), sHandle);
                                                                                                                }
                                                           return; }

if (sCommand.AnsiCompareIC(COMMAND_PROCESS_KILL) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }

                                                           tmpInt = StrToIntDef(sParams, 0);
                                                           if (tmpInt <= 0) return;

                                                           KillProcess(tmpInt);
                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }

if (sCommand.AnsiCompareIC(COMMAND_PROCESS_PRIORITY) == 0) {
                                                           if (sParams == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }

                                                           tmpInt = StrToIntDef(Split2Params(sParams, SEPARATOR_SPLIT, 1), 0);
                                                           tmpStr = Split2Params(sParams, SEPARATOR_SPLIT, 2);
                                                           if (tmpStr == "") { SServer7->SendCommand(COMMAND_ERROR, PARAMETER_MISSING_PARAMETER, sHandle); return; }
                                                           if (tmpInt <= 0) return;

                                                           if (SetPriority(tmpInt, tmpStr) == -1) {
                                                                                                  SServer7->SendCommand(COMMAND_ERROR, "", sHandle);
                                                                                                  return; }

                                                           SServer7->SendCommand(COMMAND_OK, "", sHandle);
                                                           return; }
/*
strHelp += COMMAND_WINDOW_ALWAYSONTOP + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_NOTALWAYSONTOP + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_FLASH + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_FREEZE + " window_caption" + strSeparator;
strHelp += COMMAND_WINDOW_UNFREEZE + " window_caption" + strSeparator;
*/


// No match... it was an invalid command:
SServer7->SendCommand(COMMAND_ERROR, PARAMETER_INVALID_COMMAND, sHandle);
return;

}
catch (Exception &exception)
{
SServer7->SendCommand(COMMAND_ERROR, exception.Message, sHandle);
return;
}

}



void __fastcall TSServer7::sckCommandClientRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
// *****************************************************************************
// *****************************************************************************
// Read data from the client and redirect the commands to ProcessCommand.
// Data parser is here, as well as the flood protection.
// *****************************************************************************
// *****************************************************************************

AnsiString newData;
AnsiString newCommand;
AnsiString newParams;
AnsiString tmpStr;
AnsiString tmpStr2;
unsigned int iPos;
unsigned int iPos2;
unsigned int i;
TUserInfo * infoKey = NULL;


newData = Socket->ReceiveText();


// Recover the class via its pointer stored in the list:
infoKey = (TUserInfo *) myDB.getValue(Socket->SocketHandle);
if (infoKey == NULL)
                   {
                   KickClient(Socket->SocketHandle, "");
                   return;
                   }


// Reset ping timeout timer:
infoKey->usPing->Enabled = false;
infoKey->usPing->Enabled = true;


if (infoKey->GetKey() != NULL)
                             {
                             //newData = Decrypt(Socket->ReceiveText(), infoKey->GetKey());
                             infoKey->usCommand += newData;
                             }
                             else
                             {
                             // Telnet-connection detected:
                             if (infoKey->GetAccess() >= 1) Socket->SendText(newData);
                             infoKey->usCommand += newData;
                             // Support for Backspace-character:
                             infoKey->usCommand = BackspaceToErase(infoKey->usCommand);
                             }





if (infoKey->GetAccess() < 2)
                            {
                            if (infoKey->GetAccess() == 0)
                                                         {
                                                         if (infoKey->usCommand.Length() > 80)
                                                                 {
                                                                 KickClient(Socket->SocketHandle, "");
                                                                 return;
                                                                 }
                                                         }
                             if (infoKey->usCommand.Length() > 400)
                                                                  {
                                                                  BanAddress(Socket->RemoteAddress);
                                                                  KickClient(Socket->SocketHandle, ReadFromLanguage("Logging", "logKickFlood", "% kicked and banned (flood)") + ".");
                                                                  return;
                                                                  }
                             infoKey->usFloodAmount += infoKey->usCommand.Length();
                             if (infoKey->usFloodAmount > 400)
                                                            {
                                                            BanAddress(Socket->RemoteAddress);
                                                            KickClient(Socket->SocketHandle, ReadFromLanguage("Logging", "logKickFlood", "% kicked and banned (flood)") + ".");
                                                            return;
                                                            }
                             }


if (infoKey->usRouter != NULL)
                                {
                                Route(infoKey->usCommand, Socket->SocketHandle, ROUTESEND);
                                infoKey->ClearCommand();
                                return;
                                }


iPos = AnsiPos(SEPARATOR_END, infoKey->usCommand);
if (iPos == 0) return;

tmpStr2 = infoKey->usCommand;
infoKey->ClearCommand();
// Command parser:

do
    {
    iPos = AnsiPos(SEPARATOR_END, tmpStr2);
    tmpStr = tmpStr2.SubString(1, iPos - 1);

    if (AnsiPos(SEPARATOR_COMMAND, tmpStr) == 0)
                                     {
                                     tmpStr2 = tmpStr2.SubString(iPos + 2, tmpStr2.Length() - iPos - 2);
                                     ProcessCommand(tmpStr, "", Socket->SocketHandle);
                                     continue;
                                     }

    for (i = 1; i < tmpStr.Length(); ++i)
                {
                if (tmpStr.SubString(i, 1) == SEPARATOR_COMMAND || tmpStr.SubString(i, 1) == SEPARATOR_SINGLE)
                                                                {
                                                                iPos2 = i;
                                                                newCommand = tmpStr.SubString(1, i - 1);
                                                                newParams = tmpStr.SubString(iPos2 + 1, tmpStr.Length() - iPos2);
                                                                tmpStr2 = tmpStr2.SubString(iPos + 2, tmpStr2.Length() - iPos - 2);
                                                                ProcessCommand(newCommand, newParams, Socket->SocketHandle);
                                                                break;
                                                                }
                }
    }

while (tmpStr2.Length() > 2);



}



void __fastcall TSServer7::sckCommandClientDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
// *****************************************************************************
// *****************************************************************************
// Delete the class instance and remove the pointer from the list.
// *****************************************************************************
// *****************************************************************************

AnsiString tmpStr;
TUserInfo * infoKey = NULL;

infoKey = (TUserInfo *) myDB.getValue(Socket->SocketHandle);
if (infoKey == NULL)
                   {
                   return;
                   }


tmpStr = FullReplace(ReadFromLanguage("Logging", "logLeft", "% has left") + ".", "%", infoKey->GetNick());
SendToAll(COMMAND_CHAT, tmpStr, Socket->SocketHandle);
if (frmChatInst == NULL) frmChatInst = new TfrmChat(SServer7);
if (frmChatInst != NULL) frmChatInst->txtChat->Text = frmChatInst->txtChat->Text + tmpStr + "\r\n";

if (infoKey->UsingWebcam) CloseCamera();
delete infoKey;
myDB.removeValue(Socket->SocketHandle);
}
//---------------------------------------------------------------------------


void __fastcall TSServer7::tmrLoginTimer(TObject *Sender)
{
// Login timeout:
KickClient(dynamic_cast<TComponent *>(Sender)->Tag, ReadFromLanguage("Logging", "logKickLoginTimeout", "% kicked (login timeout)"));
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::tmrShutdownTimer(TObject *Sender)
{
// Delayed shutdown:

unsigned short int h, m, s;
AnsiString hs, ms, ss;

tmrShutdown->Tag = tmrShutdown->Tag - 1;

if (tmrShutdown->Tag <= 0)
                         {
                         if (frmShutdownInst != NULL) {
                                                      frmShutdownInst->Free();
                                                      frmShutdownInst = NULL;
                                                      }
                         tmrShutdown->Enabled = False;
                         SystemShutdown(SD);
                         return;
                         }


h = tmrShutdown->Tag / 3600;
m = tmrShutdown->Tag / 60%60;
s = tmrShutdown->Tag % 60;

hs = IntToStr(h);
if (hs.Length() < 2) hs = "0" + hs;
ms = IntToStr(m);
if (ms.Length() < 2) ms = "0" + ms;
ss = IntToStr(s);
if (ss.Length() < 2) ss = "0" + ss;


if (frmShutdownInst == NULL) frmShutdownInst = new TfrmShutdown(SServer7);

if (frmShutdownInst != NULL)
                           {
                           if (frmShutdownInst->lblTime->Caption == "") frmShutdownInst->Show();
                           frmShutdownInst->lblTime->Caption = hs + ":" + ms + ":" + ss;
                           }

}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckCommandClientError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
// Errors usually occur because of dead connections, so:
KickClient(Socket->SocketHandle, ReadFromLanguage("Logging", "logKickNoResponse", "% kicked (got no response)") + ".");
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::ServicePause(TService *Sender, bool &Paused)
{
KickAll();
sckFile->Close();
sckCommand->Close();
}
//---------------------------------------------------------------------------

bool AcceptAddress(const AnsiString sAddress)
{
// *****************************************************************************
// AcceptAddress
// *****************************************************************************
// If "accept only" or "reject" is enabled, check if the address is in the accept
// or reject list. Uses wildcard compare, wild.cpp. * and ? are allowed.
// *****************************************************************************

unsigned short int i;
AnsiString curLine = "";
AnsiString curChar = "";

if (DoAccept)
            {
for (i = 1; i <= strAccept.Length(); ++i)
         {
         curChar = strAccept.SubString(i, 1);
         // Line changes:
         if (curChar == "\r" || curChar == "\n")
                                               {
                                               if (wildcmp(curLine.c_str(), sAddress.c_str())) return true; // Address is an allowed one.
                                               curLine = "";
                                               }
                                               else
                                               {
                                               curLine += curChar;
                                               }
         }
            return false; // No address found from the accept list, so do not accept.
            }

if (DoReject)
            {
for (i = 1; i <= strReject.Length(); ++i)
         {
         curChar = strReject.SubString(i, 1);
         // Line changes:
         if (curChar == "\r" || curChar == "\n")
                                               {
                                               if (wildcmp(curLine.c_str(), sAddress.c_str())) return false; // A-ha! Address is banned!
                                               curLine = "";
                                               }
                                               else
                                               {
                                               curLine += curChar;
                                               }
         }
            return true; // No address found from the reject list, so accept.
            }

// DoAccept and DoReject were both disabled, so...
return true;
}

void __fastcall TSServer7::ServiceContinue(TService *Sender,
      bool &Continued)
{
if (FilePort > 0 && !Callback)
                             {
                             sckCommand->Open();
                             SServer7->sckFile->Open();
                             }
                             else
                             {
                             tmrCallback->Enabled = true;
                             }
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::tmrJoinFloodTimer(TObject *Sender)
{
//strJoinFlood->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckRouteConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
// Router socket connected, send command OK to the client:
SendCommand(COMMAND_OK, "", dynamic_cast<TComponent *>(Sender)->Tag);
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckRouteDisconnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
// Router socket disconnected, kick the client off:
KickClient(dynamic_cast<TComponent *>(Sender)->Tag, "");
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckRouteError(TObject *Sender,
      TCustomWinSocket *Socket, TErrorEvent ErrorEvent, int &ErrorCode)
{
// Errors usually occur because of dead connections, so kick the client off:
KickClient(dynamic_cast<TComponent *>(Sender)->Tag, "");
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckRouteRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
// Redirect data from the router socket back to the client:
unsigned int tmpInt;

tmpInt = FindHandle(dynamic_cast<TComponent *>(Sender)->Tag);
if (tmpInt != -1) sckCommand->Socket->Connections[tmpInt]->SendText(Socket->ReceiveText());
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::ServiceAfterInstall(TService *Sender)
{
// Run configuration after installing the service:
AppPath = ExtractFilePath(Forms::Application->ExeName);
if (FileExists(AppPath + "SConfig.exe")) ShellExecute(NULL, "open", "SConfig.EXE", "", AppPath.c_str(), SW_SHOW);
}
//---------------------------------------------------------------------------


void __fastcall TSServer7::ServiceBeforeUninstall(TService *Sender)
{
ShellExecute(NULL, "open", "NET.EXE", "STOP SServer", "", SW_HIDE);
}
//---------------------------------------------------------------------------


void __fastcall TSServer7::tmrRefreshTrayIconTimer(TObject *Sender)
{
// Explorer crashes quite often, so make sure that the icon stays visible:
if (!Invisible) TrayIcon(Forms::Application->Handle, "SServer7", 0);
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::tmrFloodTimer(TObject *Sender)
{
unsigned int sHandle;
TUserInfo * infoKey = NULL;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   KickClient(sHandle, "");
                   return;
                   }

// Reset flood buffer.
infoKey->usFloodAmount = 0;
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::tmrCallbackTimer(TObject *Sender)
{
// Try connecting to the callback-address:
if (!sckCallback->Socket->Connected)
                                   {
                                   sckCallback->Close();
                                   sckCallback->Open();
                                   }
}
//---------------------------------------------------------------------------












void __fastcall TSServer7::sckFileClientConnect(TObject *Sender,
      TCustomWinSocket *Socket)
{
unsigned int sHandle;
TUserInfo * infoKey = NULL;

sHandle = FindAddress(Socket->RemoteAddress);
if (sHandle == -1)
                 {
                 Socket->Close();
                 return;
                 }

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   Socket->Close();
                   KickClient(sHandle, "");
                   return;
                   }


if (infoKey->GetAccess() < 2)
                            {
                            Socket->Close();
                            return;
                            }



}
//---------------------------------------------------------------------------

void __fastcall TSServer7::sckFileClientRead(TObject *Sender,
      TCustomWinSocket *Socket)
{
unsigned char Buffer[4096];
unsigned int lenBuf = 4096;
unsigned int sHandle;
TUserInfo * infoKey = NULL;


sHandle = FindAddress(Socket->RemoteAddress);
if (sHandle == -1)
                 {
                 Socket->Close();
                 return;
                 }

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   Socket->Close();
                   //KickClient(sHandle, "");
                   return;
                   }

if (infoKey->usFileHandle == NULL) return; // No download/upload in progress


try
{
// User is viewing the screen:
if (infoKey->usFileOp == 3)
                          {
                          Socket->ReceiveText();
                          if (infoKey->screenPart == NULL) {
                                                           infoKey->usFileHandle = NULL;
                                                           CheckScreen();
                                                           return;
                                                           }

                          if ((infoKey->usFilePos + lenBuf) > infoKey->screenPart->Size) lenBuf = infoKey->screenPart->Size - infoKey->usFilePos;
                          infoKey->screenPart->Read(Buffer, lenBuf);
                          infoKey->usFilePos += lenBuf;
                          Socket->SendBuf(Buffer, lenBuf);
                          if (infoKey->usFilePos >= infoKey->screenPart->Size)
                                                                                {
                                                                                infoKey->screenPart->Clear();
                                                                                infoKey->usFileHandle = NULL;
                                                                                CheckScreen();
                                                                                return;
                                                                                }
                          return;
                          }

// User is uploading a file:
if (infoKey->usFileOp == 2)
                          {
                          Socket->ReceiveBuf(Buffer, lenBuf);
                          FileWrite(infoKey->usFileHandle, Buffer, lenBuf);
                          infoKey->usFilePos += lenBuf;
                          if (infoKey->usFilePos >= infoKey->usFileSize)
                                                                        {
                                                                        FileClose(infoKey->usFileHandle);
                                                                        infoKey->usFileHandle = NULL;
                                                                        return;
                                                                        }
                          Socket->SendText("R"); 
                          return;
                          }


// User is downloading a file:
if (infoKey->usFilePos >= infoKey->usFileSize)
                                                {
                                                FileClose(infoKey->usFileHandle);
                                                infoKey->usFileHandle = NULL;
                                                return;
                                                }

Socket->ReceiveText();
if ((infoKey->usFilePos + lenBuf) > infoKey->usFileSize) lenBuf = infoKey->usFileSize - infoKey->usFilePos;
infoKey->usFilePos += lenBuf;
FileRead(infoKey->usFileHandle, Buffer, lenBuf);

if (!Socket->Connected)
                        {
                        FileClose(infoKey->usFileHandle);
                        infoKey->usFileHandle = NULL;
                        Socket->Close();
                        return;
                        }

Socket->SendBuf(Buffer, lenBuf);
}
catch (Exception &exception)
{
Socket->Close();
}
}
//---------------------------------------------------------------------------



void __fastcall TSServer7::tmrMouseListenTimer(TObject *Sender)
{
// Send mouse movement data...
unsigned int sHandle;
tagPOINT CursorPos;
TUserInfo * infoKey = NULL;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;

infoKey = (TUserInfo *) myDB.getValue(sHandle);
if (infoKey == NULL)
                   {
                   Sender->Free();
                   return;
                   }


GetCursorPos(&CursorPos);

if (infoKey->MousePosChanged(CursorPos.x, CursorPos.y))
                            {
                            SendCommand(COMMAND_REMOTE_MOUSE_MOVE, IntToStr(CursorPos.x) + SEPARATOR_SPLIT + IntToStr(CursorPos.y), sHandle);
                            infoKey->SetLastMousePos(CursorPos.x, CursorPos.y);
                            }
}
//---------------------------------------------------------------------------

void __fastcall TSServer7::tmrKeyboardListenTimer(TObject *Sender)
{
// Send keys pressed...
//unsigned int sHandle;

//sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
}
//---------------------------------------------------------------------------



void __fastcall TSServer7::tmrPingTimer(TObject *Sender)
{
// PING! PONG? Ping timeout, kick the connection off.
// *****************************************************************************
// NOTE: It kicks the Telnet-connections off as well, that's why it's optional
// in SConfig. It might also cause some trouble in several different conditions,
// like connection routing, etc. Ping timeout is 10 minutes, specified in
// void sckCommandClientConnect.

if (!PingClients) return;

unsigned int sHandle;

sHandle = dynamic_cast<TComponent *>(Sender)->Tag;
KickClient(sHandle, ReadFromLanguage("Logging", "logKickPingTimeout", "% kicked (ping timeout)") + ".");

}
//---------------------------------------------------------------------------


void __fastcall TSServer7::ServiceShutdown(TService *Sender)
{
SServer7->Free();
ExitServer();
}
//---------------------------------------------------------------------------





