/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


/*

Processes.cpp
Part of the Specrem-project.

Process and window control for Window manager.
Copyright (C) 2002  Antti Kirjavainen

*/


//---------------------------------------------------------------------------
#include <vcl.h>
#include <windows.h>
#include <tlhelp32.h>
#define NUM_PROCS	100
//---------------------------------------------------------------------------
AnsiString ListProcesses(void);
int GetPriority(const unsigned short int PID);
int SetPriority(const unsigned short int PID, const AnsiString sPriority);
int KillProcess(const unsigned short int PID);
AnsiString PriorToWord(DWORD newPri);
AnsiString ListWindows(const bool VisibleOnly);


AnsiString ListWindows(const bool VisibleOnly)
{
HWND DeskTopHwnd, WindowHwnd;
char Buff[255];
AnsiString strBuff;
AnsiString strList = "";

AnsiString SEPARATOR_SINGLE = "\n\r";


// Get desktop handle and all its child windows:
DeskTopHwnd = GetDesktopWindow();
WindowHwnd = GetWindow(DeskTopHwnd, GW_CHILD);


// Add all windows to the list, visible/invisible:
do
{
GetWindowText(WindowHwnd, Buff, 255);
strBuff = AnsiString(Buff);
if (strBuff != "" && IsWindowVisible(WindowHwnd) == VisibleOnly)   {
                                                                   strList += SEPARATOR_SINGLE;
                                                                   strList += strBuff;
                                                                   }
WindowHwnd = GetWindow(WindowHwnd, GW_HWNDNEXT);
}
while (WindowHwnd > 0);


return strList;
}

AnsiString ListProcesses(void)
{
HANDLE SnapShot;
PROCESSENTRY32 ProcessList;
AnsiString tmpStr;

AnsiString SEPARATOR_SINGLE = "\n\r";
AnsiString SEPARATOR_SPLIT = "|";


SnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, NULL);
if(Process32First(SnapShot, &ProcessList) == false)
                                                  {
                                                  // Error, exit...
                                                  CloseHandle(SnapShot);
                                                  return "";
                                                  }
while(1)
{
if (Process32Next(SnapShot, &ProcessList) == false)
                                                  {
                                                  if(GetLastError() != ERROR_NO_MORE_FILES) {
                                                                                          // Error, exit...
                                                                                          CloseHandle(SnapShot);
                                                                                          return "-1";
                                                                                          } else {
                                                                                          // ...and we're done, return the list:
                                                                                          CloseHandle(SnapShot);
                                                                                          return tmpStr;
                                                                                          }
                                                  }

// Add every process to the list:
tmpStr += SEPARATOR_SINGLE + AnsiString(ProcessList.szExeFile) + SEPARATOR_SPLIT + AnsiString(ProcessList.th32ProcessID);
}
}

AnsiString PriorToWord(DWORD newPri)
{
AnsiString PARAMETER_REALTIME = "REALTIME";
AnsiString PARAMETER_HIGH = "HIGH";
AnsiString PARAMETER_NORMAL = "NORMAL";
AnsiString PARAMETER_IDLE = "IDLE";

if (newPri == REALTIME_PRIORITY_CLASS) return PARAMETER_REALTIME;
if (newPri == HIGH_PRIORITY_CLASS) return PARAMETER_HIGH;
if (newPri == NORMAL_PRIORITY_CLASS) return PARAMETER_NORMAL;
if (newPri == IDLE_PRIORITY_CLASS) return PARAMETER_IDLE;

return NULL;
}

int SetPriority(const unsigned short int PID, const AnsiString sPriority)
{
	int LoopVar;
	HANDLE procHandle;
        DWORD newPri;

        AnsiString PARAMETER_REALTIME = "REALTIME";
        AnsiString PARAMETER_HIGH = "HIGH";
        AnsiString PARAMETER_NORMAL = "NORMAL";
        AnsiString PARAMETER_IDLE = "IDLE";

        if (sPriority.AnsiCompareIC(PARAMETER_REALTIME) == 0) newPri = REALTIME_PRIORITY_CLASS;
        if (sPriority.AnsiCompareIC(PARAMETER_HIGH) == 0) newPri = HIGH_PRIORITY_CLASS;
        if (sPriority.AnsiCompareIC(PARAMETER_NORMAL) == 0) newPri = NORMAL_PRIORITY_CLASS;
        if (sPriority.AnsiCompareIC(PARAMETER_IDLE) == 0) newPri = IDLE_PRIORITY_CLASS;

        // Open process and set priority:
        procHandle = OpenProcess(PROCESS_ALL_ACCESS, true, PID);
        if(!SetPriorityClass(procHandle, newPri)) return -1;
        CloseHandle(procHandle);

	return 0;
}

int GetPriority(const unsigned short int PID)
{
	int LoopVar;
	HANDLE procHandle;
	DWORD newPri;


        // Open process and get priority:
        procHandle = OpenProcess(PROCESS_ALL_ACCESS, true, PID);
        if((newPri = GetPriorityClass(procHandle)) == 0) return 1;
        CloseHandle(procHandle);
                        
        switch(newPri) {
                        case REALTIME_PRIORITY_CLASS:
                                return REALTIME_PRIORITY_CLASS;
                        case HIGH_PRIORITY_CLASS:
                                return HIGH_PRIORITY_CLASS;
                        case NORMAL_PRIORITY_CLASS:
                                return NORMAL_PRIORITY_CLASS;
                        case IDLE_PRIORITY_CLASS:
                                return IDLE_PRIORITY_CLASS;
	}

	return -1;
}

int KillProcess(const unsigned short int PID)
{
HANDLE ProcessHandle;

ProcessHandle = OpenProcess(PROCESS_ALL_ACCESS, false, DWORD(PID));
if (ProcessHandle == 0) return -1;

TerminateProcess(ProcessHandle, 0);
CloseHandle(ProcessHandle);

return 0;
}