/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <windows.h>
#include <vcl.h>
#include <string>
#include <stdio.h>
using namespace std;

#include "FileInfo.hpp"



/*
* Contructor
*/
FileInfo::FileInfo(void)
	{
	this->flag_isDirectory	=false;
	this->flag_isAnArchive	=false;
	this->flag_isCompressed	=false;
	this->flag_isEncrypted	=false;
	this->flag_isHidden		=false;
	this->flag_isOffline		=false;
	this->flag_isReadOnly	=false;
	this->flag_isSystem		=false;
	this->flag_isTemporary	=false;
	}

/*
* Destructor
*/
FileInfo::~FileInfo(void)
	{
	}

/*
* Retrieve the file/Folder name has a long format string.
* Return: string : The file/folder name.
*/
string FileInfo::getWideFilename(void)
	{
	return this->wideFilename;
	}

/*
* Retrieve the file/Folder name has a short format string (dos 8.3).
* Return: string : The file/folder name.
*/
string FileInfo::getShortFilename(void)
	{
	return this->shortFilename;
	}

/*
* Check if the file is a directory.
* Return: bool : true or false
*/
bool FileInfo::isDirectory(void)		{	return this->flag_isDirectory;	}

/*
* Check if the file/directory has the archive flag set.
* Applications use this attribute to mark files for backup or removal.
* Return: bool : true or false
*/
bool FileInfo::isAnArchive(void)		{	return this->flag_isAnArchive;	}

/*
* Check if the file/directory is compressed.
* For a directory, this means that compression is the default for newly created files and subdirectories
* Return: bool : true or false
*/
bool FileInfo::isCompressed(void)	{	return this->flag_isCompressed;	}

/*
* Check if the file/directory is encrypted.
* For a file, this means that all data in the file is encrypted.
* For a directory, this means that encryption is the default for newly created files and subdirectories.
* Return: bool : true or false
*/
bool FileInfo::isEncrypted(void)		{	return this->flag_isEncrypted;	}

/*
* Check if the file/directory is hidden.
* Return: bool : true or false
*/
bool FileInfo::isHidden(void)			{	return this->flag_isHidden;		}

/*
* Check if the file/directory is offline (not immediately available).
* This attribute indicates that the file data has been physically moved to offline storage.
* This attribute is used by Remote Storage, the hierarchical storage management software.
* Applications should not arbitrarily change this attribute.
* Return: bool : true or false
*/
bool FileInfo::isOffline(void)		{	return this->flag_isOffline;		}

/*
* Check if the file/directory can only be read and not modified.
* Return: bool : true or false
*/
bool FileInfo::isReadOnly(void)		{	return this->flag_isReadOnly;		}

/*
* Check if the file/directory is system file.
* For a directory, this means that compression is the default for newly created files and subdirectories
* Return: bool : true or false
*/
bool FileInfo::isSystem(void)			{	return this->flag_isSystem;		}

/*
* Check if the file/directory is temporary file.
* Return: bool : true or false
*/
bool FileInfo::isTemporary(void)		{	return this->flag_isTemporary;	}

/*
* Return the local system date/time of the creation of the file/folder.
* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
*/
structFileInfo_Date FileInfo::getDateCreate(void)			{	return this->dateCreated;	}

/*
* Return the local system date/time of the last access of the file/folder.
* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
*/
structFileInfo_Date FileInfo::getDateLastAccessed(void)	{	return this->dateLastAccessed;	}

/*
* Return the local system date/time of the last modification of the file/folder.
* Return: structFileInfo_Date : structure containing the information (see FileInfo.hpp)
*/
structFileInfo_Date FileInfo::getDateLastWrite(void)		{	return this->dateLastWrite;	}

/*
* Convert the date information into a string.
* Input: date    : the structure structFileInfo_Date containing the date information (see FileInfo.hpp).
* Return: string : format is "DD-MM-YYYY"
*/
string FileInfo::dateToString(structFileInfo_Date date)
	{
	char Buffer[11];

	sprintf(Buffer, "%02d-%02d-%04d", date.day, date.month, date.year);
	return Buffer;
        //AnsiString Buffer = "";
        //Buffer = AnsiString(date.day) + "-" + AnsiString(date.month) + "-" + AnsiString(date.year);
	//return Buffer.c_str();
	}

/*
* Convert the time information into a string.
* Input: date    : the structure structFileInfo_Date containing the time information (see FileInfo.hpp).
* Return: string : format is "HH:MM:SS"
*/
string FileInfo::timeToString(structFileInfo_Date date)
	{
	char Buffer[9];

	sprintf(Buffer, "%02d:%02d:%02d", date.hour, date.minute, date.second);
        return Buffer;
        //AnsiString Buffer = "";
        //Buffer = AnsiString(date.hour) + ":" + AnsiString(date.minute) + ":" + AnsiString(date.second);
	//return Buffer.c_str();
	}

/*
* Retrieve the file size.
* Return: ULARGE_INTEGER : The file size.
*/
ULARGE_INTEGER FileInfo::getFileSize(void)	{ return this->fileSize;	}

