/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/

// Copyright (C) 2002  Farid Bengrid


#include <windows.h>
#include <string>
using namespace std;
#include "FolderList.hpp"

/*
* Contructor
*/
FolderList::FolderList(void)
	{
	this->listFile  = new GenericList();
	this->listFolder= new GenericList();
	}

/*
* Destructor
*/
FolderList::~FolderList(void)
	{
	clear();
	delete this->listFile;
	delete this->listFolder;
	}

/*
* Store the content of the a specific folder.
* Input:  Path : a string containing the path of the folder(WITHOUT the \ at the end)
* Return: 0                : Successfull.
*         ERR_SYSTEMCODE   : A system error occured see the value of this->SystemErrorCode;
*         ERR_MEMORYFAILURE: Allocation memory failure
*/
int FolderList::storePathContent(string Path)
	{
	HANDLE hSearch;
	WIN32_FIND_DATA st_FindFileData;
	string strSearch;

	FileInfo * ptrFileInfo;
	int ErrorCode;

	//Clean the previous search store locally
	clear();
	
	//Create the search Pattern...
	strSearch=Path+"\\*";

	//Start the search;
	if((hSearch=FindFirstFile(strSearch.c_str(),&st_FindFileData))==INVALID_HANDLE_VALUE)
		{
		this->SystemErrorCode=GetLastError();
		//return ERR_SYSTEMCODE;
		return -1;
		}
	do
		{
		if (strcmp(st_FindFileData.cFileName,".")!=0 && strcmp(st_FindFileData.cFileName,".."))
			{
			ptrFileInfo=new FileInfo();
			
			//Fill the names
			ptrFileInfo->wideFilename =st_FindFileData.cFileName;
			ptrFileInfo->shortFilename=(strcmp(st_FindFileData.cAlternateFileName,"")==0?st_FindFileData.cFileName:st_FindFileData.cAlternateFileName);


			//Fill the Attributes
			ptrFileInfo->flag_isDirectory	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0);
			ptrFileInfo->flag_isAnArchive	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_ARCHIVE)!=0);
			ptrFileInfo->flag_isCompressed=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_COMPRESSED)!=0);
			ptrFileInfo->flag_isEncrypted	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_ENCRYPTED)!=0);
			ptrFileInfo->flag_isHidden		=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN)!=0);
			ptrFileInfo->flag_isOffline	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_OFFLINE)!=0);
			ptrFileInfo->flag_isReadOnly	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_READONLY)!=0);
			ptrFileInfo->flag_isSystem		=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_SYSTEM)!=0);
			ptrFileInfo->flag_isTemporary	=((st_FindFileData.dwFileAttributes&FILE_ATTRIBUTE_TEMPORARY)!=0);

			//Fill the the date/time values
			convertFileTimeToLocalTime(&st_FindFileData.ftCreationTime  ,&ptrFileInfo->dateCreated);
			convertFileTimeToLocalTime(&st_FindFileData.ftLastAccessTime,&ptrFileInfo->dateLastAccessed);
			convertFileTimeToLocalTime(&st_FindFileData.ftLastWriteTime ,&ptrFileInfo->dateLastWrite);

			//Fill the file size value
			ptrFileInfo->fileSize.LowPart =st_FindFileData.nFileSizeLow;
			ptrFileInfo->fileSize.HighPart=st_FindFileData.nFileSizeHigh;
			
			//Store the entry in the correct list
			if (ptrFileInfo->flag_isDirectory)
				{
				ErrorCode=this->listFolder->add(&ptrFileInfo,sizeof(FileInfo *));
				}
			else
				{
				ErrorCode=this->listFile->add(&ptrFileInfo,sizeof(FileInfo *));
				}
			if (ErrorCode!=0)
				{
				FindClose(hSearch);
				return ErrorCode;
				}
			}

		}while(FindNextFile(hSearch,&st_FindFileData)!=0);

	//Check if the end is due to the fact that there are no more files/folders or because of an error.
	if((this->SystemErrorCode=GetLastError())!=ERROR_NO_MORE_FILES)
		{
		//return ERR_SYSTEMCODE;
		return -1;
		FindClose(hSearch);
		}
	
	FindClose(hSearch);
	return 0;
	}

/*
* Retrieve the number of files stored in the internal list.
* Input:  none
* Return: unsigned long    : the number of files
*/
unsigned long FolderList::getNumberOfFiles(void)
	{
	return this->listFile->size();
	}

/*
* Retrieve the number of folders stored in the internal list.
* Input:  none
* Return: unsigned long    : the number of folders
*/
unsigned long FolderList::getNumberOfFolders(void)
	{
	return this->listFolder->size();
	}

/*
* Retrieve the object containing the information of the file at the specified position.
* Input:  index  : the index of the file [0, getNumberOfFiles()-1].
* Return: void * : A pointer to the object containing the information(Must be cast to FileInfo *).
*/
void * FolderList::getFileInfo(unsigned long index)
	{
	FileInfo * ptrFileInfo=0;
	if (this->listFile->get(index,&ptrFileInfo)!=0)
		return NULL;
	return ptrFileInfo;
	}

/*
* Retrieve the object containing the information of the folder at the specified position.
* Input:  index  : the index of the folder [0, getNumberOfFolders()-1].
* Return: void * : A pointer to the object containing the information(Must be cast to FileInfo *).
*/
void * FolderList::getFolderInfo(unsigned long index)
	{
	FileInfo * ptrFileInfo=0;
	if (this->listFolder->get(index,&ptrFileInfo)!=0)
		return NULL;
	return ptrFileInfo;
	}

	
/*
* Convert the date/time of a file from the system time scheme to the local time
* Input:  ptrfileTime: A pointer to the filetime system
* Output: localTime  : A pointer to the local time structure (declared in Fileinfo.hpp)
* Return: int        : 0 if successfull, -1 otherwise.
*/
int FolderList::convertFileTimeToLocalTime(FILETIME * ptrfileTime,ptrstructFileInfo_Date localTime)
	{
	SYSTEMTIME systemTimeUTC,systemTimeLocal;

	if (FileTimeToSystemTime(ptrfileTime,&systemTimeUTC)==0)
		return -1;
	if (SystemTimeToTzSpecificLocalTime(NULL, &systemTimeUTC, &systemTimeLocal)==0)
		return -1;
	
	localTime->day   =systemTimeLocal.wDay;
	localTime->month =systemTimeLocal.wMonth;
	localTime->year  =systemTimeLocal.wYear;
	localTime->hour  =systemTimeLocal.wHour;
	localTime->minute=systemTimeLocal.wMinute;
	localTime->second=systemTimeLocal.wSecond;
	return 0;
  	}
/*
* Clean the local stored info.
*/
void FolderList::clear(void)
	{
	FileInfo * ptrFileInfo=0;

	while(this->listFile->size()>0)
		{
		this->listFile->get(0,&ptrFileInfo);
		delete ptrFileInfo;
		this->listFile->remove(0);
		}

	while(this->listFolder->size()>0)
		{
		this->listFolder->get(0,&ptrFileInfo);
		delete ptrFileInfo;
		this->listFolder->remove(0);
		}
	}


