/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid

#ifndef _FOLDERLIST_HPP_
#define _FOLDERLIST_HPP_

#ifndef _FILEINFO_HPP_
#include "FileInfo.hpp"
#endif

#ifndef _GENERICLIST_H_
#include "GenericList.hpp"
#endif

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

class FolderList
	{
	public:
		DWORD SystemErrorCode;

		/*
		* Contructor
		*/
		FolderList(void);
		
		/*
		* Destructor
		*/
		~FolderList(void);

		/*
		* Store the content of the a specific folder.
		* Input:  Path : a string containing the path of the folder(WITHOUT the \ at the end)
		* Return: 0                : Successfull.
		*         ERR_SYSTEMCODE   : A system error occured see the value of this->SystemErrorCode;
		*         ERR_MEMORYFAILURE: Allocation memory failure
		*/
		int storePathContent(string Path);

		/*
		* Retrieve the number of files stored in the internal list.
		* Input:  none
		* Return: unsigned long    : the number of files
		*/
		unsigned long getNumberOfFiles(void);

		/*
		* Retrieve the number of folders stored in the internal list.
		* Input:  none
		* Return: unsigned long    : the number of folders
		*/
		unsigned long getNumberOfFolders(void);
		
		/*
		* Retrieve the object containing the information of the file at the specified position.
		* Input:  index  : the index of the file [0, getNumberOfFiles()-1].
		* Return: void * : A pointer to the object containing the information(Must be cast to FileInfo *).
		*/
		void * getFileInfo(unsigned long index);

		/*
		* Retrieve the object containing the information of the folder at the specified position.
		* Input:  index  : the index of the folder [0, getNumberOfFolders()-1].
		* Return: void * : A pointer to the object containing the information(Must be cast to FileInfo *).
		*/
		void * getFolderInfo(unsigned long index);
		
	private:
		GenericList * listFile;
		GenericList * listFolder;

		/*
		* Convert the date/time of a file from the system time scheme to the local time
		* Input:  ptrfileTime: A pointer to the filetime system
		* Output: localTime  : A pointer to the local time structure (declared in Fileinfo.hpp)
		* Return: int        : 0 if successfull, -1 otherwise.
		*/
		int convertFileTimeToLocalTime(FILETIME * ptrfileTime, ptrstructFileInfo_Date localTime);

		/*
		* Clean the local stored info.
		*/
		void clear();

	};
#undef _FOLDERLIST_HPP_
#endif