/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <stdlib.h>
#include <memory.h>
#include "GenericListItem.hpp"

GenericListItem::GenericListItem(void)
	{
	this->Previous    =NULL;
	this->Next        =NULL;
	this->ItemDataSize=0;
	this->ItemData    =NULL;
	}

GenericListItem::~GenericListItem(void)
	{
	free(this->ItemData);
	}

/*
* Store the data in the current Item
* Input:  ptrData : A pointer to the data to store.
* Input:  dataSize: The number of bytes to be copied.
* Return: 0                : Successfull
*         ERR_FUNCTIONPARAM: datasize == 0 and ptrData!=NULL
*         ERR_MEMORYFAILURE: Memory allocation failure
*/
int GenericListItem::setItemData(void * ptrData, size_t dataSize)
	{
	if (this->ItemData!=NULL)
		{
		free(this->ItemData);
		this->ItemDataSize=0;
		this->ItemData    =NULL;
		}
	if (ptrData!=NULL)
		{
		if (dataSize==0)				return ERR_FUNCTIONPARAM;
		this->ItemData=malloc(dataSize);
		if (this->ItemData==NULL)	return ERR_MEMORYFAILURE;
		memcpy(this->ItemData,ptrData,dataSize);
		this->ItemDataSize=dataSize;
		}
	return 0;
	}

/*
* Return the size in Bytes of the stored data
* Input:  none
* Return: 0 if no data, the size otherwise
*/
size_t GenericListItem::getItemSize(void)
	{
	return this->ItemDataSize;
	}

/*
* Get a copy of the data Stored
* Input:  ptrData : A pointer where the data stored will be copied.
* Return: 0                        : Successfull
*         ERR_FUNCTIONPARAM        : if ptrData is NULL
*         ERR_LISTITEM_MISSING_DATA: if there are no stored data.
*/
int GenericListItem::getItemData(void * ptrData)
	{
	if (ptrData==NULL)			return ERR_FUNCTIONPARAM;
	if (this->ItemData==NULL)	return ERR_LISTITEM_MISSING_DATA;

	memcpy(ptrData,this->ItemData,this->ItemDataSize);
	return 0;
	}

/*
* Get a Pointer to the data for Read Access (DO NOT CHANGE THE CONTENT !!!)
* Input: none
* Return: void *: Pointer to the Data
*           NULL: no Data.
*/
void * GenericListItem::getPtrToData(void)
	{
	return this->ItemData;
	}

/*
* Register the previous Item
* Input:  A Pointer to an GenericListItem class, or NULL if no previous Item
* Return: none
*/
void GenericListItem::setPreviousItem(GenericListItem * previous)
	{
	this->Previous=previous;
	}

/*
* Register the next Item
* Input:  A Pointer to an GenericListItem class, or NULL if no previous Item
* Return: none
*/
void GenericListItem::setNextItem(GenericListItem * next)
	{
	this->Next=next;
	}
			
/*
* Recover the adress of the previous Item
* Input:  none
* Return: GenericListItem * : A pointer to the previous item. NULL if no Item.
*/
GenericListItem * GenericListItem::getPreviousItem(void)
	{
	return this->Previous;
	}

/*
* Recover the adress of the next Item
* Input:  none
* Return: GenericListItem * : A pointer to the next item. NULL if no Item.
*/
GenericListItem * GenericListItem::getNextItem(void)
	{
	return this->Next;
	}

/*
* Compare the content the specify buffer and the stored one.
* Input:  ptrData : A pointer to the data to compare with .
* Input:  dataSize: The number of bytes of the data to compare with.
* Input:  none
* Return: true if equal, false if different;
*/
bool GenericListItem::contains(void * ptrData, size_t dataSize)
	{
	if (this->ItemDataSize==dataSize && memcmp(this->ItemData,ptrData,dataSize)==0)
		return true;
	return false;
	}


