/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#ifndef _GENERICLISTITEM_HPP_
#define _GENERICLISTITEM_HPP_

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

/*
* List Items
*/
class GenericListItem
	{
	public:
		/*
		* Contructor
		*/
		GenericListItem(void);

		/*
		* Destructor
		*/
		~GenericListItem(void);

		/*
		* Store the data in the current Item
		* Input:  ptrData : A pointer to the data to store.
		* Input:  dataSize: The number of bytes to be copied.
		* Return: 0                : Successfull
		*         ERR_FUNCTIONPARAM: datasize == 0 and ptrData!=NULL
		*         ERR_MEMORYFAILURE: Memory allocation failure
		*/
		int setItemData(void * ptrData, size_t dataSize);

		/*
		* Return the size in Bytes of the stored data
		* Input:  none
		* Return: 0 if no data, the size otherwise
		*/
		size_t getItemSize(void);

		/*
		* Get a copy of the data Stored
		* Input:  ptrData : A pointer where the data stored will be copied.
		* Return: 0                        : Successfull
		*         ERR_FUNCTIONPARAM        : if ptrData is NULL
		*         ERR_LISTITEM_MISSING_DATA: if there are no stored data.
		*/
		int getItemData(void * ptrData);

		/*
		* Get a Pointer to the data for Read Access (DO NOT CHANGE THE CONTENT !!!)
		* Input: none
		* Return: void *: Pointer to the Data
		*           NULL: no Data.
		*/
		void * getPtrToData(void);
		
		/*
		* Register the previous Item
		* Input:  A Pointer to an GenericListItem class, or NULL if no previous Item
		* Return: none
		*/
		void setPreviousItem(GenericListItem * previous);

		/*
		* Register the next Item
		* Input:  A Pointer to an GenericListItem class, or NULL if no previous Item
		* Return: none
		*/
		void setNextItem(GenericListItem * next);
			
		/*
		* Recover the adress of the previous Item
		* Input:  none
		* Return: GenericListItem * : A pointer to the previous item. NULL if no Item.
		*/
		GenericListItem * getPreviousItem(void);

		/*
		* Recover the adress of the next Item
		* Input:  none
		* Return: GenericListItem * : A pointer to the next item. NULL if no Item.
		*/
		GenericListItem * getNextItem(void);

		/*
		* Compare the content the specify buffer and the stored one.
		* Input:  ptrData : A pointer to the data to compare with .
		* Input:  dataSize: The number of bytes of the data to compare with.
		* Input:  none
		* Return: true if equal, false if different;
		*/
		bool contains(void * ptrData, size_t dataSize);

	private:
		GenericListItem * Previous;
		GenericListItem * Next;
		
		size_t ItemDataSize;
		void * ItemData;
	};
#endif