/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#ifndef _INIHANDLER_H_
#define _INIHANDLER_H_

#ifndef _INISECTION_H_
#include "IniSection.hpp"
#endif

#ifndef _GENERICLIST_H_
#include "..\Shared\GenericList.hpp"
#endif

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

#define MAX_BUFFER_READ	8192
#define CTX_SECTION	0
#define CTX_ITEM		1
#define CTX_NOMORE	2

class IniHandler
	{
	public:
		/*
		* Constructor
		*/
		IniHandler();

		/*
		* Destructor
		*/
		~IniHandler();

		/*
		* Add (or update) an item value (section & item will be created if not existing)
		* Input: ptrSectionName: Pointer to the section name
		* Input: ptrItemName   : Pointer to the item name
		* Input: ptrItemValue  : Pointer to the value content
		* Return: 0                : Successfull
		*         ERR_MEMORYFAILURE: Allocation memory failure
		*/
		int add(char * ptrSectionName, char * ptrItemName, char * ptrItemValue);

		/*
		* Get the number of character of the Item Value
		* Input: ptrSectionName: Pointer to the section name
		* Input: ptrItemName   : Pointer to the item name
		* Return: int                     : Successfull
		*         ERR_INI_SECTION_NOTFOUND: Section not found
		*         ERR_INI_ITEM_NOTFOUND   : Item not Found
		*/
		int getItemValueLength(char * ptrSectionName, char * ptrItemName);

		/*
		* Remove all the local entries saved in memory.
		* Input: None.
		* Return:None.
		*/
		void Clear(void);

		/*
		* Get a copy of the Item Value
		* Input: ptrSectionName: Pointer to the section name
		* Input: ptrItemName   : Pointer to the item name
		* Output:ptrItemValue  : Pointer to where the value will be copied
		* Return: 0                        : Successfull.
		*         ERR_INI_SECTION_NOTFOUND : Section not found
		*         ERR_INI_ITEM_NOTFOUND    : Item not Found
		*         ERR_INI_FIELD_EMPTY      : the Item Value is empty
		*/
		int getItemValue(char * ptrSectionName, char * ptrItemName, char * ptrItemValue);

		/*
		* Read Ini items from the specified file
		* Input: filename: The ini file 
		* Return: int               : The number of Item loaded
		*         ERR_INI_FILEERROR : Can not find/open the file specified
		*         ERR_MEMORYFAILURE : Allocation memory failure
		*/
		int readFromFile(char * filename);
		
		/*
		* Save the items contained in memory to a previously opened ini file
		* Input: none
		* Return: 0                 : Successfull
		*         ERR_INI_NOFILENAME: No ini file was previously opened...
		*         ERR_INI_FILEERROR : Can not find/open the file specified
		*/
		int save(void);
		/*
		* Save the items contained in memory to the specified ini file
		* Input: filename: The ini file 
		* Return: 0                 : Successfull
		*         ERR_MEMORYFAILURE : Allocation memory failure
		*         ERR_INI_NOFILENAME: No ini file was previously opened...
		*         ERR_INI_FILEERROR : Can not find/open the file specified
		*/
		int save(char * filename);

	private:
		GenericList * listSections;
		char * iniFilename;

		/*
		* Return a pointer to the section defined by its name
		* Input: ptrSectionName: A pointer to the name to search
		* Return: IniSection * : The pointer to the Section
		*                 NULL : not found
		*/
		IniSection * getSection(char * ptrSectionName);
		int getNextFromFile(FILE * hFile,char * Buffer);
		int cleanBuffer(char * Buffer);
		int setFilename(char * filename);
	};
#endif