/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <stdlib.h>
#include <string.h>
#include "IniItem.hpp"


/*
* Constructor
*/
IniItem::IniItem()
	{
	this->itemName=new GenericList();
	this->itemData=new GenericList();
	}

/*
* Destructor
*/
IniItem::~IniItem()
	{
	delete this->itemName;
	delete this->itemData;
	}

/*
* Add a new Item
* Input:  ItemName     : The new Item
* Input:  ItemNewValue : Its value
* Return: 0                : Successfull.
*         ERR_MEMORYFAILURE: Allocation memory failure
*/
int IniItem::addItem(char * ItemName, char * ItemValue)
	{
	int ErrCode;
	
	//Check if the Item exis already
	if (this->itemName->indexOf(ItemName,strlen(ItemName)+1)!=ERR_LIST_ITEM_NOT_FOUND)
		{
		return setItemValue(ItemName,ItemValue);
		}
	else
		{
		//This is a new item
		if((ErrCode=this->itemName->add(ItemName,strlen(ItemName)+1))!=0)
			{
			return ErrCode;
			}
	
		if ((ErrCode=this->itemData->add(ItemValue,strlen(ItemValue)+1))!=0)
			{
			this->itemName->remove(this->itemName->size()-1);
			return ErrCode;
			}
		}

	return 0;
	}

/*
* Store A new Value for the specified item
* Input:  ItemName     : The Item looked for
* Input:  ItemNewValue : The new value
* Return: 0                    : Successfull.
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*         ERR_MEMORYFAILURE    : Memory allocation failure
*/
int IniItem::setItemValue(char * ItemName, char * ItemNewValue)
	{
	long index;

	//Get the index of the name;
	if((index=this->itemName->indexOf(ItemName,strlen(ItemName)+1))==ERR_LIST_ITEM_NOT_FOUND)
		return ERR_INI_ITEM_NOTFOUND;

	return this->itemData->update(index,ItemNewValue,strlen(ItemNewValue)+1);
	}

/*
* Get the number of character of the Item Value at the specified position
* Input:  ItemName : The Item looked for
* Return: int                  : Successfull.
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*/
int IniItem::getItemValueLength(char * ItemName)
	{
	long index;

	//Get the index of the name;
	if((index=this->itemName->indexOf(ItemName,strlen(ItemName)))==ERR_LIST_ITEM_NOT_FOUND)
		return ERR_INI_ITEM_NOTFOUND;

	return this->itemData->getDataSize(index)-1;
	}

/*
* Get a copy of the Item Value at the specified position
* Input:  ItemName : The Item looked for
* Output: ItemValue: A pointer to a pre-allocated memory where the data will be copied
* Return: 0                    : Successfull.
*         ERR_FUNCTIONPARAM    : ItemValue==NULL
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
*/
int IniItem::getItemValue(char * ItemName, char * ItemValue)
	{
	long index;
	int ErrCode;
	
	//Get the index of the name;
	if((index=this->itemName->indexOf(ItemName,strlen(ItemName)+1))==ERR_LIST_ITEM_NOT_FOUND)
		return ERR_INI_ITEM_NOTFOUND;
		
	
	if((ErrCode=this->itemData->get(index,ItemValue))==ERR_LISTITEM_MISSING_DATA)
		return ERR_INI_FIELD_EMPTY;

	return ErrCode;
	}

/*
* Get the size in bytes of the Item Name and Value at the specified position
* Input:  index      : The index of the item
* Output: nameLength : A pointer to a pre-allocated memory where the data will be copied
* Output: valueLength: A pointer to a pre-allocated memory where the data will be copied
* Return: 0                    :Successful
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
*/
int IniItem::getItemLength(long index,int * nameLength, int * valueLength)
	{
	if ((*nameLength =this->itemName->getDataSize(index))==0)	return ERR_INI_ITEM_NOTFOUND;
	if ((*valueLength=this->itemData->getDataSize(index))==0)	return ERR_INI_FIELD_EMPTY;
	return 0;
	}

/*
* Get a copy of the Item Name and Value at the specified position
* Input:  index    : The index of the item
* Output: ItemName : A pointer to a pre-allocated memory where the data will be copied
* Output: ItemValue: A pointer to a pre-allocated memory where the data will be copied
* Return: 0                        : Successfull.
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
*/
int IniItem::getItem(long index,char * ItemName, char * ItemValue)
	{
	int ErrCode;
	if ((ErrCode=this->itemName->get(index,ItemName))!=0)		return (ErrCode==ERR_FUNCTIONPARAM?ERR_FUNCTIONPARAM:ERR_INI_ITEM_NOTFOUND);
	if ((ErrCode=this->itemData->get(index,ItemValue))!=0)	return (ErrCode==ERR_FUNCTIONPARAM?ERR_FUNCTIONPARAM:ERR_INI_FIELD_EMPTY);
	return 0;
	}

/*
* Get a Pointer to the Item Name and Value at the specified position
* Input:  index    : The index of the item
* Output: ptrItemName : A pointer to a pre-allocated memory where the data will be copied
* Output: ptrItemValue: A pointer to a pre-allocated memory where the data will be copied
* Return: 0                        : Successfull.
*         ERR_INI_ITEM_NOTFOUND: Item not Found
*/
int IniItem::getItemPtr(long index,char ** ptrItemName, char ** ptrItemValue)
	{
	if (index<0 || index>=this->itemName->size())		return ERR_INI_ITEM_NOTFOUND;
	
	*ptrItemName =(char *) this->itemName->getPtrToData(index);
	*ptrItemValue=(char *) this->itemData->getPtrToData(index);
	return 0;
	}

/*
* Return the number of Items
* Input : none
* return: long : the number of elements
*/
int IniItem::getNumberOfItem(void)
	{
	return this->itemName->size();
	}