/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#ifndef _INIITEM_HPP_
#define _INIITEM_HPP_

#ifndef _GENERICLIST_HPP_
#include "..\Shared\GenericList.hpp"
#endif

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

class IniItem
	{
	public:
		/*
		* Contructor
		*/
		IniItem();

		/*
		* Destructor
		*/
		~IniItem();
		
		/*
		* Add a new Item
		* Input:  ItemName     : The new Item
		* Input:  ItemNewValue : Its value
		* Return: 0                : Successfull.
		*         ERR_MEMORYFAILURE: Allocation memory failure
		*/
		int addItem(char * ItemName, char * ItemValue);

		/*
		* Store A new Value for the specified item
		* Input:  ItemName     : The Item looked for
		* Input:  ItemNewValue : The new value
		* Return: int                  : Successfull.
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*         ERR_MEMORYFAILURE    : Memory allocation failure
		*/
		int setItemValue(char * ItemName, char * ItemNewValue);

		/*
		* Get the number of character of the Item Value at the specified position
		* Input:  ItemName : The Item looked for
		* Return: int                  : Successfull.
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*/
		int getItemValueLength(char * ItemName);

		/*
		* Get a copy of the Item Value at the specified position
		* Input:  ItemName : The Item looked for
		* Output: ItemValue: A pointer to a pre-allocated memory where the data will be copied
		* Return: 0                        : Successfull.
		*         ERR_FUNCTIONPARAM        : ItemValue==NULL
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
		*/
		int getItemValue(char * ItemName, char * ItemValue);

		/*
		* Get the size in bytes of the Item Name and Value at the specified position
		* Input:  index      : The index of the item
		* Output: nameLength : A pointer to a pre-allocated memory where the data will be copied
		* Output: valueLength: A pointer to a pre-allocated memory where the data will be copied
		* Return: 0                      :Successful
		* Return: 0                    :Successful
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
		*/
		int getItemLength(long index,int * nameLength, int * valueLength);

		/*
		* Get a copy of the Item Name and Value at the specified position
		* Input:  index    : The index of the item
		* Output: ItemName : A pointer to a pre-allocated memory where the data will be copied
		* Output: ItemValue: A pointer to a pre-allocated memory where the data will be copied
		* Return: 0                        : Successfull.
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*         ERR_INI_FIELD_EMPTY  : the Item Value is empty
		*/
		int getItem(long index, char * ItemName, char * ItemValue);

		/*
		* Get a Pointer to the Item Name and Value at the specified position
		* Input:  index    : The index of the item
		* Output: ptrItemName : A pointer to a pre-allocated memory where the data will be copied
		* Output: ptrItemValue: A pointer to a pre-allocated memory where the data will be copied
		* Return: 0                        : Successfull.
		*         ERR_INI_ITEM_NOTFOUND: Item not Found
		*/
		int getItemPtr(long index,char ** ptrItemName, char ** ptrItemValue);

		/*
		* Return the number of Items
		* Input : none
		* return: long : the number of elements
		*/
		int getNumberOfItem(void);

	private:
		//Data
		GenericList * itemName;
		GenericList * itemData;
		};
#endif