/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#include <stdlib.h>
#include <string.h>
#include "IniSection.hpp"


/*
* Constructor
*/
IniSection::IniSection()
	{
	this->sectionName=NULL;
	this->Items= new IniItem();
	}

/*
* Destructor
*/
IniSection::~IniSection()
	{
	if (this->sectionName!=NULL) free(this->sectionName);
	delete this->Items;
	}
/*
* Store the name of this Section
* Input: sectionName: the Name of the section
* Return: 0                : Successfull
*         ERR_MEMORYFAILURE: Allocation memory failure.
*/
int IniSection::setName(char * sectionName)
	{
	//Clear existing if required
	if (this->sectionName!=NULL)
		free(this->sectionName);
	
	//Allocate the memory required
	if ((this->sectionName=(char *) calloc(strlen(sectionName)+1,sizeof(char)))==NULL)
		return ERR_MEMORYFAILURE;

	//Copy the name
	strcpy(this->sectionName,sectionName);

	return 0;
	}

/*
* Recover the number of characters of the Section name
* Input:  none
* Return: the number of characters
*/
int IniSection::getNameLenght(void)
	{
	return (this->sectionName==NULL?0:strlen(this->sectionName));
	}

/*
* Get a copy of the section name
* Output: ptrSectionName: A pointer to a pre-allocated memory where the section name will be copied
* Return: 0                  : Successfull.
*         ERR_FUNCTIONPARAM  : sectionName==NULL
*         ERR_INI_FIELD_EMPTY: There is no SectionName
*/
int IniSection::getName(char * ptrSectionName)
	{
	if (ptrSectionName==NULL)		return ERR_FUNCTIONPARAM;
	if (this->sectionName==NULL)	return ERR_INI_FIELD_EMPTY;
	strcpy(sectionName,this->sectionName);
	return 0;
	}

/*
* Get the Pointer of the local buffer containing the Section name
* Return: char * : The Pointer to the name
*/
char * IniSection::getPtrName(void)
	{
	return this->sectionName;
	}

/*
* Check if the name of the section is the one proposed
* Input: ptrSectionName: A pointer to the name to compare
* Return: 0                  : Successfull.
*         ERR_FUNCTIONPARAM  : sectionName==NULL
*         ERR_INI_FIELD_EMPTY: There is no SectionName
*/
bool IniSection::isEqualTo(char * ptrSectionName)
	{
	if (strlen(this->sectionName)==strlen(ptrSectionName) && strcmp(this->sectionName,ptrSectionName)==0)
		return true;
	
	return false;
	}