/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


#ifndef _INISECTION_H_
#define _INISECTION_H_

#ifndef _INIITEM_H_
#include "IniItem.hpp"
#endif

#ifndef _ERRMSG_H_
#include "..\Shared\ErrMsg.h"
#endif

class IniSection
	{
	public:
		IniItem * Items;

		/*
		* Constructor
		*/
		IniSection();
		
		/*
		* Destructor
		*/
		~IniSection();
		
		/*
		* Store the name of this Section
		* Input: sectionName: the Name of the section
		* Return: 0                : Successfull
		*         ERR_MEMORYFAILURE: Allocation memory failure.
		*/
		int setName(char * sectionName);

		/*
		* Recover the number of characters of the Section name
		* Input:  none
		* Return: the number of characters
		*/
		int getNameLenght(void);

		/*
		* Get a copy of the section name
		* Output: ptrSectionName: A pointer to a pre-allocated memory where the section name will be copied
		* Return: 0                  : Successfull.
		*         ERR_FUNCTIONPARAM  : sectionName==NULL
		*         ERR_INI_FIELD_EMPTY: There is no SectionName
		*/
		int getName(char * ptrSectionName);
		
		/*
		* Get the Pointer of the local buffer containing the Section name
		* Return: char * : The Pointer to the name
		*/
		char * getPtrName(void);

		/*
		* Check if the name of the section is the one proposed
		* Input: ptrSectionName: A pointer to the name to compare
		* Return: 0                  : Successfull.
		*         ERR_FUNCTIONPARAM  : sectionName==NULL
		*         ERR_INI_FIELD_EMPTY: There is no SectionName
		*/
		bool isEqualTo(char * ptrSectionName);

	private:
		char  * sectionName;

	};

#endif