/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


//#include <stdlib.h>
//#include <stdio.h>
#include "hashmap.h"


//*******************************************
//* Constructor
//*******************************************
hashmap::hashmap()
	{
	this->Root=NULL;
	this->Last=NULL;
	this->nbElements=0;
	//this->hLog=fopen("hashmap.log","a");
	//fprintf(this->hLog,"Hashmap constructor\n");

	}

//*******************************************
//* Destructor
//*******************************************
hashmap::~hashmap()
	{
	Hashmap_DEF * ptrCurrent;
	if (this->Root!=NULL)
		{
		ptrCurrent=Root;
		while (ptrCurrent->Next!=NULL)
			{
			ptrCurrent=(Hashmap_DEF *) ptrCurrent->Next;
			free(ptrCurrent->Prev);
			}
		free(ptrCurrent);
		}
	//fprintf(this->hLog,"Hashmap Destructor\n");
	//fclose(this->hLog);
	}

//*******************************************
//* Search the list to see if an entry exist
//* In:	key ==> identification of the entry to search
//* out: integer ==> 0 if not found, 1 if found
//*******************************************
int hashmap::contain(unsigned int key)
	{
	if (getEntry(key)!=NULL)
		return 1;
	return 0;
	}

//*******************************************
//* Add a new entry in the list
//* In:	key      ==> identification of the entry to add
//* In:  ptrValue ==> pointer to add
//* Out: integer  ==>  0 = Entry added
//*                    1 = Entry has replaced existing entry with the same key
//*                   -1 = Entry not added due to an allocation memory failure.
//*******************************************
int hashmap::add(unsigned int key, void * ptrValue)
	{
	Hashmap_DEF * Entry;

	//fprintf(this->hLog,"hashmap::add (%u,%p) ==>",key,ptrValue);

	Entry=getEntry(key);
	if (Entry!=NULL)
		{
		//replace the data field of the existing entry
		Entry->value=ptrValue;
		//fprintf(this->hLog,"return = 1\n");
		return 1;
		}
	Entry=createEntry();
	if (Entry==NULL)
		{
		//Memory Allocation Failure
		//fprintf(this->hLog,"return = -1\n");
		return -1;
		}
	Entry->key	=key;
	Entry->value=ptrValue;
	//fprintf(this->hLog,"return = 0\n");
	return 0;
	}

//*******************************************
//* Return the value associated with an identification key
//* In:	key      ==> identification of the entry to add
//* Out: pointer  ==>  the Value(pointer) or NULL if not found
//*******************************************
void * hashmap::getValue(unsigned int key)
	{
	Hashmap_DEF * Entry;
	//fprintf(this->hLog,"hashmap::getValue(%u) ==>",key);
		Entry=getEntry(key);
	if (Entry!=NULL)
		{
		//fprintf(this->hLog,"return =%p\n",Entry->value);
		return Entry->value;
		}
	//fprintf(this->hLog,"return = NULL\n");
	return NULL;
	}

//*******************************************
//* Remove an entry from the list
//* In:	key      ==> identification of the entry to add
//* Out: integer  ==>  0 = Entry removed
//*                   -1 = Entry not found
//*******************************************
int hashmap::removeValue(unsigned int key)
	{

	Hashmap_DEF * Entry;

	//fprintf(this->hLog,"hashmap::removeValue(%u) ==>",key);
	
	Entry=getEntry(key);
	if (Entry==NULL)
		{
		//fprintf(this->hLog,"return = -1\n");
		return -1;
		}

	deleteEntry(Entry);
	//fprintf(this->hLog,"return = 0\n");
	return 0;
	}
	
//*******************************************
//* Return the number of element currenly stored in the list
//* Out: integer  ==>  the number of element in the list
//*******************************************
int hashmap::getSize(void)
	{
	return this->nbElements;
	}


Hashmap_DEF * hashmap::createEntry(void)
	{
	Hashmap_DEF * NewEntry;
	NewEntry=(Hashmap_DEF *) calloc(1,sizeof(Hashmap_DEF));
	if (NewEntry==NULL)
		return NULL;
	//Add the entry at the end of the list
	if (this->Last==NULL)
		{
		//this new entry is in fact the first one !
		this->Root=NewEntry;
		}
	else
		{
		NewEntry->Prev		=this->Last;
		this->Last->Next	=NewEntry;
		}
	this->Last=NewEntry;
	this->nbElements++;
	return NewEntry;
	}
		
Hashmap_DEF * hashmap::getEntry(unsigned int key)
	{
	Hashmap_DEF * Entry;
	Entry=this->Root;

	while(Entry!=NULL && Entry->key!=key)
		Entry=(Hashmap_DEF *) Entry->Next;
	return Entry;
	}

void hashmap::deleteEntry(Hashmap_DEF * Entry)
	{
	if (Entry->Prev==NULL)
		{
		//this is a root entry;
		this->Root=(Hashmap_DEF *)Entry->Next;
		if (this->Root!=NULL)
				this->Root->Prev=NULL;
		else
				this->Last=NULL;
		}
	else if (Entry->Next==NULL)
		{
		//this is the actual last entry.
		((Hashmap_DEF *)Entry->Prev)->Next=NULL;
		this->Last=(Hashmap_DEF *) Entry->Prev;
		}
	else
		{
		//well this entry is in the middle somewhere
		//so first take care of the previous entry.
		((Hashmap_DEF *)Entry->Prev)->Next=Entry->Next;

		// now take care of the next entry
		((Hashmap_DEF *)Entry->Next)->Prev=Entry->Prev;
		}

	//now delete the entry;
	free(Entry);
	this->nbElements--;
	}

