/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


// Copyright (C) 2002  Farid Bengrid


typedef struct
	{
	void * Prev;
	void * Next;
	unsigned int key;
	void * value;
	}Hashmap_DEF;


class hashmap
	{
	public:
		//*******************************************
		//* Constructor / Destructor
		//*******************************************
		hashmap();
		~hashmap();

		//*******************************************
		//* Search the list to see if an entry exist
		//* In:	key ==> identification of the entry to search
		//* out: integer ==> 0 if not found, 1 if found
		//*******************************************
		int contain(unsigned int key);

		//*******************************************
		//* Add a new entry in the list
		//* In:	key      ==> identification of the entry to add
		//* In:  ptrValue ==> pointer to add
		//* Out: integer  ==>  0 = Entry added
		//*                    1 = Entry has replaced existing entry with the same key
		//*                   -1 = Entry not added due to an allocation memory failure.
		//*******************************************
		int add(unsigned int key, void * ptrValue);

		//*******************************************
		//* Return the value associated with an identification key
		//* In:	key      ==> identification of the entry to add
		//* Out: pointer  ==>  the Value(pointer) or NULL if not found
		//*******************************************
		void * getValue(unsigned int key);

		//*******************************************
		//* Remove an entry from the list
		//* In:	key      ==> identification of the entry to add
		//* Out: integer  ==>  0 = Entry removed
		//*                   -1 = Entry not found
		//*******************************************
		int removeValue(unsigned int key);
	
		//*******************************************
		//* Return the number of element currenly stored in the list
		//* Out: integer  ==>  the number of element in the list
		//*******************************************
		int getSize(void);

	private:
		int nbElements;
		//FILE * hLog;
		Hashmap_DEF * Root;
		Hashmap_DEF * Last;
		Hashmap_DEF * createEntry(void);
		Hashmap_DEF * getEntry(unsigned int key);
		void deleteEntry(Hashmap_DEF * Entry);
	};
