/*****************************************************************************
*
*    Specrem Pro - Remote Controlling Software
*    Copyright (C) 2002  Specrem Group
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/


/*

strHandle.cpp
Part of the Specrem-project.

String handling functions, shared between
the client and the server.

Copyright (C) 2002  Antti Kirjavainen

*/


//---------------------------------------------------------------------------
#include <vcl.h>
//---------------------------------------------------------------------------

AnsiString TotalSize(const double Bytes);
double Decimals(const double intDouble, const unsigned short int DecAmount);
AnsiString Split2Params(AnsiString sSplit, const AnsiString sSplitter, const unsigned short int sPos);
AnsiString FullReplace(AnsiString strReplace, const AnsiString strOld, const AnsiString strNew);
AnsiString DelHK(AnsiString strReplace);
AnsiString CheckNick(AnsiString newNick);
int ReadFileSize(const LPSTR pszFileName);
AnsiString SwitchToListFormat(const AnsiString sText);
AnsiString SwitchToSendFormat(const AnsiString sText);
AnsiString AddSlash(const AnsiString sPath);


AnsiString AddSlash(const AnsiString sPath)
{
if (sPath.SubString(sPath.Length(), 1) == "\\") { return sPath; } else { return sPath + "\\"; }
}

int ReadFileSize(const LPSTR pszFileName)
{
   HANDLE hFile;
   DWORD dwFileSize;


try
{
   hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL);
   if(hFile != INVALID_HANDLE_VALUE)
   {
      dwFileSize = GetFileSize(hFile, NULL);
      CloseHandle(hFile);
   }
   return dwFileSize;
}
catch (Exception &exception)
{
        if (hFile != NULL) CloseHandle(hFile);
        return -1;
}
}



AnsiString Split2Params(AnsiString sSplit, const AnsiString sSplitter, const unsigned short int sPos)
{
unsigned short int i;
unsigned short int posCount = 1;
unsigned short int aPos;
unsigned short int LastPos = 1;
unsigned short int lenSplit;
unsigned short int lenSplitter;
AnsiString RestString;


sSplit += sSplitter;

lenSplit = sSplit.Length();
lenSplitter = sSplitter.Length();


if (sPos <= 0) return "";


for (i = 1; i <= lenSplit; ++i)
        {
        if (sSplit.SubString(i, lenSplitter) == sSplitter)
                                                         {
                                                         if (posCount == sPos)
                                                                             {
                                                                             RestString = sSplit.SubString(LastPos, lenSplit - LastPos);
                                                                             aPos = RestString.AnsiPos(sSplitter);
                                                                             if (aPos == 0)
                                                                                          {
                                                                                          aPos = RestString.Length();
                                                                                          }
                                                                                          else
                                                                                          {
                                                                                          if (lenSplitter == 1)
                                                                                                              {
                                                                                                              RestString = RestString.SubString(1, aPos - 1);
                                                                                                              }
                                                                                                              else
                                                                                                              {
                                                                                                              RestString = RestString.SubString(1, aPos - (lenSplitter - 1));
                                                                                                              }
                                                                                          aPos = RestString.Length(); 
                                                                                          }
                                                                             return (RestString.SubString(1, aPos));
                                                                             }
                                                                             else
                                                                             {
                                                                             LastPos = i + lenSplitter;
                                                                             posCount++;
                                                                             }
                                                         }

        }

if (sPos == 1) return sSplit;
return "";
}



AnsiString FullReplace(AnsiString strReplace, const AnsiString strOld, const AnsiString strNew)

// Continue replacing as long as the old string is found from strReplace.
// StringReplace works quite badly in BC++, so created this small function
// to do the job and make sure everything really is replaced.

{
unsigned short int wLimit;
TReplaceFlags rFlags;


wLimit = 0;
rFlags.Contains(rfReplaceAll);

do
{
// If the replace doesn't work, make sure we don't get stuck:
wLimit++;
strReplace = StringReplace(strReplace, strOld, strNew, rFlags);
if (wLimit >= 1000) break;
}
while (strReplace.AnsiPos(strOld) > 0 && wLimit < 1000);

return strReplace;
}


AnsiString DelHK(AnsiString strReplace)
// Remove the hotkey-sign (&) from a string:
{
TReplaceFlags rFlags;

rFlags.Contains(rfReplaceAll);

return StringReplace(strReplace, "&", "", rFlags);
}

AnsiString CheckNick(AnsiString newNick)
{
if (newNick.AnsiCompareIC("Server") == 0) return NULL;

if (newNick.Length() == 0) return NULL;

return newNick;
}

double Decimals(const double intDouble, const unsigned short int DecAmount)
{
// Keep as many decimals, as specified with parameter DecAmount:
AnsiString strTmp;
AnsiString strNew;
AnsiString strCur;
unsigned short int i;
unsigned short int l;

strTmp = FloatToStr(intDouble);
strNew = "";

for (i = 1; i <= strTmp.Length(); ++i)
         {
      strCur = strTmp.SubString(i, 1); 
      if (strCur == ',')
                        {
                        // Decimals found, start counting them until DecAmount:
                        strNew = strNew + ',';
                        for (l = 1; l <= DecAmount; ++l)
                                 {
                                 if (strNew.Length() < l)
                                                        {
                                                        return StrToFloat(strNew);
                                                        }
                                 strCur = strTmp.SubString(i + 1, 1);
                                 strNew += strCur;
                                 }

                        // ...and return the result:
                        return StrToFloat(strNew);
                        }
                        else
                        {
                        strNew += strCur;
                        }
         }


// No decimals found, return the result:
return (StrToFloat(strNew));
}



AnsiString TotalSize(const double Bytes)
// Convert size in bytes to KB and MB.
// Convert the result to 2 decimals at max.
{
double KB;

if (Bytes < 1024) return FloatToStr(Bytes) + " " + "Bytes"; // Smaller than one KB
if (Bytes == 1024) return FloatToStr(1) + " " + "KB"; // One KB


KB = Bytes / 1024;
if (KB < 1024) return FloatToStr(Decimals(KB, 2)) + " " + "KB"; // Smaller than one MB

KB = KB / 1024;
return FloatToStr(Decimals(KB, 2)) + " " + "MB"; // We're talking about MB now


}

AnsiString SwitchToListFormat(const AnsiString sText)
{
// Switch data to list format:
AnsiString tmpStr;


tmpStr = sText;
tmpStr = FullReplace(tmpStr, "\n\r\n\r", "||");
tmpStr = FullReplace(tmpStr, "\n\r", "||");
tmpStr = FullReplace(tmpStr, "||", "\r\n");

return tmpStr;
}

AnsiString SwitchToSendFormat(const AnsiString sText)
{
// Switch list to sending format:
AnsiString tmpStr;


tmpStr = sText;
tmpStr = FullReplace(tmpStr, "\r\n\r\n", "||");
tmpStr = FullReplace(tmpStr, "\r\n", "||");
tmpStr = FullReplace(tmpStr, "||", "\n\r");


return tmpStr;
}

